@@ -1,6 +1,6 @@
 /*
     This file is part of KAddressbook.
-    Copyright (c) 2003 Tobias Koenig <tokoe@kde.org>
+    Copyright (c) 2003 - 2004 Tobias Koenig <tokoe@kde.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -26,7 +26,6 @@
 #include <qfile.h>
 #include <qlayout.h>
 #include <qregexp.h>
-#include <qstatusbar.h>
 #include <qvbox.h>
 
 #include <kabc/addresseelist.h>
@@ -34,23 +33,28 @@
 #include <kabc/resource.h>
 #include <kabc/stdaddressbook.h>
 #include <kabc/vcardconverter.h>
+#include <kabc/resourcefile.h>
 #include <kaboutdata.h>
 #include <kaccelmanager.h>
 #include <kapplication.h>
+#include <dcopclient.h>
 #include <kactionclasses.h>
 #include <kcmdlineargs.h>
 #include <kcmultidialog.h>
 #include <kdebug.h>
 #include <kdeversion.h>
+#include <kimproxy.h>
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kprinter.h>
 #include <kprotocolinfo.h>
 #include <kresources/selectdialog.h>
 #include <kstandarddirs.h>
+#include <kstatusbar.h>
 #include <kstdguiitem.h>
 #include <ktempfile.h>
 #include <kxmlguiclient.h>
+#include <ktoolbar.h>
 #include <libkdepim/addresseeview.h>
 #include <libkdepim/categoryeditdialog.h>
 #include <libkdepim/categoryselectdialog.h>
@@ -66,7 +70,9 @@
 #include "kaddressbookservice.h"
 #include "kaddressbookiface.h"
 #include "ldapsearchdialog.h"
+#include "locationmap.h"
 #include "printing/printingwizard.h"
+#include "searchmanager.h"
 #include "undocmds.h"
 #include "viewmanager.h"
 #include "xxportmanager.h"
@@ -74,7 +80,7 @@
 #include "kabcore.h"
 
 KABCore::KABCore( KXMLGUIClient *client, bool readWrite, QWidget *parent,
-                  const char *name )
+                  const QString &file, const char *name )
   : KAB::Core( client, parent, name ), mStatusBar( 0 ), mViewManager( 0 ),
     mExtensionManager( 0 ), mCategorySelectDialog( 0 ), mCategoryEditDialog( 0 ),
     mLdapSearchDialog( 0 ), mReadWrite( readWrite ), mModified( false )
@@ -83,7 +89,16 @@ KABCore::KABCore( KXMLGUIClient *client, bool readWrite, QWidget *parent,
 
   mIsPart = !parent->isA( "KAddressBookMain" );
 
-  mAddressBook = KABC::StdAddressBook::self( true );
+  if ( file.isEmpty() ) {
+    mAddressBook = KABC::StdAddressBook::self( true );
+  } else {
+    kdDebug(5720) << "KABCore(): document '" << file << "'" << endl;
+    mAddressBook = new KABC::AddressBook;
+    mAddressBook->addResource( new KABC::ResourceFile( file ) );
+    if ( !mAddressBook->load() ) {
+      KMessageBox::error( parent, i18n("Unable to load '%1'.").arg( file ) );
+    }
+  }
   mAddressBook->setErrorHandler( new KABC::GuiErrorHandler( mWidget ) );
 
   mAddressBook->addCustomField( i18n( "Department" ), KABC::Field::Organization,
@@ -103,6 +118,8 @@ KABCore::KABCore( KXMLGUIClient *client, bool readWrite, QWidget *parent,
   mAddressBook->addCustomField( i18n( "Anniversary" ), KABC::Field::Personal,
                                 "X-Anniversary", "KADDRESSBOOK" );
 
+  mSearchManager = new KAB::SearchManager( mAddressBook, parent );
+
   initGUI();
 
   connect( mAddressBook, SIGNAL( addressBookChanged( AddressBook* ) ),
@@ -126,19 +143,25 @@ KABCore::KABCore( KXMLGUIClient *client, bool readWrite, QWidget *parent,
   connect( mXXPortManager, SIGNAL( modified() ),
            SLOT( setModified() ) );
 
-  connect( mJumpButtonBar, SIGNAL( jumpToLetter( const QString& ) ),
-           SLOT( incrementalJumpButtonSearch( const QString& ) ) );
+  connect( mJumpButtonBar, SIGNAL( jumpToLetter( const QStringList& ) ),
+           SLOT( incrementalJumpButtonSearch( const QStringList& ) ) );
   connect( mViewManager, SIGNAL( sortFieldChanged() ),
            mJumpButtonBar, SLOT( updateButtons() ) );
+  connect( mIncSearchWidget, SIGNAL( doReset() ),
+           mJumpButtonBar, SLOT( reset() ) );
 
   connect( mDetails, SIGNAL( highlightedMessage( const QString& ) ),
            SLOT( detailsHighlighted( const QString& ) ) );
 
   mAddressBookService = new KAddressBookService( this );
 
+  mSearchManager->reload();
+
   setModified( false );
 
   KAcceleratorManager::manage( mWidget );
+
+  mKIMProxy = ::KIMProxy::instance( kapp->dcopClient() );
 }
 
 KABCore::~KABCore()
@@ -146,8 +169,11 @@ KABCore::~KABCore()
   saveSettings();
   KABPrefs::instance()->writeConfig();
 
+  mAddressBook->disconnect();
+
   mAddressBook = 0;
   KABC::StdAddressBook::close();
+  mKIMProxy = 0;
 }
 
 void KABCore::restoreSettings()
@@ -160,24 +186,25 @@ void KABCore::restoreSettings()
   mActionDetails->setChecked( state );
   setDetailsVisible( state );
 
+  mViewManager->restoreSettings();
+  mExtensionManager->restoreSettings();
+
+  mIncSearchWidget->setCurrentItem( KABPrefs::instance()->mCurrentIncSearchField );
+
   QValueList<int> splitterSize = KABPrefs::instance()->mExtensionsSplitter;
   if ( splitterSize.count() == 0 ) {
-    splitterSize.append( mWidget->width() / 2 );
-    splitterSize.append( mWidget->width() / 2 );
+    splitterSize.append( mDetailsSplitter->height() / 2 );
+    splitterSize.append( mDetailsSplitter->height() / 2 );
   }
   mExtensionBarSplitter->setSizes( splitterSize );
 
   splitterSize = KABPrefs::instance()->mDetailsSplitter;
   if ( splitterSize.count() == 0 ) {
-    splitterSize.append( mWidget->height() / 2 );
-    splitterSize.append( mWidget->height() / 2 );
+    splitterSize.append( 360 );
+    splitterSize.append( 260 );
   }
   mDetailsSplitter->setSizes( splitterSize );
 
-  mViewManager->restoreSettings();
-  mExtensionManager->restoreSettings();
-
-  mIncSearchWidget->setCurrentItem( KABPrefs::instance()->mCurrentIncSearchField );
 }
 
 void KABCore::saveSettings()
@@ -209,11 +236,6 @@ KActionCollection *KABCore::actionCollection() const
   return guiClient()->actionCollection();
 }
 
-KABC::Field *KABCore::currentSearchField() const
-{
-  return mIncSearchWidget->currentField();
-}
-
 KABC::Field *KABCore::currentSortField() const
 {
   return mViewManager->currentSortField();
@@ -252,9 +274,9 @@ QWidget *KABCore::widget() const
 KAboutData *KABCore::createAboutData()
 {
   KAboutData *about = new KAboutData( "kaddressbook", I18N_NOOP( "KAddressBook" ),
-                                      "3.2.1", I18N_NOOP( "The KDE Address Book" ),
+                                      "3.3.2", I18N_NOOP( "The KDE Address Book" ),
                                       KAboutData::License_GPL_V2,
-                                      I18N_NOOP( "(c) 1997-2003, The KDE PIM Team" ) );
+                                      I18N_NOOP( "(c) 1997-2004, The KDE PIM Team" ) );
   about->addAuthor( "Tobias Koenig", I18N_NOOP( "Current maintainer" ), "tokoe@kde.org" );
   about->addAuthor( "Don Sanders", I18N_NOOP( "Original author" ) );
   about->addAuthor( "Cornelius Schumacher",
@@ -272,12 +294,12 @@ KAboutData *KABCore::createAboutData()
   return about;
 }
 
-void KABCore::setStatusBar( QStatusBar *statusBar )
+void KABCore::setStatusBar( KStatusBar *statusBar )
 {
   mStatusBar = statusBar;
 }
 
-QStatusBar *KABCore::statusBar() const
+KStatusBar *KABCore::statusBar() const
 {
   return mStatusBar;
 }
@@ -288,13 +310,6 @@ void KABCore::setContactSelected( const QString &uid )
   if ( !mDetails->isHidden() )
     mDetails->setAddressee( addr );
 
-  if ( !addr.isEmpty() ) {
-    emit contactSelected( addr.formattedName() );
-    KABC::Picture pic = addr.photo();
-    if ( pic.isIntern() )
-      emit contactSelected( pic.data() );
-  }
-
   mExtensionManager->setSelectionChanged();
 
   // update the actions
@@ -310,8 +325,10 @@ void KABCore::setContactSelected( const QString &uid )
   mActionEditAddressee->setEnabled( selected );
   mActionMail->setEnabled( selected );
   mActionMailVCard->setEnabled( selected );
+  mActionChat->setEnabled( selected && mKIMProxy && mKIMProxy->initialize() );
   mActionWhoAmI->setEnabled( selected );
   mActionCategories->setEnabled( selected );
+  mActionMerge->setEnabled( selected );
 }
 
 void KABCore::sendMail()
@@ -333,7 +350,8 @@ void KABCore::mailVCard()
 
 void KABCore::mailVCard( const QStringList& uids )
 {
-  QStringList urls;
+  //QStringList urls;
+  KURL::List urls;
 
   // Create a temp dir, so that we can put the files in it with proper names
   KTempFile tempDir;
@@ -369,15 +387,23 @@ void KABCore::mailVCard( const QStringList& uids )
 
       outFile.close();
 
-      urls.append( fileName );
+      KURL url( fileName );
+      url.setFileEncoding( "UTF-8" );
+      urls.append( url );
     }
   }
-
   kapp->invokeMailer( QString::null, QString::null, QString::null,
                       QString::null,  // subject
                       QString::null,  // body
                       QString::null,
-                      urls );  // attachments
+                      urls.toStringList() );  // attachments
+}
+
+void KABCore::startChat()
+{
+  QStringList uids = mViewManager->selectedUids();
+  if ( !uids.isEmpty() )
+    mKIMProxy->chatWithContact( uids.first() );
 }
 
 void KABCore::browse( const QString& url )
@@ -400,6 +426,18 @@ void KABCore::deleteContacts()
 void KABCore::deleteContacts( const QStringList &uids )
 {
   if ( uids.count() > 0 ) {
+    QStringList names;
+    QStringList::ConstIterator it = uids.begin();
+    while ( it != uids.end() ) {
+      KABC::Addressee addr = mAddressBook->findByUid( *it );
+      names.append( addr.realName().isEmpty() ? addr.preferredEmail() : addr.realName() );
+      ++it;
+    }
+
+    if ( KMessageBox::warningContinueCancelList( mWidget, i18n( "Do you really want to delete these contacts?" ),
+                                         names, "", KGuiItem( i18n("&Delete"), "editdelete") ) == KMessageBox::Cancel )
+      return;
+
     PwDeleteCommand *command = new PwDeleteCommand( mAddressBook, uids );
     UndoStack::instance()->push( command );
     RedoStack::instance()->clear();
@@ -458,6 +496,26 @@ void KABCore::pasteContacts( KABC::Addressee::List &list )
   setModified( true );
 }
 
+void KABCore::mergeContacts()
+{
+  KABC::Addressee::List list = mViewManager->selectedAddressees();
+  if ( list.count() < 2 )
+    return;
+
+  KABC::Addressee addr = mergeContacts( list );
+
+  KABC::Addressee::List::Iterator it = list.begin();
+  ++it;
+  while ( it != list.end() ) {
+    mAddressBook->removeAddressee( *it );
+    ++it;
+  }
+
+  mAddressBook->insertAddressee( addr );
+
+  mSearchManager->reload();
+}
+
 void KABCore::setWhoAmI()
 {
   KABC::Addressee::List addrList = mViewManager->selectedAddressees();
@@ -472,75 +530,16 @@ void KABCore::setWhoAmI()
     static_cast<KABC::StdAddressBook*>( KABC::StdAddressBook::self( true ) )->setWhoAmI( addrList[ 0 ] );
 }
 
-void KABCore::setSearchFields( const KABC::Field::List &fields )
-{
-  mIncSearchWidget->setFields( fields );
-}
-
 void KABCore::incrementalTextSearch( const QString& text )
 {
-  incrementalSearch( text, true );
+  setContactSelected( QString::null );
+  mSearchManager->search( text, mIncSearchWidget->currentField() );
 }
 
-void KABCore::incrementalJumpButtonSearch( const QString& text )
+void KABCore::incrementalJumpButtonSearch( const QStringList& characters )
 {
-  incrementalSearch( text, false );
-}
-
-void KABCore::incrementalSearch( const QString& text, bool search )
-{
-  mViewManager->setSelected( QString::null, false );
-
-  if ( !text.isEmpty() ) {
-    KABC::Field *field = ( search ? mIncSearchWidget->currentField() :
-                                    mViewManager->currentSortField() );
-
-#if KDE_VERSION >= 319
-    KABC::AddresseeList list( mAddressBook->allAddressees() );
-    if (  field ) {
-      list.sortByField( field );
-      KABC::AddresseeList::Iterator it;
-      for ( it = list.begin(); it != list.end(); ++it ) {
-        if ( (search && field->value( *it ).find( text, 0, false ) != -1) ||
-             (!search && field->value( *it).startsWith( text, false)) ) {
-          mViewManager->setSelected( (*it).uid(), true );
-          return;
-        }
-      }
-    } else {
-      KABC::AddresseeList::Iterator it;
-      for ( it = list.begin(); it != list.end(); ++it ) {
-        KABC::Field::List fieldList = KABC::Field::allFields();
-        KABC::Field::List::ConstIterator fieldIt;
-        for ( fieldIt = fieldList.begin(); fieldIt != fieldList.end(); ++fieldIt ) {
-          if ( (*fieldIt)->value( *it ).find( text, 0, false ) != -1 ) {
-            mViewManager->setSelected( (*it).uid(), true );
-            return;
-          }
-        }
-      }
-    }
-#else
-    KABC::AddressBook::Iterator it;
-    for ( it = mAddressBook->begin(); it != mAddressBook->end(); ++it ) {
-      if ( field ) {
-        if ( field->value( *it ).startsWith( text, false ) ) {
-          mViewManager->setSelected( (*it).uid(), true );
-          return;
-        }
-      } else {
-        KABC::Field::List fieldList = mIncSearchWidget->fields();
-        KABC::Field::List::ConstIterator fieldIt;
-        for ( fieldIt = fieldList.begin(); fieldIt != fieldList.end(); ++fieldIt ) {
-          if ( (*fieldIt)->value( *it ).startsWith( text, false ) ) {
-            mViewManager->setSelected( (*it).uid(), true );
-            return;
-          }
-        }
-      }
-    }
-#endif
-  }
+  setContactSelected( QString::null );
+  mSearchManager->setJumpButtonFilter( characters, mViewManager->currentSortField() );
 }
 
 void KABCore::setModified()
@@ -553,7 +552,7 @@ void KABCore::setModified( bool modified )
   mModified = modified;
   mActionSave->setEnabled( mModified );
 
-  mViewManager->refreshView();
+  mSearchManager->reload();
 }
 
 bool KABCore::modified() const
@@ -697,15 +696,34 @@ void KABCore::editContact( const QString &uid )
 void KABCore::save()
 {
   KABC::StdAddressBook *b = dynamic_cast<KABC::StdAddressBook*>( mAddressBook );
-  if ( !b || !b->save() ) {
-    QString text = i18n( "There was an error while attempting to save the "
-    			"address book. Please check that some other application is "
-    			"not using it. " );
+  if ( b ) {
+    if ( !b->save() ) {
+      QString text = i18n( "There was an error while attempting to save the "
+    	                   "address book. Please check that no other application "
+                           "is using it." );
 
-    KMessageBox::error( mWidget, text, i18n( "Unable to Save" ) );
+      KMessageBox::error( mWidget, text, i18n( "Unable to Save" ) );
+    } else {
+      setModified( false );
+    }
+  } else {
+    // FIXME: Handle locking properly, i.e. get the ticket before doing the
+    // first change to the addressbook and don't give up the ticket in case of a
+    // save error without asking the user.
+    KABC::Ticket *ticket = mAddressBook->requestSaveTicket();
+    if ( ticket ) {
+      if ( !mAddressBook->save( ticket ) ) {
+        KMessageBox::error( mWidget, i18n("Error saving address book.") );
+        mAddressBook->releaseSaveTicket( ticket );
+      } else {
+        setModified( false );
+      }
+    } else {
+      KMessageBox::error( mWidget,
+                          i18n("Unable to get access for saving the address "
+                               "book.") );
+    }
   }
-
-  setModified( false );
 }
 
 void KABCore::undo()
@@ -783,7 +801,7 @@ QString KABCore::getNameByPhone( const QString &phone )
           ++phoneIter) {
       // Get rid of separator chars so just the numbers are compared.
       if ( (*phoneIter).number().replace( r, "" ) == localPhone.replace( r, "" ) ) {
-        ownerName = (*iter).formattedName();
+        ownerName = (*iter).realName();
         found = true;
       }
     }
@@ -796,16 +814,16 @@ void KABCore::openLDAPDialog()
 {
   if ( !KProtocolInfo::isKnownProtocol( KURL( "ldap://localhost" ) ) ) {
     KMessageBox::error( mWidget, i18n( "Your KDE installation is missing LDAP "
-                                       "support, please ask your administrator or distributor for more information!" ),
+                                       "support, please ask your administrator or distributor for more information." ),
                         i18n( "No LDAP IO slave available" ) );
     return;
   }
 
   if ( !mLdapSearchDialog ) {
-    mLdapSearchDialog = new LDAPSearchDialog( mAddressBook, mWidget );
-    connect( mLdapSearchDialog, SIGNAL( addresseesAdded() ), mViewManager,
-            SLOT( refreshView() ) );
-    connect( mLdapSearchDialog, SIGNAL( addresseesAdded() ), this,
+    mLdapSearchDialog = new LDAPSearchDialog( mAddressBook, this, mWidget );
+    connect( mLdapSearchDialog, SIGNAL( addresseesAdded() ), mSearchManager,
+            SLOT( addressBookChanged() ) );
+    connect( mLdapSearchDialog, SIGNAL( addresseesAdded() ),
             SLOT( setModified() ) );
   } else
     mLdapSearchDialog->restoreSettings();
@@ -825,6 +843,7 @@ void KABCore::configure()
 
   dlg.addModule( "kabconfig.desktop" );
   dlg.addModule( "kabldapconfig.desktop" );
+  dlg.addModule( "kabcustomfields.desktop" );
 
   dlg.exec();
 }
@@ -843,18 +862,40 @@ void KABCore::print()
 
 void KABCore::detailsHighlighted( const QString &msg )
 {
-  if ( statusBar() )
-    statusBar()->message( msg );
+  if ( mStatusBar )
+    mStatusBar->changeItem( msg, 1 );
+}
+
+void KABCore::showContactsAddress( const QString &addrUid )
+{
+  QStringList uidList = mViewManager->selectedUids();
+  if ( uidList.isEmpty() )
+    return;
+
+  KABC::Addressee addr = mAddressBook->findByUid( uidList.first() );
+  if ( addr.isEmpty() )
+    return;
+
+  KABC::Address::List list = addr.addresses();
+  KABC::Address::List::Iterator it;
+  for ( it = list.begin(); it != list.end(); ++it )
+    if ( (*it).id() == addrUid ) {
+      LocationMap::instance()->showAddress( *it );
+      break;
+    }
 }
 
 void KABCore::configurationChanged()
 {
   mExtensionManager->reconfigure();
+  mSearchManager->reconfigure();
+  mViewManager->refreshView();
 }
 
 void KABCore::addressBookChanged()
 {
-  mViewManager->refreshView();
+  mJumpButtonBar->updateButtons();
+  mSearchManager->reload();
 }
 
 AddresseeEditorDialog *KABCore::createAddresseeEditorDialog( QWidget *parent,
@@ -876,28 +917,33 @@ void KABCore::slotEditorDestroyed( const QString &uid )
 
   KABC::Addressee addr = mAddressBook->findByUid( uid );
 
+  QApplication::setOverrideCursor( Qt::waitCursor );
+
   KABLock::self( mAddressBook )->unlock( addr.resource() );
+
+  QApplication::restoreOverrideCursor();
 }
 
 void KABCore::initGUI()
 {
-  QVBoxLayout *topLayout = new QVBoxLayout( mWidget, KDialog::marginHint(),
-                                            KDialog::spacingHint() );
-  QHBoxLayout *hbox = new QHBoxLayout( topLayout, KDialog::spacingHint() );
-
-  mIncSearchWidget = new IncSearchWidget( mWidget );
+  QVBoxLayout *topLayout = new QVBoxLayout( mWidget, 0, 0 );
+  KToolBar* searchTB = new KToolBar( mWidget, "search toolbar");
+  searchTB->boxLayout()->setSpacing( KDialog::spacingHint() );
+  mIncSearchWidget = new IncSearchWidget( searchTB, "kde toolbar widget");
+  searchTB->setStretchableWidget( mIncSearchWidget );
   connect( mIncSearchWidget, SIGNAL( doSearch( const QString& ) ),
            SLOT( incrementalTextSearch( const QString& ) ) );
 
-  mFilterSelectionWidget = new FilterSelectionWidget( mWidget );
-  hbox->addWidget( mIncSearchWidget );
-  hbox->addWidget( mFilterSelectionWidget );
+  mFilterSelectionWidget = new FilterSelectionWidget( searchTB , "kde toolbar widget" );
 
-  hbox = new QHBoxLayout( topLayout, KDialog::spacingHint() );
+  QHBoxLayout *hbox = new QHBoxLayout( mWidget, 0, 0 );
 
   mDetailsSplitter = new QSplitter( mWidget );
   hbox->addWidget( mDetailsSplitter );
 
+  topLayout->addWidget( searchTB );
+  topLayout->addWidget( mDetailsSplitter );
+
   mExtensionBarSplitter = new QSplitter( mDetailsSplitter );
   mExtensionBarSplitter->setOrientation( Qt::Vertical );
 
@@ -907,12 +953,13 @@ void KABCore::initGUI()
   mDetailsPage = new QWidget( detailsWidget );
   detailsLayout->addWidget( mDetailsPage );
 
-  QHBoxLayout *detailsPageLayout = new QHBoxLayout( mDetailsPage,
-                                                    KDialog::marginHint(),
-                                                KDialog::spacingHint() );
+  QHBoxLayout *detailsPageLayout = new QHBoxLayout( mDetailsPage, 0, 0 );
   mDetails = new KPIM::AddresseeView( mDetailsPage );
   detailsPageLayout->addWidget( mDetails );
 
+  connect( mDetails, SIGNAL( addressClicked( const QString&) ),
+           this, SLOT( showContactsAddress( const QString& ) ) );
+
   mViewManager = new ViewManager( this, mExtensionBarSplitter );
   mViewManager->setFilterSelectionWidget( mFilterSelectionWidget );
 
@@ -940,28 +987,38 @@ void KABCore::initActions()
   KAction *action;
 
   // file menu
-  mActionMail = KStdAction::mail( this, SLOT( sendMail() ), actionCollection() );
+  mActionMail = new KAction( i18n( "&Send Email to Contact..." ), "mail_send", 0,
+                             this, SLOT( sendMail() ), actionCollection(), "file_mail" );
   action = KStdAction::print( this, SLOT( print() ), actionCollection() );
   mActionMail->setWhatsThis( i18n( "Send a mail to all selected contacts." ) );
   action->setWhatsThis( i18n( "Print a special number of contacts." ) );
 
-  mActionSave = new KAction(  KStdGuiItem::save().text(), "filesave", CTRL+Key_S, this,
+  mActionSave = KStdAction::save( this,
                              SLOT( save() ), actionCollection(), "file_sync" );
   mActionSave->setWhatsThis( i18n( "Save all changes of the address book to the storage backend." ) );
 
-  action = new KAction( i18n( "&New Contact..." ), "filenew", CTRL+Key_N, this,
+  action = new KAction( i18n( "&New Contact..." ), "identity", CTRL+Key_N, this,
                SLOT( newContact() ), actionCollection(), "file_new_contact" );
-  action->setWhatsThis( i18n( "Create a new contact<p>You will be presented with a dialog where you can add all data of a person, including addresses and phonenumbers." ) );
+  action->setWhatsThis( i18n( "Create a new contact<p>You will be presented with a dialog where you can add all data about a person, including addresses and phone numbers." ) );
 
   mActionMailVCard = new KAction( i18n("Send &Contact..."), "mail_post_to", 0,
                                   this, SLOT( mailVCard() ),
                                   actionCollection(), "file_mail_vcard" );
   mActionMailVCard->setWhatsThis( i18n( "Send a mail with the selected contact as attachment." ) );
 
+  mActionChat = new KAction( i18n("Chat &With..."), 0,
+                                  this, SLOT( startChat() ),
+                                  actionCollection(), "file_chat" );
+  mActionChat->setWhatsThis( i18n( "Start a chat with the selected contact." ) );
+
   mActionEditAddressee = new KAction( i18n( "&Edit Contact..." ), "edit", 0,
                                       this, SLOT( editContact() ),
                                       actionCollection(), "file_properties" );
-  mActionEditAddressee->setWhatsThis( i18n( "Edit a contact<p>You will be presented with a dialog where you can change all data of a person, including addresses and phonenumbers." ) );
+  mActionEditAddressee->setWhatsThis( i18n( "Edit a contact<p>You will be presented with a dialog where you can change all data about a person, including addresses and phone numbers." ) );
+
+  mActionMerge = new KAction( i18n( "&Merge Contacts" ), "", 0,
+                              this, SLOT( mergeContacts() ),
+                              actionCollection(), "edit_merge" );
 
   // edit menu
   mActionCopy = KStdAction::copy( this, SLOT( copyContacts() ), actionCollection() );
@@ -996,6 +1053,11 @@ void KABCore::initActions()
   mActionDetails->setWhatsThis( i18n( "Toggle whether the details page shall be visible." ) );
   connect( mActionDetails, SIGNAL( toggled( bool ) ), SLOT( setDetailsVisible( bool ) ) );
 
+#if KDE_IS_VERSION(3,2,90)
+  mActionJumpBar->setCheckedState( i18n( "Hide Jump Bar") );
+  mActionDetails->setCheckedState( i18n( "Hide Details") );
+#endif
+
   if ( mIsPart )
     action = new KAction( i18n( "&Configure KAddressBook..." ), "configure", 0,
                           this, SLOT( configure() ), actionCollection(),
@@ -1010,10 +1072,10 @@ void KABCore::initActions()
                         this, SLOT( openLDAPDialog() ), actionCollection(), "ldap_lookup" );
   action->setWhatsThis( i18n( "Search for contacts on a LDAP server<p>You will be presented with a dialog, where you can search for contacts and select the ones you want to add to your local address book." ) );
 
-  mActionWhoAmI = new KAction( i18n( "Set 'Who Am I'" ), "personal", 0, this,
+  mActionWhoAmI = new KAction( i18n( "Set as Personal Contact Data" ), "personal", 0, this,
                                SLOT( setWhoAmI() ), actionCollection(),
                                "edit_set_personal" );
-  mActionWhoAmI->setWhatsThis( i18n( "Set the personal contact<p>The data of this contact will be used in many other KDE applications, so you don't have to input your personal data several times." ) );
+  mActionWhoAmI->setWhatsThis( i18n( "Set the personal contact<p>The data of this contact will be used in many other KDE applications, so you do not have to input your personal data several times." ) );
 
   mActionCategories = new KAction( i18n( "Select Categories..." ), 0, this,
                                    SLOT( setCategories() ), actionCollection(),
@@ -1052,6 +1114,143 @@ void KABCore::updateActionMenu()
   mActionRedo->setEnabled( !redo->isEmpty() );
 }
 
+KABC::Addressee KABCore::mergeContacts( const KABC::Addressee::List &list )
+{
+  if ( list.count() == 0 ) //emtpy
+    return KABC::Addressee();
+  else if ( list.count() == 1 ) // nothing to merge
+    return list.first();
+
+  KABC::Addressee masterAddressee = list.first();
+
+  KABC::Addressee::List::ConstIterator contactIt = list.begin();
+  for ( ++contactIt; contactIt != list.end(); ++contactIt ) {
+    // ADR + LABEL
+    KABC::Address::List addresses = (*contactIt).addresses();
+    KABC::Address::List masterAddresses = masterAddressee.addresses();
+    KABC::Address::List::Iterator addrIt ;
+    for ( addrIt = addresses.begin(); addrIt != addresses.end(); ++addrIt ) {
+      if ( !masterAddresses.contains( *addrIt ) )
+        masterAddressee.insertAddress( *addrIt );
+    }
+
+    if ( masterAddressee.birthday().isNull() && !(*contactIt).birthday().isNull() )
+      masterAddressee.setBirthday( (*contactIt).birthday() );
+
+
+    // CATEGORIES
+    QStringList::Iterator it;
+    QStringList categories = (*contactIt).categories();
+    QStringList masterCategories = masterAddressee.categories();
+    QStringList newCategories( masterCategories );
+    for ( it = categories.begin(); it != categories.end(); ++it )
+      if ( !masterCategories.contains( *it ) )
+        newCategories.append( *it );
+    masterAddressee.setCategories( newCategories );
+
+    // CLASS
+    if ( !masterAddressee.secrecy().isValid() && (*contactIt).secrecy().isValid() )
+      masterAddressee.setSecrecy( (*contactIt).secrecy() );
+
+    // EMAIL
+    QStringList emails = (*contactIt).emails();
+    QStringList masterEmails = masterAddressee.emails();
+    for ( it = emails.begin(); it != emails.end(); ++it )
+      if ( !masterEmails.contains( *it ) )
+        masterAddressee.insertEmail( *it, false );
+
+    // FN
+    if ( masterAddressee.formattedName().isEmpty() && !(*contactIt).formattedName().isEmpty() )
+      masterAddressee.setFormattedName( (*contactIt).formattedName() );
+
+    // GEO
+    if ( !masterAddressee.geo().isValid() && (*contactIt).geo().isValid() )
+      masterAddressee.setGeo( (*contactIt).geo() );
+
+/*
+  // KEY
+  // LOGO
+*/
+
+    // MAILER
+    if ( masterAddressee.mailer().isEmpty() && !(*contactIt).mailer().isEmpty() )
+      masterAddressee.setMailer( (*contactIt).mailer() );
+
+    // N
+    if ( masterAddressee.assembledName().isEmpty() && !(*contactIt).assembledName().isEmpty() )
+      masterAddressee.setNameFromString( (*contactIt).assembledName() );
+
+    // NICKNAME
+    if ( masterAddressee.nickName().isEmpty() && !(*contactIt).nickName().isEmpty() )
+      masterAddressee.setNickName( (*contactIt).nickName() );
+
+    // NOTE
+    if ( masterAddressee.note().isEmpty() && !(*contactIt).note().isEmpty() )
+      masterAddressee.setNote( (*contactIt).note() );
+
+    // ORG
+    if ( masterAddressee.organization().isEmpty() && !(*contactIt).organization().isEmpty() )
+      masterAddressee.setOrganization( (*contactIt).organization() );
+
+/*
+  // PHOTO
+*/
+
+    // PROID
+    if ( masterAddressee.productId().isEmpty() && !(*contactIt).productId().isEmpty() )
+      masterAddressee.setProductId( (*contactIt).productId() );
+
+    // REV
+    if ( masterAddressee.revision().isNull() && !(*contactIt).revision().isNull() )
+      masterAddressee.setRevision( (*contactIt).revision() );
+
+    // ROLE
+    if ( masterAddressee.role().isEmpty() && !(*contactIt).role().isEmpty() )
+      masterAddressee.setRole( (*contactIt).role() );
+
+    // SORT-STRING
+    if ( masterAddressee.sortString().isEmpty() && !(*contactIt).sortString().isEmpty() )
+      masterAddressee.setSortString( (*contactIt).sortString() );
+
+/*
+  // SOUND
+*/
+
+    // TEL
+    KABC::PhoneNumber::List phones = (*contactIt).phoneNumbers();
+    KABC::PhoneNumber::List masterPhones = masterAddressee.phoneNumbers();
+    KABC::PhoneNumber::List::ConstIterator phoneIt;
+    for ( phoneIt = phones.begin(); phoneIt != phones.end(); ++phoneIt )
+      if ( !masterPhones.contains( *it ) )
+        masterAddressee.insertPhoneNumber( *it );
+
+    // TITLE
+    if ( masterAddressee.title().isEmpty() && !(*contactIt).title().isEmpty() )
+      masterAddressee.setTitle( (*contactIt).title() );
+
+    // TZ
+    if ( !masterAddressee.timeZone().isValid() && (*contactIt).timeZone().isValid() )
+      masterAddressee.setTimeZone( (*contactIt).timeZone() );
+
+    // UID // ignore UID
+
+    // URL
+    if ( masterAddressee.url().isEmpty() && !(*contactIt).url().isEmpty() )
+      masterAddressee.setUrl( (*contactIt).url() );
+
+    // X-
+    QStringList customs = (*contactIt).customs();
+    QStringList masterCustoms = masterAddressee.customs();
+    QStringList newCustoms( masterCustoms );
+    for ( it = customs.begin(); it != customs.end(); ++it )
+      if ( !masterCustoms.contains( *it ) )
+        newCustoms.append( *it );
+    masterAddressee.setCustoms( newCustoms );
+  }
+
+  return masterAddressee;
+}
+
 void KABCore::setCategories()
 {
   // Show the category dialog
