@@ -1,4 +1,26 @@
-// $Id: mobilegui.cpp,v 1.2 2001/05/01 23:21:21 cschumac Exp $
+/*
+    This file is part of Kandy.
+
+    Copyright (c) 2001 Cornelius Schumacher <schumacher@kde.org>
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+    As a special exception, permission is given to link this program
+    with any edition of Qt, and distribute the resulting executable,
+    without including the source code for Qt in the source distribution.
+*/
 
 #include <qlabel.h>
 #include <qlistview.h>
@@ -8,10 +30,11 @@
 
 #include <kdebug.h>
 #include <kfiledialog.h>
-#include <kabapi.h>
 #include <kmessagebox.h>
 #include <klocale.h>
-#include <kapp.h>
+#include <kapplication.h>
+
+#include <kabc/stdaddressbook.h>
 
 #include "atcommand.h"
 #include "commandscheduler.h"
@@ -45,7 +68,7 @@ class SyncEntryKab : public SyncEntry {
     QString mPhone;
     QString mPhonetype;
 
-    KabKey mKey;
+    KABC::Addressee mAddressee;
 };
 
 class SyncEntryMobile : public SyncEntry {
@@ -88,9 +111,9 @@ class AddressSyncer {
       mCommonEntries.setAutoDelete(true);
     }
   
-    QList<SyncEntryKab> mKabEntries;
-    QList<SyncEntryMobile> mMobileEntries;
-    QList<SyncEntryCommon> mCommonEntries; 
+    QPtrList<SyncEntryKab> mKabEntries;
+    QPtrList<SyncEntryMobile> mMobileEntries;
+    QPtrList<SyncEntryCommon> mCommonEntries; 
 };
 
 
@@ -143,8 +166,8 @@ class PhoneBookItem : public QCheckListItem {
  */
 MobileGui::MobileGui(CommandScheduler *scheduler,QWidget* parent,
                      const char* name,WFlags fl)
-    : MobileGui_base(parent,name,fl),
-      DCOPObject("KandyIface")
+    : DCOPObject("KandyIface"),
+      MobileGui_base(parent,name,fl)
 {
   mSyncing = false;
   mScheduler = scheduler;
@@ -156,6 +179,7 @@ MobileGui::MobileGui(CommandScheduler *scheduler,QWidget* parent,
 
 MobileGui::~MobileGui()
 {
+  delete mSyncer;
 }
 
 void MobileGui::exit()
@@ -180,12 +204,12 @@ void MobileGui::readPhonebook()
 
 void MobileGui::writePhonebook()
 {
-  kdDebug() << "MobileGui::writePhonebook" << endl;
+  kdDebug(5960) << "MobileGui::writePhonebook" << endl;
 
   for(uint i=0;i<mSyncer->mMobileEntries.count();++i) {
     SyncEntryMobile *entry = mSyncer->mMobileEntries.at(i);
 
-//    kdDebug() << "Writing " << entry->mIndex << " " << entry->mName
+//    kdDebug(5960) << "Writing " << entry->mIndex << " " << entry->mName
 //              << " " << entry->mPhone << endl;
 
     QString id = "+cpbw=" + entry->mIndex;
@@ -196,8 +220,8 @@ void MobileGui::writePhonebook()
     cmd->addParameter(new ATParameter(entry->mType));
     cmd->addParameter(new ATParameter(quote(entry->mName)));
 
-    kdDebug() << "  " << cmd->cmd() << endl;
-    kdDebug() << "  id: " << cmd->id() << endl;
+    kdDebug(5960) << "  " << cmd->cmd() << endl;
+    kdDebug(5960) << "  id: " << cmd->id() << endl;
     
     mScheduler->execute(cmd);
   }
@@ -205,71 +229,32 @@ void MobileGui::writePhonebook()
   emit statusMessage(i18n("Writing mobile phonebook..."));  
 }
 
-void MobileGui::readKab()
+void MobileGui::readKabc()
 {
-  kdDebug() << "MobileGui::readKab()" << endl;
+  kdDebug(5960) << "MobileGui::readKabc()" << endl;
 
   mSyncer->mKabEntries.clear();
 
-  KabAPI kab(0);
-  if (kab.init() != AddressBook::NoError) {
-    kdDebug() << "Error initing kab" << endl;
-    return;
-  }
+  KABC::AddressBook *addressBook = KABC::StdAddressBook::self();
 
-  KabKey key;
-  AddressBook::Entry entry;
-
-  // Search for entry that has same X-KANDY-Index
-  int num = kab.addressbook()->noOfEntries();
-  for (int i = 0; i < num; ++i) {
-    if (AddressBook::NoError != kab.addressbook()->getKey(i,key)) {
-      kdDebug() << "Error getting key for index " << i << " from kab." << endl;
-      continue;
-    }
-    if (AddressBook::NoError != kab.addressbook()->getEntry(key,entry))
-    {
-      kdDebug() << "Error getting entry for index " << i << " from kab." << endl;
-      continue;
-    }
-
-//    kdDebug() << "Entry: " << entry.firstname << " " << entry.lastname << endl;
-//    QStringList phones = entry.telephone;
-//    for(QStringList::ConstIterator it2 = phones.begin(); it2 != phones.end(); ++it2) {
-//      kdDebug() << "  " << (*it2) << endl;
-//    }
+  KABC::AddressBook::Iterator it;
+  for( it = addressBook->begin(); it != addressBook->end(); ++it ) {
     
-    QString index,type,name,phone;
-    QString phonetype = "1";
+    QString index = (*it).custom("KANDY","Index");
+    QString type = (*it).custom("KANDY","Type");
+    QString name = (*it).custom("KANDY","Name");
+    QString phonetype = (*it).custom("KANDY","Phonetype");
     
-    QStringList::ConstIterator cIt = entry.custom.begin();
-    while(cIt != entry.custom.end()) {
-      if ((*cIt).startsWith("X-KANDY-Index:"))
-        index = (*cIt).mid((*cIt).find(":")+1);
-      else if ((*cIt).startsWith("X-KANDY-Type:"))
-        type = (*cIt).mid((*cIt).find(":")+1);
-      else if ((*cIt).startsWith("X-KANDY-Name:"))
-        name = (*cIt).mid((*cIt).find(":")+1);
-      else if ((*cIt).startsWith("X-KANDY-Phonetype:"))
-        phonetype = (*cIt).mid((*cIt).find(":")+1);
-
-      ++cIt;
-    }
-
     // Get phonenumber according to phonetype. If no number of this type was
     // found use the first phone number as default
-    QString phoneDefault;
-    for(QStringList::ConstIterator pIt = entry.telephone.begin();
-        pIt != entry.telephone.end(); ++pIt) {
-      if ((*pIt) == phonetype) {
-        ++pIt;
-        phone = (*pIt);
-      } else {
-        ++pIt;
-      }
-      if (phoneDefault.isEmpty()) phoneDefault = (*pIt);
+    KABC::PhoneNumber phoneNumber;
+    if ( phonetype.isEmpty() ) {
+      KABC::PhoneNumber::List phoneNumbers = (*it).phoneNumbers();
+      phoneNumber = phoneNumbers.first();
+    } else {
+      phoneNumber = (*it).phoneNumber( phonetype.toInt() );
     }
-    if (phone.isEmpty()) phone = phoneDefault;
+    QString phone = phoneNumber.number();
 
     SyncEntryKab *kabEntry;
     if (!index.isEmpty()) {
@@ -278,13 +263,14 @@ void MobileGui::readKab()
     } else {
       // This entry has never been on the phone.
       index = "";
-      name = entry.fn;
+      name = (*it).realName();
       if (phone.left(1) == "+") type = "145";
       else type = "129";
       
       kabEntry = new SyncEntryKab(false,index,name,type,phone,phonetype);
     }
-    kabEntry->mKey = key;
+
+    kabEntry->mAddressee = (*it);
 
     mSyncer->mKabEntries.append(kabEntry);
   }
@@ -295,62 +281,49 @@ void MobileGui::readKab()
   emit transientStatusMessage(i18n("Read KDE address book."));
 }
 
-void MobileGui::writeKab()
+void MobileGui::writeKabc()
 {
-  kdDebug() << "MobileGui::writeKab()" << endl;
+  kdDebug(5960) << "MobileGui::writeKabc()" << endl;
 
-  KabAPI kab(0);
-  if (kab.init() != AddressBook::NoError) {
-    kdDebug() << "Error initing kab" << endl;
+  KABC::AddressBook *addressBook = KABC::StdAddressBook::self();
+
+  KABC::Ticket *ticket = addressBook->requestSaveTicket();
+
+  if ( !ticket ) {
+    kdDebug(5960) << "Error! No ticket to save." << endl;
     return;
   }
 
   for(uint i=0;i<mSyncer->mKabEntries.count();++i) {
     SyncEntryKab *kabEntry = mSyncer->mKabEntries.at(i);
 
-    AddressBook::Entry entry;
+    KABC::Addressee entry = kabEntry->mAddressee;
     
     QString name = kabEntry->mName;
     QString phonenumber = kabEntry->mPhone;
     QString index = kabEntry->mIndex;
     QString type = kabEntry->mType;
 
-    QString indexEntry = "X-KANDY-Index:" + index;
+    entry.insertCustom( "KANDY", "Index", index );
     
-    entry.fn = name;
+    entry.setFormattedName( name );
 
     // Try to identify type of phonenumber and write it to the corresponding
     // telephone entry    
+    int phoneType = 0;
     if (phonenumber.left(3) == "017" || phonenumber.left(6) == "+49017") {
-      entry.telephone << QString::number(1) << phonenumber;
-      entry.custom << "X-KANDY-Phonetype:1";
-    } else {
-      entry.telephone << QString::number(0) << phonenumber;
-      entry.custom << "X-KANDY-Phonetype:0";
+      phoneType = KABC::PhoneNumber::Cell;
     }
-    entry.custom << "X-KANDY-Name:" + name;
-    entry.custom << "X-KANDY-Type:" + type;
-    entry.custom << indexEntry;
+    entry.insertPhoneNumber( KABC::PhoneNumber( phonenumber, phoneType ) );
+    entry.insertCustom( "KANDY", "Phonetype", QString::number( phoneType ) );
+    
+    entry.insertCustom( "KANDY", "Name", name );
+    entry.insertCustom( "KANDY", "Type", type );
 
-    if (kabEntry->mKey.getKey().isEmpty()) {
-      AddressBook::ErrorCode error = kab.add(entry,kabEntry->mKey);
-      if (error != AddressBook::NoError) {
-        kdDebug() << "kab.add returned with error " << error << endl;
-      } else {
-        kdDebug() << "Added " << name << endl;
-      }
-    } else {
-      AddressBook::ErrorCode error = kab.addressbook()->change(kabEntry->mKey,
-                                                               entry);
-      if (error != AddressBook::NoError) {
-        kdDebug() << "kab.change returned with error " << error << endl;
-      } else {
-        kdDebug() << "Changed " << name << endl;
-      }
-    }
+    addressBook->insertAddressee( entry );
   }
   
-  kab.save(true);
+  addressBook->save( ticket );
   
   emit transientStatusMessage(i18n("Wrote KDE address book"));
 }
@@ -384,7 +357,7 @@ void MobileGui::processResult(ATCommand *command)
   if (command->id() == mSyncReadId) {
     mSyncReadId = "";
     mergePhonebooks();
-    writeKab();
+    writeKabc();
     writePhonebook();
     mSyncWriteId = mLastWriteId;
   }
@@ -397,18 +370,18 @@ void MobileGui::processResult(ATCommand *command)
 
 void MobileGui::fillPhonebook(ATCommand *cmd)
 {
-  kdDebug() << "MobileGui::fillPhonebook()" << endl;
+  kdDebug(5960) << "MobileGui::fillPhonebook()" << endl;
 
-//  kdDebug() << "--- " << cmd->resultString() << endl;
+//  kdDebug(5960) << "--- " << cmd->resultString() << endl;
 
   mSyncer->mMobileEntries.clear();
     
-  QList<QStringList> *list = cmd->resultFields();
+  QPtrList<QStringList> *list = cmd->resultFields();
   
   QStringList *fields = list->first();
   while(fields) {
     if (fields->count() != 4) {
-      kdDebug() << "Error! Unexpected number of address fields." << endl;
+      kdDebug(5960) << "Error! Unexpected number of address fields." << endl;
     } else {
       QString index = (*fields)[0];
       QString phone = (*fields)[1];
@@ -472,7 +445,7 @@ void MobileGui::savePhonebook()
 
 void MobileGui::mergePhonebooks()
 {
-  kdDebug() << "MobileGui::mergePhonebooks()" << endl;
+  kdDebug(5960) << "MobileGui::mergePhonebooks()" << endl;
 
   // Update selection state from GUI.
   PhoneBookItem *item = (PhoneBookItem *)mKabBook->firstChild();
@@ -488,7 +461,7 @@ void MobileGui::mergePhonebooks()
 
   mSyncer->mCommonEntries.clear();
 
-//  kdDebug() << " Insert kab list" << endl;
+//  kdDebug(5960) << " Insert kab list" << endl;
 
   // Put Kab list into common list
   for(uint i=0;i<mSyncer->mKabEntries.count();++i) {
@@ -497,12 +470,12 @@ void MobileGui::mergePhonebooks()
     }
   }
 
-//  kdDebug() << " Insert mobile list" << endl;
+//  kdDebug(5960) << " Insert mobile list" << endl;
 
   // Put mobile list into common list. Merge equivalent entries.
   for(uint i=0;i<mSyncer->mMobileEntries.count();++i) {
     SyncEntryMobile *mobileEntry = mSyncer->mMobileEntries.at(i);
-//    kdDebug() << "--- Inserting " << mobileEntry->mName << endl;
+//    kdDebug(5960) << "--- Inserting " << mobileEntry->mName << endl;
   
     uint j=0;
     for(;j<mSyncer->mCommonEntries.count();++j) {
@@ -523,7 +496,7 @@ void MobileGui::mergePhonebooks()
     }
   }
   
-//  kdDebug() << " Resolve conflicts" << endl;
+//  kdDebug(5960) << " Resolve conflicts" << endl;
 
   // Resolve conflicts
   bool kabUpdated = false;
@@ -539,15 +512,16 @@ void MobileGui::mergePhonebooks()
       } else {
         // Merge mobileEntrys
         // This alters the mobile and kab lists. Perhaps we should reflect this in the GUI.
-        QString text = "<b>Kab Entry:</b><br>";
+        QString text = "<qt><b>" + i18n("Kab Entry:") + "</b><br>";
         text += "  " + kabEntry->mName + " " + kabEntry->mPhone + "<br>";
-        text += "<b>Mobile Entry:</b><br>";
+        text += "<b>" + i18n("Mobile Entry:") + "</b><br>";
         text += "  " + mobileEntry->mName + " " + mobileEntry->mPhone;
+        text += "</qt>";
       
         QMessageBox *msg = new QMessageBox(i18n("Conflicting Entries"),text,
                                          QMessageBox::Warning,1,2,0,this);
-        msg->setButtonText(1,i18n("Use Kab entry"));
-        msg->setButtonText(2,i18n("Use Mobile entry"));
+        msg->setButtonText(1,i18n("Use Kab Entry"));
+        msg->setButtonText(2,i18n("Use Mobile Entry"));
         switch (msg->exec()) {
           case 1: // use kab entry
             mobileEntry->mPhone = kabEntry->mPhone;
@@ -564,7 +538,7 @@ void MobileGui::mergePhonebooks()
     }
   }
 
-//  kdDebug() << " Create new entries" << endl;
+//  kdDebug(5960) << " Create new entries" << endl;
 
   // Create new entries
   for(uint i=0;i<mSyncer->mCommonEntries.count();++i) {
@@ -573,7 +547,7 @@ void MobileGui::mergePhonebooks()
     SyncEntryMobile *mobileEntry = entry->mMobileEntry;
 
     if (kabEntry && !mobileEntry) {
-      kdDebug() << "Creating mobile entry for " << kabEntry->mPhone << endl;
+      kdDebug(5960) << "Creating mobile entry for " << kabEntry->mPhone << endl;
       // Create mobile entry
       // The type should be generated here.
       // The values should be checked for validity.
@@ -603,7 +577,7 @@ void MobileGui::mergePhonebooks()
       mobileUpdated = true;
     } else if (mobileEntry && !kabEntry) {
       // Create kab entry
-      QString phonetype = "2";
+      QString phonetype = "0";
       entry->mKabEntry = new SyncEntryKab(true,mobileEntry->mIndex,mobileEntry->mName,
                                           mobileEntry->mType,mobileEntry->mPhone,
                                           phonetype);
@@ -613,13 +587,13 @@ void MobileGui::mergePhonebooks()
     }
   }
 
-//  kdDebug() << "Update gui" << endl;
+//  kdDebug(5960) << "Update gui" << endl;
 
   // Update kab and mobile entries
   if (kabUpdated) updateKabBook();
   if (mobileUpdated) updateMobileBook();
 
-  kdDebug() << "MobileGui::mergePhonebooks() done." << endl;
+  kdDebug(5960) << "MobileGui::mergePhonebooks() done." << endl;
 }
 
 void MobileGui::syncPhonebooks()
@@ -627,7 +601,7 @@ void MobileGui::syncPhonebooks()
   if (mSyncing) return;
 
   mSyncing = true;
-  readKab();
+  readKabc();
   readPhonebook();
   mSyncReadId = "+cpbr=1,150";
 }
