@@ -1,36 +1,37 @@
 /* -*- Mode: C++ -*-
-   $Id: KDGanttView.h,v 1.2 2003/10/24 14:26:38 thorsen Exp $
+   $Id: KDGanttView.h,v 1.4 2004/03/21 13:34:16 rogowski Exp $
    KDChart - a multi-platform charting engine
 */
 
 /****************************************************************************
-** Copyright (C) 2002 Klarlvdalens Datakonsult AB.  All rights reserved.
-**
-** This file is part of the KDGantt library.
-**
-** This file may be distributed and/or modified under the terms of the
-** GNU General Public License version 2 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.
-**
-** Licensees holding valid commercial KDGantt licenses may use this file in
-** accordance with the KDGantt Commercial License Agreement provided with
-** the Software.
-**
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
-**
-** See http://www.klaralvdalens-datakonsult.se/Public/products/ for
-**   information about KDGantt Commercial License Agreements.
-**
-** Contact info@klaralvdalens-datakonsult.se if any conditions of this
-** licensing are not clear to you.
-**
-** As a special exception, permission is given to link this program
-** with any edition of Qt, and distribute the resulting executable,
-** without including the source code for Qt in the source distribution.
-**
-**********************************************************************/
+ ** Copyright (C)  2001-2004 Klarlvdalens Datakonsult AB.  All rights reserved.
+ **
+ ** This file is part of the KDGantt library.
+ **
+ ** This file may be distributed and/or modified under the terms of the
+ ** GNU General Public License version 2 as published by the Free Software
+ ** Foundation and appearing in the file LICENSE.GPL included in the
+ ** packaging of this file.
+ **
+ ** Licensees holding valid commercial KDGantt licenses may use this file in
+ ** accordance with the KDGantt Commercial License Agreement provided with
+ ** the Software.
+ **
+ ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
+ ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ **
+ ** See http://www.klaralvdalens-datakonsult.se/Public/products/ for
+ **   information about KDGantt Commercial License Agreements.
+ **
+ ** Contact info@klaralvdalens-datakonsult.se if any conditions of this
+ ** licensing are not clear to you.
+ **
+ ** As a special exception, permission is given to link this program
+ ** with any edition of Qt, and distribute the resulting executable,
+ ** without including the source code for Qt in the source distribution.
+ **
+ **********************************************************************/
+
 #ifndef KDGANTTVIEW_H
 #define KDGANTTVIEW_H
 
@@ -46,11 +47,12 @@
 
 #include "KDGanttViewItem.h"
 #include "KDGanttViewTaskLinkGroup.h"
-#include "KDMinimizeSplitter.h"
+#include "KDGanttMinimizeSplitter.h"
+#include "KDGanttViewItemDrag.h"
 
 class KDGanttViewTaskLink;
 class QPrinter;
-class QFile;
+class QIODevice;
 class itemAttributeDialog;
 class KDListView;
 class KDGanttViewItem;
@@ -62,10 +64,12 @@ class KDTimeHeaderWidget;
 class KDLegendWidget;
 class KDGanttCanvasView;
 class KDGanttViewTaskLink;
+class KDGanttMinimizeSplitter;
 
-class KDGanttView : public KDMinimizeSplitter
+class KDGanttView : public KDGanttMinimizeSplitter
 {
     Q_OBJECT
+
     Q_PROPERTY( bool showLegend READ showLegend WRITE setShowLegend )
     Q_PROPERTY( bool showListView READ showListView WRITE setShowListView )
     Q_PROPERTY( bool showTaskLinks READ showTaskLinks WRITE setShowTaskLinks )
@@ -81,15 +85,27 @@ class KDGanttView : public KDMinimizeSplitter
     Q_PROPERTY( QColor textColor READ textColor WRITE setTextColor )
     Q_PROPERTY( int majorScaleCount READ majorScaleCount WRITE setMajorScaleCount )
     Q_PROPERTY( int minorScaleCount READ minorScaleCount WRITE setMinorScaleCount )
+    Q_PROPERTY( int autoScaleMinorTickCount READ autoScaleMinorTickCount WRITE setAutoScaleMinorTickCount )
+    Q_PROPERTY( Scale maximumScale READ maximumScale WRITE setMaximumScale )
+    Q_PROPERTY( Scale minimumScale READ minimumScale WRITE setMinimumScale )
     Q_PROPERTY( int minimumColumnWidth READ minimumColumnWidth WRITE setMinimumColumnWidth )
+    Q_PROPERTY( int ganttMaximumWidth READ ganttMaximumWidth WRITE setGanttMaximumWidth )
     Q_PROPERTY( QColor weekendBackgroundColor READ weekendBackgroundColor WRITE setWeekendBackgroundColor )
+    Q_PROPERTY( QColor ganttViewBackgroundColor READ gvBackgroundColor WRITE setGvBackgroundColor )
+    Q_PROPERTY( QColor listViewBackgroundColor READ lvBackgroundColor WRITE setLvBackgroundColor )
+    Q_PROPERTY( QColor timeHeaderBackgroundColor READ timeHeaderBackgroundColor WRITE setTimeHeaderBackgroundColor )
+    Q_PROPERTY( QColor legendHeaderBackgroundColor READ legendHeaderBackgroundColor WRITE setLegendHeaderBackgroundColor )
     Q_PROPERTY( double zoomFactor READ zoomFactor )
-    Q_PROPERTY( bool showHeaderPopupMenu READ showHeaderPopupMenu
-                WRITE setShowHeaderPopupMenu )
-        Q_PROPERTY( bool headerVisible READ headerVisible
-                    WRITE setHeaderVisible )
-        Q_PROPERTY( bool showLegendButton READ showLegendButton
-                    WRITE setShowLegendButton )
+    Q_PROPERTY( bool showHeaderPopupMenu READ showHeaderPopupMenu WRITE setShowHeaderPopupMenu )
+    Q_PROPERTY( bool showTimeTablePopupMenu READ showTimeTablePopupMenu WRITE setShowTimeTablePopupMenu )
+    Q_PROPERTY( bool headerVisible READ headerVisible WRITE setHeaderVisible )
+    Q_PROPERTY( bool showLegendButton READ showLegendButton	WRITE setShowLegendButton )
+    Q_PROPERTY( bool legendIsDockwindow READ legendIsDockwindow WRITE setLegendIsDockwindow )
+    Q_PROPERTY( bool displayEmptyTasksAsLine READ displayEmptyTasksAsLine WRITE setDisplayEmptyTasksAsLine )
+    Q_PROPERTY( QBrush noInformationBrush READ noInformationBrush WRITE setNoInformationBrush )
+    Q_PROPERTY( bool dragEnabled READ dragEnabled WRITE setDragEnabled )
+    Q_PROPERTY( bool dropEnabled READ dropEnabled WRITE setDropEnabled )
+    Q_PROPERTY( bool calendarMode READ calendarMode WRITE setCalendarMode )
 
     Q_ENUMS( Scale )
     Q_ENUMS( YearFormat )
@@ -97,40 +113,48 @@ class KDGanttView : public KDMinimizeSplitter
 
 public:
     enum Scale { Minute, Hour, Day, Week, Month, Auto };
-    enum YearFormat { FourDigit, TwoDigit, TwoDigitApostrophe };
-    enum HourFormat { Hour_24, Hour_12 };
+    enum YearFormat { FourDigit, TwoDigit, TwoDigitApostrophe, NoDate };
+    enum HourFormat { Hour_24, Hour_12, Hour_24_FourDigit };
     enum RepaintMode { No, Medium, Always };
 
     KDGanttView( QWidget* parent = 0, const char* name = 0 );
     ~KDGanttView();
 
     virtual void show();
-
+    virtual bool close ( bool alsoDelete );
     void setRepaintMode( RepaintMode mode );
     void setUpdateEnabled( bool enable);
     bool getUpdateEnabled( )const;
 
+    void setGanttMaximumWidth( int w );
+    int ganttMaximumWidth() const;
     void setShowLegend( bool show );
     bool showLegend() const;
+    void setLegendIsDockwindow( bool dock );
+    bool legendIsDockwindow( ) const;
+    QDockWindow* legendDockwindow( ) const;
     void setShowListView( bool show );
     bool showListView() const;
     void setEditorEnabled( bool enable );
     bool editorEnabled() const;
-
+    void setListViewWidth( int );
+    int listViewWidth();
     void setEditable( bool editable );
     bool editable() const;
     void setCalendarMode( bool mode );
-    bool calendarMode();
+    bool calendarMode() const;
     void setDisplaySubitemsAsGroup( bool show );
     bool displaySubitemsAsGroup() const;
+    void setDisplayEmptyTasksAsLine( bool show );
+    bool displayEmptyTasksAsLine() const;
 
     void setHorBackgroundLines( int count = 2,
 				QBrush brush =
-				QBrush( QColor ( 200,200,200 ), 
+				QBrush( QColor ( 200,200,200 ),
 					Qt::Dense6Pattern ));
     int horBackgroundLines( QBrush& brush );
-    bool saveProject( QFile* );
-    bool loadProject( QFile* );
+    bool saveProject( QIODevice* );
+    bool loadProject( QIODevice* );
     void print( QPrinter* printer = 0 ,
 		bool printListView = true, bool printTimeLine = true,
 		bool printLegend = false );
@@ -140,7 +164,6 @@ public:
     void setZoomFactor( double factor, bool absolute );
     double zoomFactor() const;
     void zoomToFit();
-    void zoomToSelection( const QDateTime& start,  const QDateTime&  end);
     void ensureVisible( KDGanttViewItem* );
     void center( KDGanttViewItem* );
     void centerTimeline( const QDateTime& center );
@@ -153,35 +176,48 @@ public:
     bool showTaskLinks() const;
 
     void setFont(const QFont& f);
-    void setShowHeaderPopupMenu( bool );
+    void setShowHeaderPopupMenu( bool show = true,
+                                 bool showZoom = true,
+                                 bool showScale = true,
+                                 bool showTime = true,
+                                 bool showYear = true,
+                                 bool showGrid = true,
+                                 bool showPrint = false);
     bool showHeaderPopupMenu() const;
     void setShowTimeTablePopupMenu( bool );
     bool showTimeTablePopupMenu() const;
     void setShapes( KDGanttViewItem::Type type,
                     KDGanttViewItem::Shape start,
                     KDGanttViewItem::Shape middle,
-                    KDGanttViewItem::Shape end );
+                    KDGanttViewItem::Shape end,
+		    bool overwriteExisting = true  );
     bool shapes( KDGanttViewItem::Type type,
                  KDGanttViewItem::Shape& start,
                  KDGanttViewItem::Shape& middle,
                  KDGanttViewItem::Shape& end ) const;
     void setColors( KDGanttViewItem::Type type,
                     const QColor& start, const QColor& middle,
-                    const QColor& end );
+                    const QColor& end,
+		    bool overwriteExisting = true );
     bool colors( KDGanttViewItem::Type type,
                  QColor& start, QColor& middle, QColor& end ) const;
-    void setDefaultColor( KDGanttViewItem::Type type, const QColor& );
+    void setDefaultColor( KDGanttViewItem::Type type,
+			  const QColor&,
+			  bool overwriteExisting = true );
     QColor defaultColor( KDGanttViewItem::Type type ) const;
     void setHighlightColors( KDGanttViewItem::Type type,
                              const QColor& start, const QColor& middle,
-                             const QColor& end );
+                             const QColor& end,
+			     bool overwriteExisting = true );
     bool highlightColors( KDGanttViewItem::Type type,
                           QColor& start, QColor& middle, QColor& end ) const;
-    void setDefaultHighlightColor( KDGanttViewItem::Type type, const QColor& );
+    void setDefaultHighlightColor( KDGanttViewItem::Type type,
+				   const QColor&,
+				   bool overwriteExisting = true );
     QColor defaultHighlightColor( KDGanttViewItem::Type type ) const;
     void setTextColor( const QColor& color );
     QColor textColor() const;
-  
+
     void setNoInformationBrush( const QBrush& brush );
     QBrush noInformationBrush() const;
 
@@ -240,6 +276,19 @@ public:
     QColor weekendBackgroundColor() const;
     void setWeekdayBackgroundColor( const QColor& color, int weekday );
     QColor weekdayBackgroundColor(int weekday) const;
+
+
+    void setPaletteBackgroundColor(const QColor& col);
+    void setGvBackgroundColor ( const QColor & );
+    void setLvBackgroundColor ( const QColor & );
+    void setTimeHeaderBackgroundColor ( const QColor & );
+    void setLegendHeaderBackgroundColor ( const QColor & );
+    QColor gvBackgroundColor () const ;
+    QColor lvBackgroundColor () const ;
+    QColor timeHeaderBackgroundColor () const ;
+    QColor legendHeaderBackgroundColor () const ;
+    void addUserdefinedLegendHeaderWidget( QWidget * w );
+
     void setWeekendDays( int start, int end );
     void weekendDays( int& start, int& end ) const;
 
@@ -247,52 +296,74 @@ public:
 
     void setHeaderVisible( bool );
     bool headerVisible() const;
-    
+
     void setShowLegendButton( bool show );
     bool showLegendButton() const;
-    
+
     // Pass-through methods from QListView
     virtual int addColumn( const QString& label, int width = -1 );
     virtual int addColumn( const QIconSet& iconset, const QString& label,
                            int width = -1 );
     virtual void removeColumn( int index );
     KDGanttViewItem* selectedItem() const;
-    void setSelected( KDGanttViewItem*, bool );    
+    void setSelected( KDGanttViewItem*, bool );
     KDGanttViewItem* firstChild() const;
     KDGanttViewItem* lastItem() const;
     int childCount() const;
     void clear();
 
+    void setDragEnabled( bool b );
+    void setDropEnabled( bool b );
+    void setDragDropEnabled( bool b );
+    bool dragEnabled() const;
+    bool dropEnabled() const;
+    bool isDragEnabled() const;
+    bool isDropEnabled() const;
+
+    virtual bool lvDropEvent ( QDropEvent *e, KDGanttViewItem*, KDGanttViewItem*);
+    virtual void lvStartDrag (KDGanttViewItem*);
+    virtual bool lvDragMoveEvent (QDragMoveEvent * e,KDGanttViewItem*, KDGanttViewItem*);
+    virtual void lvDragEnterEvent (QDragEnterEvent * e );
     QSize sizeHint();
-    KDGanttViewItem* getItemByName( const QString& name );
-    QDateTime getDateTimeForCoordX(int coordX);
+    KDGanttViewItem* getItemByName( const QString& name ) const;
+    QDateTime getDateTimeForCoordX(int coordX, bool global = true ) const;
+    KDGanttViewItem* getItemByListViewPos( const QPoint& pos ) const;
+    KDGanttViewItem* getItemByGanttViewPos( const QPoint& pos ) const;
+    KDGanttViewItem* getItemAt( const QPoint& pos , bool global = true ) const;
+
+    // setting the vertical scrollbars of the listview and the timetable
+    // default values: always off for the listview, always on for the timetable
+    void setLvVScrollBarMode( QScrollView::ScrollBarMode );
+    void setGvVScrollBarMode( QScrollView::ScrollBarMode );
+
 public slots:
     void editItem( KDGanttViewItem* );
+    void zoomToSelection( const QDateTime& start,  const QDateTime&  end);
+
+signals:
+    void timeIntervallSelected( const QDateTime& start,  const QDateTime&  end);
+    void timeIntervalSelected( const QDateTime& start,  const QDateTime&  end);
+    void rescaling( Scale );
 
-  signals:
-   
     // the following signals are emitted if an item is clicked in the
     // listview (inclusive) or in the ganttview
     void itemLeftClicked( KDGanttViewItem* );
     void itemMidClicked( KDGanttViewItem* );
     void itemRightClicked( KDGanttViewItem* );
     void itemDoubleClicked( KDGanttViewItem* );
-   
-    void mouseButtonClicked ( int button, KDGanttViewItem* item, const QPoint & pos, int c );
+
     // the following signals are emitted if an item is clicked in the
     // listview (exlusive) or in the ganttview
-
     // gv... means item in ganttview clicked
 
-    // gvCurrentChanged() is emitted whenever the user clicks on the gantt view
-    // the parameter is 0, if no item was clicked
     void gvCurrentChanged( KDGanttViewItem* );
     void gvItemLeftClicked( KDGanttViewItem* );
     void gvItemMidClicked( KDGanttViewItem* );
     void gvItemRightClicked( KDGanttViewItem* );
+  // the point is the global position!!
     void gvMouseButtonClicked ( int button, KDGanttViewItem* item, const QPoint & pos);
     void gvItemDoubleClicked( KDGanttViewItem* );
-    // the point is the global positiion!!
+    // the point is the global position!!
     void gvContextMenuRequested ( KDGanttViewItem * item, const QPoint & pos );
 
     // lv... means item in listview clicked
@@ -302,10 +373,10 @@ public slots:
     void lvItemLeftClicked( KDGanttViewItem* );
     void lvItemMidClicked( KDGanttViewItem* );
     void lvItemRightClicked( KDGanttViewItem* );
-    void lvContextMenuRequested ( KDGanttViewItem * item, const QPoint & pos, int col ); 
-    void lvMouseButtonClicked ( int button, KDGanttViewItem* item, const QPoint & pos, int c ); 
-    void lvItemDoubleClicked( KDGanttViewItem* ); 
-    void lvSelectionChanged( KDGanttViewItem* ); 
+    void lvContextMenuRequested ( KDGanttViewItem * item, const QPoint & pos, int col );
+    void lvMouseButtonClicked ( int button, KDGanttViewItem* item, const QPoint & pos, int c );
+    void lvItemDoubleClicked( KDGanttViewItem* );
+    void lvSelectionChanged( KDGanttViewItem* );
 
     void itemConfigured( KDGanttViewItem* );
 
@@ -314,7 +385,7 @@ public slots:
     void taskLinkRightClicked( KDGanttViewTaskLink* );
     void taskLinkDoubleClicked( KDGanttViewTaskLink* );
 
-
+    void dropped ( QDropEvent * e, KDGanttViewItem* droppedItem, KDGanttViewItem* itemBelowMouse);
 private slots:
     void forceRepaint( int val = 0 );
     void slotSelectionChanged( QListViewItem* item );
@@ -328,6 +399,7 @@ private slots:
     void addTickRight();
     void addTickLeft();
     void enableAdding( int );
+    void slot_lvDropped(QDropEvent* e, KDGanttViewItem* droppedItem, KDGanttViewItem* itemBelowMouse );
 private:
     struct legendItem {
       KDGanttViewItem::Shape shape;
@@ -365,17 +437,24 @@ private:
     KDLegendWidget * myLegend;
     itemAttributeDialog* myItemAttributeDialog;
     QVBox * leftWidget,  * rightWidget;
+    QHBox * spacerLeft;
     QScrollView* myTimeHeaderScroll;
-    QWidget* spacerLeft, *spacerRight;
- 
+    QHBox* myTimeHeaderContainer ;
+    QWidget* timeHeaderSpacerWidget;
+    QWidget *spacerRight;
+
     bool listViewIsVisible;
     bool chartIsEditable;
     bool editorIsEnabled;
     bool _displaySubitemsAsGroup;
+    bool _displayEmptyTasksAsLine;
     bool _showLegendButton;
     bool _showHeader;
     bool _enableAdding;
     bool fCenterTimeLineAfterShow;
+    bool fDragEnabled;
+    bool fDropEnabled;
+    bool closingBlocked;
     QDateTime dtCenterTimeLineAfterShow;
     KDGanttViewItem::Shape myDefaultShape [9];
     QColor myColor[9],myColorHL[9];
@@ -387,7 +466,15 @@ private:
     void addTaskLinkGroup(KDGanttViewTaskLinkGroup*);
     void removeTaskLinkGroup(KDGanttViewTaskLinkGroup*);
     int getIndex( KDGanttViewItem::Type ) const;
+    void notifyEditdialog( KDGanttViewItem * );
     void initDefaults();
+    KDGanttViewItem* myCurrentItem;
+    KDGanttMinimizeSplitter *mySplitter;
+protected:
+  virtual QDragObject * dragObject ();
+  virtual void startDrag ();
 };
 
+
+
 #endif
