@@ -35,10 +35,14 @@
 
 #include "imapjob.h"
 #include "kmfolderimap.h"
+#include "kmfolder.h"
 #include "kmmsgpart.h"
+#include "progressmanager.h"
+using KPIM::ProgressManager;
 
 #include <kio/scheduler.h>
 #include <kdebug.h>
+#include <klocale.h>
 #include <mimelib/body.h>
 #include <mimelib/bodypart.h>
 #include <mimelib/string.h>
@@ -49,16 +53,16 @@ namespace KMail {
 //-----------------------------------------------------------------------------
 ImapJob::ImapJob( KMMessage *msg, JobType jt, KMFolderImap* folder,
     QString partSpecifier, const AttachmentStrategy *as )
-  : FolderJob( msg, jt, folder, partSpecifier ),
-    mAttachmentStrategy( as )
+  : FolderJob( msg, jt, folder? folder->folder() : 0, partSpecifier ),
+    mAttachmentStrategy( as ), mParentProgressItem(0)
 {
 }
 
 //-----------------------------------------------------------------------------
 ImapJob::ImapJob( QPtrList<KMMessage>& msgList, QString sets, JobType jt,
                   KMFolderImap* folder )
-  : FolderJob( msgList, sets, jt, folder ),
-    mAttachmentStrategy ( 0 )
+  : FolderJob( msgList, sets, jt, folder? folder->folder() : 0 ),
+    mAttachmentStrategy ( 0 ), mParentProgressItem(0)
 {
 }
 
@@ -66,17 +70,23 @@ void ImapJob::init( JobType jt, QString sets, KMFolderImap* folder,
                     QPtrList<KMMessage>& msgList )
 {
   mJob = 0;
+
   assert(jt == tGetMessage || folder);
   KMMessage* msg = msgList.first();
+   // guard against empty list
+  if ( !msg ) {
+    deleteLater();
+    return;
+  }
   mType = jt;
-  mDestFolder = folder;
+  mDestFolder = folder? folder->folder() : 0;
   // refcount++
   if (folder) {
     folder->open();
   }
   KMFolder *msg_parent = msg->parent();
   if (msg_parent) {
-    if (!folder || folder!= msg_parent) {
+    if (!folder || folder!= msg_parent->storage()) {
       msg_parent->open();
     }
   }
@@ -88,8 +98,8 @@ void ImapJob::init( JobType jt, QString sets, KMFolderImap* folder,
   KMAcctImap *account;
   if (folder) {
     account = folder->account();
-  } else { 
-    account = static_cast<KMFolderImap*>(msg_parent)->account();
+  } else {
+    account = static_cast<KMFolderImap*>(msg_parent->storage())->account();
   }
   if ( !account ||
        account->makeConnection() == ImapAccountBase::Error ) {
@@ -101,16 +111,16 @@ void ImapJob::init( JobType jt, QString sets, KMFolderImap* folder,
   {
     // transfers the complete message to the server
     KURL url = account->getUrl();
-    QString flags = KMFolderImap::statusToFlags( msg->status() ); 
+    QString flags = KMFolderImap::statusToFlags( msg->status() );
     url.setPath( folder->imapPath() + ";SECTION=" + flags );
     ImapAccountBase::jobData jd;
-    jd.parent = 0; jd.offset = 0;
-    jd.total = 1; jd.done = 0;
+    jd.parent = 0; jd.offset = 0; jd.done = 0;
+    jd.total = msg->msgSizeServer();
     jd.msgList.append(msg);
     QCString cstr( msg->asString() );
-    int a = cstr.find( "\nX-UID: " );
-    int b = cstr.find( "\n", a );
-    if ( a != -1 && b != -1 && cstr.find( "\n\n" ) > a ) cstr.remove( a, b-a );
+    int a = cstr.find("\nX-UID: ");
+    int b = cstr.find('\n', a);
+    if (a != -1 && b != -1 && cstr.find("\n\n") > a) cstr.remove(a, b-a);
     mData.resize( cstr.length() + cstr.contains( "\n" ) - cstr.contains( "\r\n" ) );
     unsigned int i = 0;
     char prevChar = '\0';
@@ -126,6 +136,16 @@ void ImapJob::init( JobType jt, QString sets, KMFolderImap* folder,
       i++;
     }
     jd.data = mData;
+    jd.progressItem = ProgressManager::createProgressItem(
+                          mParentProgressItem,
+                          "ImapJobUploading"+ProgressManager::getUniqueID(),
+                          i18n("Uploading message data"),
+                          i18n("Destination folder: ") + mDestFolder->prettyURL(),
+                          true,
+                          account->useSSL() || account->useTLS() );
+    jd.progressItem->setTotalItems( jd.total );
+    connect ( jd.progressItem, SIGNAL( progressItemCanceled( ProgressItem* ) ),
+              account, SLOT( slotAbortRequested( ProgressItem* ) ) );
     KIO::SimpleJob *simpleJob = KIO::put( url, 0, FALSE, FALSE, FALSE );
     KIO::Scheduler::assignJobToSlave( account->slave(), simpleJob );
     mJob = simpleJob;
@@ -136,16 +156,18 @@ void ImapJob::init( JobType jt, QString sets, KMFolderImap* folder,
              SLOT(slotPutMessageDataReq(KIO::Job *, QByteArray &)) );
     connect( mJob, SIGNAL(infoMessage(KIO::Job *, const QString &)),
              SLOT(slotPutMessageInfoData(KIO::Job *, const QString &)) );
+    connect( mJob, SIGNAL(processedSize(KIO::Job *, KIO::filesize_t)),
+             SLOT(slotProcessedSize(KIO::Job *, KIO::filesize_t)));
   }
   else if ( jt == tCopyMessage || jt == tMoveMessage )
   {
     KURL url = account->getUrl();
     KURL destUrl = account->getUrl();
     destUrl.setPath(folder->imapPath());
-    KMFolderImap *imapDestFolder = static_cast<KMFolderImap*>(msg_parent);
+    KMFolderImap *imapDestFolder = static_cast<KMFolderImap*>(msg_parent->storage());
     url.setPath( imapDestFolder->imapPath() + ";UID=" + sets );
     ImapAccountBase::jobData jd;
-    jd.parent = 0; mOffset = 0;
+    jd.parent = 0; jd.offset = 0;
     jd.total = 1; jd.done = 0;
     jd.msgList = msgList;
 
@@ -153,7 +175,17 @@ void ImapJob::init( JobType jt, QString sets, KMFolderImap* folder,
     QDataStream stream( packedArgs, IO_WriteOnly );
 
     stream << (int) 'C' << url << destUrl;
-
+    jd.progressItem = ProgressManager::createProgressItem(
+                          mParentProgressItem,
+                          "ImapJobCopyMove"+ProgressManager::getUniqueID(),
+                          i18n("Server operation"),
+                          i18n("Source folder: ") + msg_parent->prettyURL()
+                          + " - destination folder: " + mDestFolder->prettyURL(),
+                          true,
+                          account->useSSL() || account->useTLS() );
+    jd.progressItem->setTotalItems( jd.total );
+    connect ( jd.progressItem, SIGNAL( progressItemCanceled( ProgressItem* ) ),
+              account, SLOT( slotAbortRequested( ProgressItem* ) ) );
     KIO::SimpleJob *simpleJob = KIO::special( url, packedArgs, FALSE );
     KIO::Scheduler::assignJobToSlave( account->slave(), simpleJob );
     mJob = simpleJob;
@@ -165,7 +197,8 @@ void ImapJob::init( JobType jt, QString sets, KMFolderImap* folder,
       connect( mJob, SIGNAL(infoMessage(KIO::Job *, const QString &)),
                SLOT(slotCopyMessageInfoData(KIO::Job *, const QString &)) );
     }
-  } else {
+  }
+  else {
     slotGetNextMessage();
   }
 }
@@ -174,42 +207,47 @@ void ImapJob::init( JobType jt, QString sets, KMFolderImap* folder,
 //-----------------------------------------------------------------------------
 ImapJob::~ImapJob()
 {
-
   if ( mDestFolder )
   {
-    KMAcctImap *account = static_cast<KMFolderImap*>(mDestFolder)->account();
-    if ( account ) // just to be sure this job is removed from the list
-      account->mJobList.remove(this);
-    if ( account && mJob )
-    {
+    KMAcctImap *account = static_cast<KMFolderImap*>(mDestFolder->storage())->account();
+    if ( account &&  mJob ) {
       ImapAccountBase::JobIterator it = account->findJob( mJob );
-      if ( it != account->jobsEnd() && !(*it).msgList.isEmpty() )
-      {
-        for ( QPtrListIterator<KMMessage> mit( (*it).msgList ); mit.current(); ++mit )
-          mit.current()->setTransferInProgress(false);
+      if ( it != account->jobsEnd() ) {
+        if( (*it).progressItem ) {
+          (*it).progressItem->setComplete();
+          (*it).progressItem = 0;
+        }
+        if ( !(*it).msgList.isEmpty() ) {
+          for ( QPtrListIterator<KMMessage> mit( (*it).msgList ); mit.current(); ++mit )
+            mit.current()->setTransferInProgress( false );
+        }
       }
+      account->removeJob( mJob );
     }
-      
+    account->mJobList.remove( this );
     mDestFolder->close();
   }
 
-  if (mSrcFolder) {
+  if ( mSrcFolder ) {
     if (!mDestFolder || mDestFolder != mSrcFolder) {
       if (! (mSrcFolder->folderType() == KMFolderTypeImap) ) return;
-      KMAcctImap *account = static_cast<KMFolderImap*>(mSrcFolder)->account();
-      if ( account ) // just to be sure this job is removed from the list
-        account->mJobList.remove(this);
-      if ( account && mJob )
-      {
+      KMAcctImap *account = static_cast<KMFolderImap*>(mSrcFolder->storage())->account();
+      if ( account && mJob ) {
         ImapAccountBase::JobIterator it = account->findJob( mJob );
-        if ( it != account->jobsEnd() && !(*it).msgList.isEmpty() )
-        {
-          for ( QPtrListIterator<KMMessage> mit( (*it).msgList ); mit.current(); ++mit )
-            mit.current()->setTransferInProgress(false);
+        if ( it != account->jobsEnd() ) {
+          if( (*it).progressItem ) {
+            (*it).progressItem->setComplete();
+            (*it).progressItem = 0;
+          }
+          if ( !(*it).msgList.isEmpty() ) {
+            for ( QPtrListIterator<KMMessage> mit( (*it).msgList ); mit.current(); ++mit )
+              mit.current()->setTransferInProgress( false );
+          }
         }
+        account->removeJob( mJob ); // remove the associated kio job
       }
+      account->mJobList.remove( this ); // remove the folderjob
     }
-      
     mSrcFolder->close();
   }
 }
@@ -219,9 +257,9 @@ ImapJob::~ImapJob()
 void ImapJob::slotGetNextMessage()
 {
   KMMessage *msg = mMsgList.first();
-  KMFolderImap *msgParent = static_cast<KMFolderImap*>(msg->parent());
+  KMFolderImap *msgParent = static_cast<KMFolderImap*>(msg->storage());
   KMAcctImap *account = msgParent->account();
-  if ( msg->headerField("X-UID").isEmpty() )
+  if ( msg->UID() == 0 )
   {
     emit messageRetrieved( msg );
     account->mJobList.remove( this );
@@ -229,7 +267,11 @@ void ImapJob::slotGetNextMessage()
     return;
   }
   KURL url = account->getUrl();
-  QString path = msgParent->imapPath() + ";UID=" + msg->headerField("X-UID");
+  QString path = msgParent->imapPath() + ";UID=" + QString::number(msg->UID());
+  ImapAccountBase::jobData jd;
+  jd.parent = 0; jd.offset = 0;
+  jd.total = 1; jd.done = 0;
+  jd.msgList.append( msg );
   if ( !mPartSpecifier.isEmpty() )
   {
     if ( mPartSpecifier.find ("STRUCTURE", 0, false) != -1 ) {
@@ -237,18 +279,31 @@ void ImapJob::slotGetNextMessage()
     } else if ( mPartSpecifier == "HEADER" ) {
       path += ";SECTION=HEADER";
     } else {
-      path += ";SECTION=BODY.PEEK[" + mPartSpecifier +"]";
+      path += ";SECTION=BODY.PEEK[" + mPartSpecifier + "]";
+      DwBodyPart * part = msg->findDwBodyPart( msg->getFirstDwBodyPart(), mPartSpecifier );
+      if (part)
+        jd.total = part->BodySize();
     }
   } else {
       path += ";SECTION=BODY.PEEK[]";
+      if (msg->msgSizeServer() > 0)
+        jd.total = msg->msgSizeServer();
   }
   url.setPath( path );
 //  kdDebug(5006) << "ImapJob::slotGetNextMessage - retrieve " << url.path() << endl;
-  ImapAccountBase::jobData jd;
-  jd.parent = 0;
-  jd.total = 1; jd.done = 0;
   // protect the message, otherwise we'll get crashes afterwards
   msg->setTransferInProgress( true );
+  jd.progressItem = ProgressManager::createProgressItem(
+                          mParentProgressItem,
+                          "ImapJobDownloading"+ProgressManager::getUniqueID(),
+                          i18n("Downloading message data"),
+                          i18n("Message with subject: ") + msg->subject(),
+                          true,
+                          account->useSSL() || account->useTLS() );
+  connect ( jd.progressItem, SIGNAL( progressItemCanceled( ProgressItem* ) ),
+            account, SLOT( slotAbortRequested( ProgressItem* ) ) );
+  jd.progressItem->setTotalItems( jd.total );
+
   KIO::SimpleJob *simpleJob = KIO::get( url, FALSE, FALSE );
   KIO::Scheduler::assignJobToSlave( account->slave(), simpleJob );
   mJob = simpleJob;
@@ -263,6 +318,11 @@ void ImapJob::slotGetNextMessage()
   }
   connect( mJob, SIGNAL(data(KIO::Job *, const QByteArray &)),
            msgParent, SLOT(slotSimpleData(KIO::Job *, const QByteArray &)) );
+  if ( jd.total > 1 )
+  {
+    connect(mJob, SIGNAL(processedSize(KIO::Job *, KIO::filesize_t)),
+        this, SLOT(slotProcessedSize(KIO::Job *, KIO::filesize_t)));
+  }
 }
 
 
@@ -274,7 +334,7 @@ void ImapJob::slotGetMessageResult( KIO::Job * job )
     deleteLater();
     return;
   }
-  KMFolderImap* parent = static_cast<KMFolderImap*>(msg->parent());
+  KMFolderImap* parent = static_cast<KMFolderImap*>(msg->storage());
   if (msg->transferInProgress())
     msg->setTransferInProgress( false );
   KMAcctImap *account = parent->account();
@@ -288,7 +348,10 @@ void ImapJob::slotGetMessageResult( KIO::Job * job )
   bool gotData = true;
   if (job->error())
   {
-    account->slotSlaveError( account->slave(), job->error(), job->errorText() );
+    QString errorStr = i18n( "Error while retrieving messages from the server." );
+    if ( (*it).progressItem )
+      (*it).progressItem->setStatus( errorStr );
+    account->handleJobError( job, errorStr );
     return;
   } else {
     if ((*it).data.size() > 0)
@@ -297,20 +360,35 @@ void ImapJob::slotGetMessageResult( KIO::Job * job )
       if ( mPartSpecifier.isEmpty() ||
            mPartSpecifier == "HEADER" )
       {
-        uint size = msg->headerField("X-Length").toUInt();
-        QString uid = msg->headerField("X-UID");
-        msg->fromByteArray( (*it).data );
-        msg->setHeaderField("X-UID",uid);
-        // set correct size
-        if ( size > 0 ) msg->setMsgLength(size);
-        if ( mPartSpecifier.isEmpty() ) 
+        uint size = msg->msgSizeServer();
+        if ( size > 0 && mPartSpecifier.isEmpty() )
+          (*it).done = size;
+        ulong uid = msg->UID();
+        // must set this first so that msg->fromByteArray sets the attachment status
+        if ( mPartSpecifier.isEmpty() )
           msg->setComplete( true );
         else
           msg->setReadyToShow( false );
+
+        // Convert CR/LF to LF.
+        size_t dataSize = (*it).data.size();
+        dataSize = FolderStorage::crlf2lf( (*it).data.data(), dataSize ); // always <=
+        (*it).data.resize( dataSize );
+
+        msg->fromByteArray( (*it).data );
+        // reconstruct as it may be overwritten above
+        msg->setUID(uid);
+        if ( size > 0 && msg->msgSizeServer() == 0 )
+          msg->setMsgSizeServer(size);
+
       } else {
         // Update the body of the retrieved part (the message notifies all observers)
         msg->updateBodyPart( mPartSpecifier, (*it).data );
         msg->setReadyToShow( true );
+        // Update the attachment state, we have to do this for every part as we actually
+        // do not know if the message has no attachment or we simply did not load the header
+        if (msg->attachmentState() != KMMsgHasAttachment)
+          msg->updateAttachmentState();
       }
     } else {
       kdDebug(5006) << "ImapJob::slotGetMessageResult - got no data for " << mPartSpecifier << endl;
@@ -354,7 +432,7 @@ void ImapJob::slotGetBodyStructureResult( KIO::Job * job )
     deleteLater();
     return;
   }
-  KMFolderImap* parent = static_cast<KMFolderImap*>(msg->parent());
+  KMFolderImap* parent = static_cast<KMFolderImap*>(msg->storage());
   if (msg->transferInProgress())
     msg->setTransferInProgress( false );
   KMAcctImap *account = parent->account();
@@ -364,12 +442,11 @@ void ImapJob::slotGetBodyStructureResult( KIO::Job * job )
   }
   ImapAccountBase::JobIterator it = account->findJob( job );
   if ( it == account->jobsEnd() ) return;
-  
+
 
   if (job->error())
   {
-    account->slotSlaveError( account->slave(), job->error(),
-        job->errorText() );
+    account->handleJobError( job, i18n( "Error while retrieving information on the structure of a message." ) );
     return;
   } else {
     if ((*it).data.size() > 0)
@@ -388,7 +465,7 @@ void ImapJob::slotGetBodyStructureResult( KIO::Job * job )
 //-----------------------------------------------------------------------------
 void ImapJob::slotPutMessageDataReq( KIO::Job *job, QByteArray &data )
 {
-  KMAcctImap *account = static_cast<KMFolderImap*>(mDestFolder)->account();
+  KMAcctImap *account = static_cast<KMFolderImap*>(mDestFolder->storage())->account();
   ImapAccountBase::JobIterator it = account->findJob( job );
   if ( it == account->jobsEnd() ) return;
 
@@ -409,14 +486,14 @@ void ImapJob::slotPutMessageDataReq( KIO::Job *job, QByteArray &data )
 void ImapJob::slotPutMessageResult( KIO::Job *job )
 {
   KMMessage *msg = mMsgList.first();
-  KMAcctImap *account = static_cast<KMFolderImap*>(mDestFolder)->account();
+  KMAcctImap *account = static_cast<KMFolderImap*>(mDestFolder->storage())->account();
   ImapAccountBase::JobIterator it = account->findJob( job );
   if ( it == account->jobsEnd() ) return;
-
   if (job->error())
   {
-    account->slotSlaveError( account->slave(), job->error(),
-        job->errorText() );
+    if ( (*it).progressItem )
+      (*it).progressItem->setStatus( i18n("Uploading message data failed.") );
+    account->handlePutError( job, *it, mDestFolder );
     return;
   } else {
     if ( !(*it).msgList.isEmpty() )
@@ -428,6 +505,8 @@ void ImapJob::slotPutMessageResult( KIO::Job *job )
       emit messageStored(msg);
     }
     msg = 0;
+    if ( (*it).progressItem )
+      (*it).progressItem->setStatus( i18n("Uploading message data completed.") );
   }
   if (account->slave()) {
     account->removeJob(it);
@@ -439,7 +518,7 @@ void ImapJob::slotPutMessageResult( KIO::Job *job )
 //-----------------------------------------------------------------------------
 void ImapJob::slotCopyMessageInfoData(KIO::Job * job, const QString & data)
 {
-  KMFolderImap * imapFolder = static_cast<KMFolderImap*>(mDestFolder);
+  KMFolderImap * imapFolder = static_cast<KMFolderImap*>(mDestFolder->storage());
   KMAcctImap *account = imapFolder->account();
   ImapAccountBase::JobIterator it = account->findJob( job );
   if ( it == account->jobsEnd() ) return;
@@ -451,8 +530,8 @@ void ImapJob::slotCopyMessageInfoData(KIO::Job * job, const QString & data)
     QString newUid = data.section(' ', 2, 2);
 
     // get lists of uids
-    QValueList<int> olduids = KMFolderImap::splitSets(oldUid);
-    QValueList<int> newuids = KMFolderImap::splitSets(newUid);
+    QValueList<ulong> olduids = KMFolderImap::splitSets(oldUid);
+    QValueList<ulong> newuids = KMFolderImap::splitSets(newUid);
 
     int index = -1;
     if ( !(*it).msgList.isEmpty() )
@@ -460,7 +539,7 @@ void ImapJob::slotCopyMessageInfoData(KIO::Job * job, const QString & data)
       KMMessage * msg;
       for ( msg = (*it).msgList.first(); msg; msg = (*it).msgList.next() )
       {
-        uint uid = msg->headerField("X-UID").toInt();
+        ulong uid = msg->UID();
         index = olduids.findIndex(uid);
         if (index > -1)
         {
@@ -470,7 +549,7 @@ void ImapJob::slotCopyMessageInfoData(KIO::Job * job, const QString & data)
         }
       }
     } else if (mMsgList.first()) {
-      uint uid = mMsgList.first()->headerField("X-UID").toInt();
+      ulong uid = mMsgList.first()->UID();
       index = olduids.findIndex(uid);
       if (index > -1)
       {
@@ -485,24 +564,22 @@ void ImapJob::slotCopyMessageInfoData(KIO::Job * job, const QString & data)
 //----------------------------------------------------------------------------
 void ImapJob::slotPutMessageInfoData(KIO::Job *job, const QString &data)
 {
-  KMFolderImap * imapFolder = static_cast<KMFolderImap*>(mDestFolder);
+  KMFolderImap * imapFolder = static_cast<KMFolderImap*>(mDestFolder->storage());
   KMAcctImap *account = imapFolder->account();
   ImapAccountBase::JobIterator it = account->findJob( job );
   if ( it == account->jobsEnd() ) return;
 
   if (data.find("UID") != -1)
   {
-    int uid = (data.right(data.length()-4)).toInt();
+    ulong uid = (data.right(data.length()-4)).toInt();
 
     if ( !(*it).msgList.isEmpty() )
     {
       const ulong * sernum = (ulong *)(*it).msgList.last()->getMsgSerNum();
-//      kdDebug(5006) << "insert sernum " << (*it).msgList.last()->getMsgSerNum() << " for " << uid << endl;
       imapFolder->insertUidSerNumEntry(uid, sernum);
     } else if (mMsgList.first())
     {
       const ulong * sernum = (ulong *)mMsgList.first()->getMsgSerNum();
-//      kdDebug(5006) << "insert sernum " << mMsgList.first()->getMsgSerNum() << " for " << uid << endl;
       imapFolder->insertUidSerNumEntry(uid, sernum);
     }
   }
@@ -512,14 +589,13 @@ void ImapJob::slotPutMessageInfoData(KIO::Job *job, const QString &data)
 //-----------------------------------------------------------------------------
 void ImapJob::slotCopyMessageResult( KIO::Job *job )
 {
-  KMAcctImap *account = static_cast<KMFolderImap*>(mDestFolder)->account();
+  KMAcctImap *account = static_cast<KMFolderImap*>(mDestFolder->storage())->account();
   ImapAccountBase::JobIterator it = account->findJob( job );
   if ( it == account->jobsEnd() ) return;
 
   if (job->error())
   {
-    account->slotSlaveError( account->slave(), job->error(),
-        job->errorText() );
+    account->handleJobError( job, i18n("Error while copying messages.") );
     return;
   } else {
     if ( !(*it).msgList.isEmpty() )
@@ -539,13 +615,39 @@ void ImapJob::slotCopyMessageResult( KIO::Job *job )
 //-----------------------------------------------------------------------------
 void ImapJob::execute()
 {
-  init( mType, mSets, static_cast<KMFolderImap*>( mDestFolder ), mMsgList );
+  init( mType, mSets, mDestFolder?
+        dynamic_cast<KMFolderImap*>( mDestFolder->storage() ):0, mMsgList );
 }
 
 //-----------------------------------------------------------------------------
-void ImapJob::expireMessages()
+void ImapJob::setParentFolder( const KMFolderImap* parent )
 {
+  mParentFolder = const_cast<KMFolderImap*>( parent );
+}
+
+//-----------------------------------------------------------------------------
+void ImapJob::slotProcessedSize(KIO::Job * job, KIO::filesize_t processed)
+{
+  KMMessage *msg = mMsgList.first();
+  if (!msg || !job) {
     return;
+  }
+  KMFolderImap* parent = 0;
+  if ( msg->parent() && msg->parent()->folderType() == KMFolderTypeImap )
+    parent = static_cast<KMFolderImap*>(msg->parent()->storage());
+  else if (mDestFolder) // put
+    parent = static_cast<KMFolderImap*>(mDestFolder->storage());
+  if (!parent) return;
+  KMAcctImap *account = parent->account();
+  if ( !account ) return;
+  ImapAccountBase::JobIterator it = account->findJob( job );
+  if ( it == account->jobsEnd() ) return;
+  (*it).done = processed;
+  if ( (*it).progressItem ) {
+    (*it).progressItem->setCompletedItems( processed );
+    (*it).progressItem->updateProgress();
+  }
+  emit progress( (*it).done, (*it).total );
 }
 
 }//namespace KMail
