@@ -15,10 +15,12 @@
 #include "kmfolderindex.h"
 #include "kmfoldermgr.h"
 #include "kmsender.h"
-#include "kmidentity.h"
-#include "identitymanager.h"
-#include "identitycombo.h"
-#include "kfileio.h"
+#include <libkpimidentities/identity.h>
+#include <libkpimidentities/identitymanager.h>
+#include <libkpimidentities/identitycombo.h>
+#include <libkdepim/kfileio.h>
+#include <libkdepim/collectingprocess.h>
+using KPIM::CollectingProcess;
 #include "kmfawidgets.h"
 #include "kmfoldercombobox.h"
 #include "kmmsgbase.h"
@@ -104,7 +106,7 @@ int KMFilterAction::tempOpenFolder(KMFolder* aFolder)
 }
 
 void KMFilterAction::sendMDN( KMMessage * msg, KMime::MDN::DispositionType d,
-			      const QValueList<KMime::MDN::DispositionModifier> & m ) {
+                              const QValueList<KMime::MDN::DispositionModifier> & m ) {
   if ( !msg ) return;
   KMMessage * mdn = msg->createMDN( KMime::MDN::AutomaticAction, d, false, m );
   if ( mdn && !kmkernel->msgSender()->send( mdn, FALSE ) ) {
@@ -307,7 +309,10 @@ bool KMFilterActionWithFolder::folderRemoved( KMFolder* aFolder, KMFolder* aNewF
 {
   if ( aFolder == mFolder ) {
     mFolder = aNewFolder;
-    mFolderName = QString::null;
+    if ( aNewFolder )
+      mFolderName = QString::null;
+    else
+      mFolderName = i18n( "<select a folder>" );
     return TRUE;
   } else
     return FALSE;
@@ -405,24 +410,24 @@ QString KMFilterActionWithCommand::substituteCommandLineArgsFor( KMMessage *aMsg
     if ( (*it) != lastSeen ) {
       KTempFile *tf = new KTempFile();
       if ( tf->status() != 0 ) {
-	tf->close();
-	delete tf;
-	kdDebug(5006) << "KMFilterActionWithCommand: Could not create temp file!" << endl;
-	return QString::null;
+        tf->close();
+        delete tf;
+        kdDebug(5006) << "KMFilterActionWithCommand: Could not create temp file!" << endl;
+        return QString::null;
       }
       tf->setAutoDelete(TRUE);
       aTempFileList.append( tf );
       tempFileName = tf->name();
       if ((*it) == -1)
-        kCStringToFile( aMsg->asString(), tempFileName, //###
+        KPIM::kCStringToFile( aMsg->asString(), tempFileName, //###
                           false, false, false );
       else if (aMsg->numBodyParts() == 0)
-        kByteArrayToFile( aMsg->bodyDecodedBinary(), tempFileName,
+        KPIM::kByteArrayToFile( aMsg->bodyDecodedBinary(), tempFileName,
                           false, false, false );
       else {
-	KMMessagePart msgPart;
+        KMMessagePart msgPart;
         aMsg->bodyPart( (*it), &msgPart );
-        kByteArrayToFile( msgPart.bodyDecodedBinary(), tempFileName,
+        KPIM::kByteArrayToFile( msgPart.bodyDecodedBinary(), tempFileName,
                           false, false, false );
       }
       tf->close();
@@ -479,37 +484,30 @@ KMFilterAction::ReturnCode KMFilterActionWithCommand::genericProcess(KMMessage*
 
   // write message to file
   QString tempFileName = inFile->name();
-  kCStringToFile( aMsg->asString(), tempFileName, //###
-		  false, false, false );
+  KPIM::kCStringToFile( aMsg->asString(), tempFileName, //###
+                  false, false, false );
   inFile->close();
 
-  KProcess shProc;
+  CollectingProcess shProc;
   shProc.setUseShell(true);
   shProc << commandLine;
 
-  // let the kernel collect the output for us:
-  if ( withOutput )
-    QObject::connect( &shProc, SIGNAL(receivedStdout(KProcess*,char*,int)),
-		      kmkernel, SLOT(slotCollectStdOut(KProcess*,char*,int)) );
-
   // run process:
   if ( !shProc.start( KProcess::Block,
-		      withOutput ? KProcess::Stdout : KProcess::NoCommunication ) )
+                      withOutput ? KProcess::Stdout
+                                 : KProcess::NoCommunication ) )
     return ErrorButGoOn;
 
   if ( !shProc.normalExit() || shProc.exitStatus() != 0 ) {
-    // eat the output to avoid acummulation for following processes
-    if ( withOutput ) 
-      kmkernel->getCollectedStdOut( &shProc );
     return ErrorButGoOn;
   }
 
   if ( withOutput ) {
     // read altered message:
-    QByteArray msgText = kmkernel->getCollectedStdOut( &shProc );
+    QByteArray msgText = shProc.collectedStdout();
 
     if ( !msgText.isEmpty() ) {
-    /* If the pipe through alters the message, it could very well 
+    /* If the pipe through alters the message, it could very well
        happen that it no longer has a X-UID header afterwards. That is
        unfortunate, as we need to removed the original from the folder
        using that, and look it up in the message. When the (new) message
@@ -549,7 +547,7 @@ KMFilterAction* KMFilterActionBounce::newAction(void)
 }
 
 KMFilterActionBounce::KMFilterActionBounce()
-  : KMFilterActionWithNone( "bounce", i18n("bounce") )
+  : KMFilterActionWithNone( "bounce", i18n("Bounce") )
 {
 }
 
@@ -584,7 +582,7 @@ KMFilterAction* KMFilterActionSendReceipt::newAction(void)
 }
 
 KMFilterActionSendReceipt::KMFilterActionSendReceipt()
-  : KMFilterActionWithNone( "confirm delivery", i18n("confirm delivery") )
+  : KMFilterActionWithNone( "confirm delivery", i18n("Confirm Delivery") )
 {
 }
 
@@ -620,7 +618,7 @@ KMFilterAction* KMFilterActionTransport::newAction(void)
 }
 
 KMFilterActionTransport::KMFilterActionTransport()
-  : KMFilterActionWithString( "set transport", i18n("set transport to") )
+  : KMFilterActionWithString( "set transport", i18n("Set Transport To") )
 {
 }
 
@@ -651,7 +649,7 @@ KMFilterAction* KMFilterActionReplyTo::newAction(void)
 }
 
 KMFilterActionReplyTo::KMFilterActionReplyTo()
-  : KMFilterActionWithString( "set Reply-To", i18n("set Reply-To to") )
+  : KMFilterActionWithString( "set Reply-To", i18n("Set Reply-To To") )
 {
   mParameter = "";
 }
@@ -687,7 +685,7 @@ KMFilterAction* KMFilterActionIdentity::newAction()
 }
 
 KMFilterActionIdentity::KMFilterActionIdentity()
-  : KMFilterActionWithUOID( "set identity", i18n("set identity to") )
+  : KMFilterActionWithUOID( "set identity", i18n("Set Identity To") )
 {
   mParameter = kmkernel->identityManager()->defaultIdentity().uoid();
 }
@@ -700,21 +698,21 @@ KMFilterAction::ReturnCode KMFilterActionIdentity::process(KMMessage* msg) const
 
 QWidget * KMFilterActionIdentity::createParamWidget( QWidget * parent ) const
 {
-  IdentityCombo * ic = new IdentityCombo( parent );
+  KPIM::IdentityCombo * ic = new KPIM::IdentityCombo( kmkernel->identityManager(), parent );
   ic->setCurrentIdentity( mParameter );
   return ic;
 }
 
 void KMFilterActionIdentity::applyParamWidgetValue( QWidget * paramWidget )
 {
-  IdentityCombo * ic = dynamic_cast<IdentityCombo*>( paramWidget );
+  KPIM::IdentityCombo * ic = dynamic_cast<KPIM::IdentityCombo*>( paramWidget );
   assert( ic );
   mParameter = ic->currentIdentity();
 }
 
 void KMFilterActionIdentity::clearParamWidget( QWidget * paramWidget ) const
 {
-  IdentityCombo * ic = dynamic_cast<IdentityCombo*>( paramWidget );
+  KPIM::IdentityCombo * ic = dynamic_cast<KPIM::IdentityCombo*>( paramWidget );
   assert( ic );
   ic->setCurrentItem( 0 );
   //ic->setCurrentIdentity( kmkernel->identityManager()->defaultIdentity() );
@@ -722,7 +720,7 @@ void KMFilterActionIdentity::clearParamWidget( QWidget * paramWidget ) const
 
 void KMFilterActionIdentity::setParamWidgetValue( QWidget * paramWidget ) const
 {
-  IdentityCombo * ic = dynamic_cast<IdentityCombo*>( paramWidget );
+  KPIM::IdentityCombo * ic = dynamic_cast<KPIM::IdentityCombo*>( paramWidget );
   assert( ic );
   ic->setCurrentIdentity( mParameter );
 }
@@ -769,7 +767,7 @@ KMFilterAction* KMFilterActionSetStatus::newAction()
 }
 
 KMFilterActionSetStatus::KMFilterActionSetStatus()
-  : KMFilterActionWithStringList( "set status", i18n("mark as") )
+  : KMFilterActionWithStringList( "set status", i18n("Mark As") )
 {
   // if you change this list, also update
   // KMFilterActionSetStatus::stati above
@@ -809,8 +807,8 @@ void KMFilterActionSetStatus::argsFromString( const QString argsStr )
   if ( argsStr.length() == 1 ) {
     for ( int i = 0 ; i < StatiCount ; i++ )
       if ( KMMsgBase::statusToStr(stati[i])[0] == argsStr[0] ) {
-	mParameter = *mParameterList.at(i+1);
-	return;
+        mParameter = *mParameterList.at(i+1);
+        return;
       }
   }
   mParameter = *mParameterList.at(0);
@@ -861,7 +859,7 @@ static const int numMDNs = sizeof mdns / sizeof *mdns;
 
 
 KMFilterActionFakeDisposition::KMFilterActionFakeDisposition()
-  : KMFilterActionWithStringList( "fake mdn", i18n("send fake MDN") )
+  : KMFilterActionWithStringList( "fake mdn", i18n("Send Fake MDN") )
 {
   // if you change this list, also update
   // mdns above
@@ -898,8 +896,8 @@ void KMFilterActionFakeDisposition::argsFromString( const QString argsStr )
     }
     for ( int i = 0 ; i < numMDNs ; i++ )
       if ( char(mdns[i]) == argsStr[0] ) { // send
-	mParameter = *mParameterList.at(i+2);
-	return;
+        mParameter = *mParameterList.at(i+2);
+        return;
       }
   }
   mParameter = *mParameterList.at(0);
@@ -935,14 +933,14 @@ KMFilterAction* KMFilterActionRemoveHeader::newAction()
 }
 
 KMFilterActionRemoveHeader::KMFilterActionRemoveHeader()
-  : KMFilterActionWithStringList( "remove header", i18n("remove header") )
+  : KMFilterActionWithStringList( "remove header", i18n("Remove Header") )
 {
   mParameterList << ""
-		 << "Reply-To"
-		 << "Delivered-To"
-		 << "X-KDE-PR-Message"
-		 << "X-KDE-PR-Package"
-		 << "X-KDE-PR-Keywords";
+                 << "Reply-To"
+                 << "Delivered-To"
+                 << "X-KDE-PR-Message"
+                 << "X-KDE-PR-Package"
+                 << "X-KDE-PR-Keywords";
   mParameter = *mParameterList.at(0);
 }
 
@@ -1006,14 +1004,14 @@ private:
 };
 
 KMFilterActionAddHeader::KMFilterActionAddHeader()
-  : KMFilterActionWithStringList( "add header", i18n("add header") )
+  : KMFilterActionWithStringList( "add header", i18n("Add Header") )
 {
   mParameterList << ""
-		 << "Reply-To"
-		 << "Delivered-To"
-		 << "X-KDE-PR-Message"
-		 << "X-KDE-PR-Package"
-		 << "X-KDE-PR-Keywords";
+                 << "Reply-To"
+                 << "Delivered-To"
+                 << "X-KDE-PR-Message"
+                 << "X-KDE-PR-Package"
+                 << "X-KDE-PR-Keywords";
   mParameter = *mParameterList.at(0);
 }
 
@@ -1033,7 +1031,7 @@ QWidget* KMFilterActionAddHeader::createParamWidget( QWidget* parent ) const
   QComboBox *cb = new QComboBox( TRUE, w, "combo" );
   cb->setInsertionPolicy( QComboBox::AtBottom );
   hbl->addWidget( cb, 0 /* stretch */ );
-  QLabel *l = new QLabel( i18n("with value"), w );
+  QLabel *l = new QLabel( i18n("With value:"), w );
   l->setFixedWidth( l->sizeHint().width() );
   hbl->addWidget( l, 0 );
   QLineEdit *le = new KLineEdit( w, "ledit" );
@@ -1138,15 +1136,15 @@ private:
 };
 
 KMFilterActionRewriteHeader::KMFilterActionRewriteHeader()
-  : KMFilterActionWithStringList( "rewrite header", i18n("rewrite header") )
+  : KMFilterActionWithStringList( "rewrite header", i18n("Rewrite Header") )
 {
   mParameterList << ""
-		 << "Subject"
-		 << "Reply-To"
-		 << "Delivered-To"
-		 << "X-KDE-PR-Message"
-		 << "X-KDE-PR-Package"
-		 << "X-KDE-PR-Keywords";
+                 << "Subject"
+                 << "Reply-To"
+                 << "Delivered-To"
+                 << "X-KDE-PR-Message"
+                 << "X-KDE-PR-Package"
+                 << "X-KDE-PR-Keywords";
   mParameter = *mParameterList.at(0);
 }
 
@@ -1158,7 +1156,7 @@ KMFilterAction::ReturnCode KMFilterActionRewriteHeader::process(KMMessage* msg)
   KRegExp3 rx = mRegExp; // KRegExp3::replace is not const.
 
   QString newValue = rx.replace( msg->headerField( mParameter.latin1() ),
-				     mReplacementString );
+                                     mReplacementString );
 
   msg->setHeaderField( mParameter.latin1(), newValue );
   return GoOn;
@@ -1174,14 +1172,14 @@ QWidget* KMFilterActionRewriteHeader::createParamWidget( QWidget* parent ) const
   cb->setInsertionPolicy( QComboBox::AtBottom );
   hbl->addWidget( cb, 0 /* stretch */ );
 
-  QLabel *l = new QLabel( i18n("replace"), w );
+  QLabel *l = new QLabel( i18n("Replace:"), w );
   l->setFixedWidth( l->sizeHint().width() );
   hbl->addWidget( l, 0 );
 
   QLineEdit *le = new KLineEdit( w, "search" );
   hbl->addWidget( le, 1 );
 
-  l = new QLabel( i18n("with"), w );
+  l = new QLabel( i18n("With:"), w );
   l->setFixedWidth( l->sizeHint().width() );
   hbl->addWidget( l, 0 );
 
@@ -1294,7 +1292,7 @@ KMFilterAction* KMFilterActionMove::newAction(void)
 }
 
 KMFilterActionMove::KMFilterActionMove()
-  : KMFilterActionWithFolder( "transfer", i18n("file into folder") )
+  : KMFilterActionWithFolder( "transfer", i18n("File into Folder") )
 {
 }
 
@@ -1330,7 +1328,7 @@ KMFilterAction* KMFilterActionForward::newAction(void)
 }
 
 KMFilterActionForward::KMFilterActionForward()
-  : KMFilterActionWithAddress( "forward", i18n("forward to") )
+  : KMFilterActionWithAddress( "forward", i18n("Forward To") )
 {
 }
 
@@ -1339,6 +1337,11 @@ KMFilterAction::ReturnCode KMFilterActionForward::process(KMMessage* aMsg) const
   if ( mParameter.isEmpty() )
     return ErrorButGoOn;
 
+  // avoid endless loops when this action is used in a filter
+  // which applies to sent messages
+  if ( KMMessage::addressIsInAddressList( mParameter, aMsg->to() ) )
+    return ErrorButGoOn;
+
   // Create the forwarded message by hand to make forwarding of messages with
   // attachments work.
   // Note: This duplicates a lot of code from KMMessage::createForward() and
@@ -1433,7 +1436,7 @@ KMFilterAction* KMFilterActionRedirect::newAction(void)
 }
 
 KMFilterActionRedirect::KMFilterActionRedirect()
-  : KMFilterActionWithAddress( "redirect", i18n("redirect to") )
+  : KMFilterActionWithAddress( "redirect", i18n("Redirect To") )
 {
 }
 
@@ -1443,9 +1446,8 @@ KMFilterAction::ReturnCode KMFilterActionRedirect::process(KMMessage* aMsg) cons
   if ( mParameter.isEmpty() )
     return ErrorButGoOn;
 
-  msg = aMsg->createRedirect();
-  msg->setTo( mParameter );
-
+  msg = aMsg->createRedirect2( mParameter );
+  
   sendMDN( aMsg, KMime::MDN::Dispatched );
 
   if ( !kmkernel->msgSender()->send( msg, FALSE ) ) {
@@ -1474,7 +1476,7 @@ KMFilterAction* KMFilterActionExec::newAction(void)
 }
 
 KMFilterActionExec::KMFilterActionExec()
-  : KMFilterActionWithCommand( "execute", i18n("execute command") )
+  : KMFilterActionWithCommand( "execute", i18n("Execute Command") )
 {
 }
 
@@ -1555,7 +1557,7 @@ KMFilterAction* KMFilterActionExtFilter::newAction(void)
 }
 
 KMFilterActionExtFilter::KMFilterActionExtFilter()
-  : KMFilterActionWithCommand( "filter app", i18n("pipe through") )
+  : KMFilterActionWithCommand( "filter app", i18n("Pipe Through") )
 {
 }
 KMFilterAction::ReturnCode KMFilterActionExtFilter::process(KMMessage* aMsg) const
@@ -1589,7 +1591,7 @@ void KMFilterActionExtFilter::processAsync(KMMessage* aMsg) const
 
   // write message to file
   QString tempFileName = inFile->name();
-  kCStringToFile( aMsg->asString(), tempFileName, //###
+  KPIM::kCStringToFile( aMsg->asString(), tempFileName, //###
       false, false, false );
   inFile->close();
 
@@ -1655,7 +1657,7 @@ const QString KMFilterActionWithTest::argsAsString() const
 
 
 KMFilterActionExecSound::KMFilterActionExecSound()
-  : KMFilterActionWithTest( "play sound", i18n("play a sound") )
+  : KMFilterActionWithTest( "play sound", i18n("Play Sound") )
 {
 }
 
