@@ -24,6 +24,7 @@
 #include <kstatusbar.h>
 #include <kwin.h>
 #include <kconfig.h>
+#include <kstdaction.h>
 
 #include <qcheckbox.h>
 #include <qlayout.h>
@@ -55,15 +56,18 @@ KMFldSearch::KMFldSearch(KMMainWidget* w, const char* name,
   mStopped(false),
   mCloseRequested(false),
   mSortColumn(0),
-#if QT_VERSION >= 0x030200
   mSortOrder(Ascending),
-#endif
   mFolder(0),
   mTimer(new QTimer(this)),
   mLastFocus(0),
   mKMMainWidget(w)
 {
+#if !KDE_IS_VERSION( 3, 2, 91 )
+  // HACK - KWin keeps all dialogs on top of their mainwindows, but that's probably
+  // wrong (#76026), and should be done only for modals. CVS HEAD should get
+  // proper fix in KWin (l.lunak@kde.org)
   XDeleteProperty( qt_xdisplay(), winId(), XA_WM_TRANSIENT_FOR );
+#endif
   KWin::setIcons(winId(), kapp->icon(), kapp->miniIcon());
 
   KConfig* config = KMKernel::config();
@@ -91,7 +95,9 @@ KMFldSearch::KMFldSearch(KMMainWidget* w, const char* name,
 
   connect(mCbxFolders, SIGNAL(activated(int)),
           this, SLOT(slotFolderActivated(int)));
-
+  connect(mCbxFolders, SIGNAL(activated(int)),
+          this, SLOT(slotFilterAvailableRules()));
+    
   mChkSubFolders = new QCheckBox(i18n("I&nclude sub-folders"), searchWidget);
   mChkSubFolders->setChecked(true);
   hbl->addWidget(mChkSubFolders);
@@ -104,7 +110,9 @@ KMFldSearch::KMFldSearch(KMMainWidget* w, const char* name,
   mPatternEdit->setFrameStyle( QFrame::NoFrame | QFrame::Plain );
   mPatternEdit->setInsideMargin( 0 );
   mSearchPattern = new KMSearchPattern();
-  KMFolderSearch *searchFolder = dynamic_cast<KMFolderSearch*>(curFolder);
+  KMFolderSearch *searchFolder = 0;
+  if (curFolder)
+      searchFolder = dynamic_cast<KMFolderSearch*>(curFolder->storage());
   if (searchFolder) {
       KConfig config(curFolder->location());
       KMFolder *root = searchFolder->search()->root();
@@ -125,19 +133,8 @@ KMFldSearch::KMFldSearch(KMMainWidget* w, const char* name,
   if ( list )
       object = list->first();
   delete list;
-  if (!searchFolder && object && object->inherits( "QComboBox" )) {
-      QComboBox *combo = (QComboBox*)object;
-      combo->setCurrentText("Subject");
-  }
-  list = mPatternEdit->queryList( 0, "mRuleValue" );
-  object = 0;
-  if ( list )
-      object = list->first();
-  delete list;
-  if (object && object->inherits( "QWidget" )) {
-      QWidget *widget = (QComboBox*)object;
-      widget->setFocus();
-  }
+  if (!searchFolder && object && ::qt_cast<QComboBox*>(object))
+      static_cast<QComboBox*>(object)->setCurrentText("Subject");
 
   vbl->addWidget( mPatternEdit );
 
@@ -191,7 +188,7 @@ KMFldSearch::KMFldSearch(KMMainWidget* w, const char* name,
   hbl2->addWidget(mSearchFolderLbl);
   mSearchFolderEdt = new KLineEdit(searchWidget);
   if (searchFolder)
-    mSearchFolderEdt->setText(searchFolder->name());
+    mSearchFolderEdt->setText(searchFolder->folder()->name());
   else
     mSearchFolderEdt->setText(i18n("Last Search"));
 
@@ -234,6 +231,21 @@ KMFldSearch::KMFldSearch(KMMainWidget* w, const char* name,
   connect(this, SIGNAL(user2Clicked()), SLOT(slotStop()));
   connect(this, SIGNAL(finished()), this, SLOT(deleteLater()));
 
+  // give focus to the value field of the first search rule
+  object = mPatternEdit->child( "regExpLineEdit" );
+  if ( object && object->isWidgetType() ) {
+      static_cast<QWidget*>(object)->setFocus();
+      //kdDebug(5006) << "KMFldSearch: focus has been given to widget "
+      //              << object->name() << endl;
+  }
+  else
+      kdDebug(5006) << "KMFldSearch: regExpLineEdit not found" << endl;
+
+  // Remove the <message> and <body> rule fields when searching imap folders
+  slotFilterAvailableRules();
+  connect(mChkbxAllFolders, SIGNAL(toggled(bool)),
+          this, SLOT(slotFilterAvailableRules()));
+
   //set up actions
   KActionCollection *ac = actionCollection();
   mReplyAction = new KAction( i18n("&Reply..."), "mail_reply", 0, this,
@@ -259,13 +271,11 @@ KMFldSearch::KMFldSearch(KMMainWidget* w, const char* name,
 					SLOT(slotForwardAttachedMsg()), ac,
 					"search_message_forward_as_attachment" );
   mForwardActionMenu->insert( mForwardAttachedAction );
-  mSaveAsAction = new KAction( i18n("Save &As..."), "filesave", 0,
-                               this, SLOT(slotSaveMsg()), ac, "search_file_save_as" );
+  mSaveAsAction = KStdAction::saveAs( this, SLOT(slotSaveMsg()), ac, "search_file_save_as" );
   mSaveAtchAction = new KAction( i18n("Save Attachments..."), "attach", 0,
                                  this, SLOT(slotSaveAttachments()), ac, "search_save_attachments" );
 
-  mPrintAction = new KAction( i18n( "&Print..." ), "fileprint", 0, this,
-			      SLOT(slotPrintMsg()), ac, "search_print" );
+  mPrintAction = KStdAction::print( this, SLOT(slotPrintMsg()), ac, "search_print" );
   mClearAction = new KAction( i18n("Clear Selection"), 0, 0, this,
 			      SLOT(slotClearSelection()), ac, "search_clear_selection" );
   connect(mTimer, SIGNAL(timeout()), this, SLOT(updStatus()));
@@ -356,6 +366,20 @@ void KMFldSearch::slotFolderActivated(int /*nr*/)
 }
 
 
+//-----------------------------------------------------------------------------
+void KMFldSearch::slotFilterAvailableRules()
+{
+    static bool lastHeadersOnly = false;
+    KMFolder* folder = mCbxFolders->getFolder();
+    bool headersOnly = folder->folderType() == KMFolderTypeImap;
+    if (mChkbxAllFolders->isChecked())
+	headersOnly = false;
+    if (headersOnly != lastHeadersOnly)
+	mPatternEdit->setHeadersOnly( headersOnly );
+    lastHeadersOnly = headersOnly;
+}
+
+
 //-----------------------------------------------------------------------------
 void KMFldSearch::activateFolder(KMFolder *curFolder)
 {
@@ -379,9 +403,7 @@ void KMFldSearch::slotSearch()
     mLbxMatches->clear();
 
     mSortColumn = mLbxMatches->sortColumn();
-#if QT_VERSION >= 0x030200
     mSortOrder = mLbxMatches->sortOrder();
-#endif
     mLbxMatches->setSorting(-1);
     mLbxMatches->setShowSortIndicator(false);
 
@@ -396,7 +418,7 @@ void KMFldSearch::slotSearch()
       int count = 0;
       KMFolder *folder;
       while ((folder = mgr->find(fullName))) {
-        if (folder->inherits("KMFolderSearch"))
+        if (folder->storage()->inherits("KMFolderSearch"))
           break;
         fullName = QString("%1 %2").arg(baseName).arg(++count);
       }
@@ -405,7 +427,7 @@ void KMFldSearch::slotSearch()
         folder = mgr->createFolder(fullName, FALSE, KMFolderTypeSearch,
             &mgr->dir());
 
-      mFolder = (KMFolderSearch*)folder;
+      mFolder = dynamic_cast<KMFolderSearch*>( folder->storage() );
     }
     mFolder->stopSearch();
     disconnect(mFolder, SIGNAL(msgAdded(int)),
@@ -434,9 +456,9 @@ void KMFldSearch::slotSearch()
     mFolder->setSearch(search);
     enableGUI();
 
-    if (mFolder && !mFolders.contains(mFolder.operator->())) {
+    if (mFolder && !mFolders.contains(mFolder.operator->()->folder())) {
 	mFolder->open();
-	mFolders.append(mFolder.operator->());
+	mFolders.append(mFolder.operator->()->folder());
     }
     mTimer->start(200);
 }
@@ -453,9 +475,7 @@ void KMFldSearch::searchDone()
     if (mCloseRequested)
 	close();
 
-#if QT_VERSION >= 0x030200
     mLbxMatches->setSorting(mSortColumn, mSortOrder == Ascending);
-#endif
     mLbxMatches->setShowSortIndicator(true);
 }
 
@@ -471,7 +491,7 @@ void KMFldSearch::slotAddMsg(int idx)
 	mFolders.append(pFolder);
 	pFolder->open();
     }
-    if(pFolder->type() == KFolderTreeItem::SentMail)
+    if(pFolder->whoField() == "To")
         from = msg->to();
     else
         from = msg->from();
@@ -527,6 +547,7 @@ void KMFldSearch::closeEvent(QCloseEvent *e)
 	//Cancel search in progress by setting the search folder search to
 	//the null search
 	mFolder->setSearch(new KMSearch());
+	QTimer::singleShot(0, this, SLOT(slotClose()));
     } else {
 	KDialogBase::closeEvent(e);
     }
@@ -541,7 +562,7 @@ void KMFldSearch::updateCreateButton( const QString &s)
 //-----------------------------------------------------------------------------
 void KMFldSearch::renameSearchFolder()
 {
-    if (mFolder && (mFolder->name() !=mSearchFolderEdt->text())) {
+    if (mFolder && (mFolder->folder()->name() != mSearchFolderEdt->text())) {
 	int i = 1;
 	QString name =  mSearchFolderEdt->text();
 	while (i < 100) {
@@ -561,7 +582,7 @@ void KMFldSearch::openSearchFolder()
 {
     renameSearchFolder();
     KMFolderTree *folderTree = mKMMainWidget->folderTree();
-    QListViewItem *index = folderTree->indexOfFolder((KMFolder*)mFolder);
+    QListViewItem *index = folderTree->indexOfFolder(mFolder->folder());
     if (index) {
 	folderTree->ensureItemVisible(index);
 	folderTree->doFolderSelected(index);
@@ -572,7 +593,7 @@ void KMFldSearch::openSearchFolder()
 //-----------------------------------------------------------------------------
 void KMFldSearch::folderInvalidated(KMFolder *folder)
 {
-    if (folder == mFolder) {
+    if (folder->storage() == mFolder) {
 	mLbxMatches->clear();
 	if (mFolder->search())
 	    connect(mFolder->search(), SIGNAL(finished(bool)),
@@ -611,10 +632,8 @@ void KMFldSearch::enableGUI()
     KMSearch const *search = (mFolder) ? (mFolder->search()) : 0;
     bool searching = (search) ? (search->running()) : false;
     actionButton(KDialogBase::Close)->setEnabled(!searching);
-    if (mChkbxSpecificFolders->isChecked()) {
-	mCbxFolders->setEnabled(!searching);
-	mChkSubFolders->setEnabled(!searching);
-    }
+    mCbxFolders->setEnabled(!searching);
+    mChkSubFolders->setEnabled(!searching);
     mChkbxAllFolders->setEnabled(!searching);
     mChkbxSpecificFolders->setEnabled(!searching);
     mPatternEdit->setEnabled(!searching);
