@@ -1,4 +1,5 @@
-/* Virtual base class for mail folder
+/* -*- mode: C++ -*-
+ * Virtual base class for mail folder
  *
  * Author: Stefan Taferner <taferner@kde.org>
  * This code is under GPL
@@ -16,6 +17,8 @@
 #include "kmglobal.h"
 #include "folderjob.h"
 using KMail::FolderJob;
+#include "mailinglist-magic.h"
+using KMail::MailingList;
 
 #include "mimelib/string.h"
 
@@ -29,6 +32,8 @@ class KMAcctList;
 class KMMsgDict;
 class KMMsgDictREntry;
 class QTimer;
+class FolderStorage;
+class KMFolderTreeItem;
 
 namespace KMail {
    class AttachmentStrategy;
@@ -57,92 +62,106 @@ public:
   /** Usually a parent is given. But in some cases there is no
     fitting parent object available. Then the name of the folder
     is used as the absolute path to the folder file. */
-  KMFolder(KMFolderDir* parent=0, const QString& name=QString::null);
-  virtual ~KMFolder();
+  KMFolder( KMFolderDir* parent, const QString& name,
+            KMFolderType aFolderType );
+  ~KMFolder();
+
+  /** This is used by the storage to read the folder specific configuration */
+  void readConfig( KConfig* config );
+
+  /** This is used by the storage to save the folder specific configuration */
+  void writeConfig( KConfig* config ) const;
+
+  FolderStorage* storage() { return mStorage; }
+  /** if the folder is const, the storage should be as well */
+  const FolderStorage* storage() const { return mStorage; }
 
   /** Returns the type of this folder */
-  virtual KMFolderType folderType() const { return KMFolderTypeUnknown; }
+  KMFolderType folderType() const;
 
   /** Returns the filename of the folder (reimplemented in KMFolderImap) */
-  virtual QString fileName() const { return name(); }
+  QString fileName() const;
 
   /** Returns full path to folder file */
   QString location() const;
 
   /** Returns full path to index file */
-  virtual QString indexLocation() const = 0;
+  QString indexLocation() const;
 
   /** Returns full path to sub directory file */
   QString subdirLocation() const;
 
   /** Returns the folder directory associated with this node or
       0 if no such directory exists */
-  virtual KMFolderDir* child() const
+  KMFolderDir* child() const
     { return mChild; }
 
   /** Create a child folder directory and associates it with this folder */
-  virtual KMFolderDir* createChildFolder();
+  KMFolderDir* createChildFolder();
 
   /** Set the folder directory associated with this node */
-  virtual void setChild( KMFolderDir* aChild )
-    { mChild = aChild; }
+  void setChild( KMFolderDir* aChild );
 
   /** Returns, if the folder can't contain mails, but only subfolder */
-  virtual bool noContent() const
-    { return mNoContent; }
+  bool noContent() const;
 
   /** Specify, that the folder can't contain mails. */
-  virtual void setNoContent(bool aNoContent)
-    { mNoContent = aNoContent; }
+  void setNoContent(bool aNoContent);
+
+  /** Returns, if the folder can't have children */
+  bool noChildren() const;
+
+  /** Specify, that the folder can't have children */
+  void setNoChildren(bool aNoChildren);
 
   /** Read message at given index. Indexing starts at zero */
-  virtual KMMessage* getMsg(int idx);
+  KMMessage* getMsg(int idx);
 
   /** Replace KMMessage with KMMsgInfo and delete KMMessage  */
-  virtual KMMsgInfo* unGetMsg(int idx);
+  KMMsgInfo* unGetMsg(int idx);
 
   /** Checks if the message is already "gotten" with getMsg */
-  virtual bool isMessage(int idx);
+  bool isMessage(int idx);
 
   /** Read a message and return a referece to a string */
-  virtual QCString& getMsgString(int idx, QCString& mDest) = 0;
+  QCString& getMsgString(int idx, QCString& mDest);
 
   /** Read a message and returns a DwString */
-  virtual DwString getDwString(int idx) = 0;
+  DwString getDwString(int idx);
 
   /**
    * Removes and deletes all jobs associated with the particular message
    */
-  virtual void ignoreJobsForMessage( KMMessage* );
+  void ignoreJobsForMessage( KMMessage* );
 
   /**
    * These methods create respective FolderJob (You should derive FolderJob
    * for each derived KMFolder).
    */
-  virtual FolderJob* createJob( KMMessage *msg, FolderJob::JobType jt = FolderJob::tGetMessage,
-                                KMFolder *folder = 0, QString partSpecifier = QString::null, 
-                                const AttachmentStrategy *as = 0 ) const;
-  virtual FolderJob* createJob( QPtrList<KMMessage>& msgList, const QString& sets,
-                                FolderJob::JobType jt = FolderJob::tGetMessage,
-                                KMFolder *folder = 0 ) const;
+  FolderJob* createJob( KMMessage *msg, FolderJob::JobType jt = FolderJob::tGetMessage,
+                        KMFolder *folder = 0, QString partSpecifier = QString::null,
+                        const AttachmentStrategy *as = 0 ) const;
+  FolderJob* createJob( QPtrList<KMMessage>& msgList, const QString& sets,
+                        FolderJob::JobType jt = FolderJob::tGetMessage,
+                        KMFolder *folder = 0 ) const;
 
   /** Provides access to the basic message fields that are also stored
     in the index. Whenever you only need subject, from, date, status
     you should use this method instead of getMsg() because getMsg()
     will load the message if necessary and this method does not. */
-  virtual const KMMsgBase* getMsgBase(int idx) const = 0;
-  virtual KMMsgBase* getMsgBase(int idx) = 0;
+  const KMMsgBase* getMsgBase(int idx) const;
+  KMMsgBase* getMsgBase(int idx);
 
   /** Same as getMsgBase(int). */
-  virtual const KMMsgBase* operator[](int idx) const { return getMsgBase(idx); }
+  const KMMsgBase* operator[](int idx) const;
 
   /** Same as getMsgBase(int). This time non-const. */
-  virtual KMMsgBase* operator[](int idx) { return getMsgBase(idx); }
+  KMMsgBase* operator[](int idx);
 
   /** Detach message from this folder. Usable to call addMsg() afterwards.
     Loads the message if it is not loaded up to now. */
-  virtual KMMessage* take(int idx);
-  virtual void take(QPtrList<KMMessage> msgList);
+  KMMessage* take(int idx);
+  void take(QPtrList<KMMessage> msgList);
 
   /** Add the given message to the folder. Usually the message
     is added at the end of the folder. Returns zero on success and
@@ -150,127 +169,126 @@ public:
     is stored in index_return if given.
     Please note that the message is added as is to the folder and the folder
     takes ownership of the message (deleting it in the destructor).*/
-  virtual int addMsg(KMMessage* msg, int* index_return = 0) = 0;
+  int addMsg(KMMessage* msg, int* index_return = 0);
 
   /** (Note(bo): This needs to be fixed better at a later point.)
       This is overridden by dIMAP because addMsg strips the X-UID
       header from the mail. */
-  virtual int addMsgKeepUID(KMMessage* msg, int* index_return = 0) {
-    return addMsg(msg, index_return);
-  }
+  int addMsgKeepUID(KMMessage* msg, int* index_return = 0);
 
   /** Called by derived classes implementation of addMsg.
       Emits msgAdded signals */
   void emitMsgAddedSignals(int idx);
 
-  /** Returns FALSE, if the message has to be retrieved from an IMAP account
-   * first. In this case this function does this and cares for the rest */
-  virtual bool canAddMsgNow(KMMessage* aMsg, int* aIndex_ret);
-
   /** Remove (first occurrence of) given message from the folder. */
-  virtual void removeMsg(int i, bool imapQuiet = FALSE);
-  virtual void removeMsg(QPtrList<KMMessage> msgList, bool imapQuiet = FALSE);
+  void removeMsg(int i, bool imapQuiet = FALSE);
+  void removeMsg(QPtrList<KMMessage> msgList, bool imapQuiet = FALSE);
 
   /** Delete messages in the folder that are older than days. Return the
    * number of deleted messages. */
-  virtual int expungeOldMsg(int days);
+  int expungeOldMsg(int days);
 
   /** Detaches the given message from it's current folder and
     adds it to this folder. Returns zero on success and an errno error
     code on failure. The index of the new message is stored in index_return
     if given. */
-  virtual int moveMsg(KMMessage* msg, int* index_return = 0);
-  virtual int moveMsg(QPtrList<KMMessage>, int* index_return = 0);
+  int moveMsg(KMMessage* msg, int* index_return = 0);
+  int moveMsg(QPtrList<KMMessage>, int* index_return = 0);
 
   /** Returns the index of the given message or -1 if not found. */
-  virtual int find(const KMMsgBase* msg) const = 0;
+  int find(const KMMsgBase* msg) const;
   int find( const KMMessage * msg ) const;
 
   /** Number of messages in this folder. */
-  virtual int count(bool cache = false) const;
+  int count(bool cache = false) const;
 
   /** Number of new or unread messages in this folder. */
-  virtual int countUnread();
+  int countUnread();
 
   /** Number of new or unread messages in this folder and all folders
       contained by this folder */
-  virtual int countUnreadRecursive();
+  int countUnreadRecursive();
 
   /** Called by KMMsgBase::setStatus when status of a message has changed
       required to keep the number unread messages variable current. */
-  virtual void msgStatusChanged( const KMMsgStatus oldStatus,
-                                 const KMMsgStatus newStatus,
-				 int idx);
+  void msgStatusChanged( const KMMsgStatus oldStatus,
+                         const KMMsgStatus newStatus,
+                         int idx);
 
   /** Open folder for access. Does not work if the parent is not set.
     Does nothing if the folder is already opened. To reopen a folder
     call close() first.
     Returns zero on success and an error code equal to the c-library
     fopen call otherwise (errno). */
-  virtual int open() = 0;
+  int open();
 
   /** Check folder for permissions
     Returns zero if readable and writable. */
-  virtual int canAccess() = 0;
+  int canAccess();
 
   /** Close folder. If force is TRUE the files are closed even if
     others still use it (e.g. other mail reader windows). */
-  virtual void close(bool force=FALSE) = 0;
+  void close(bool force=FALSE);
 
   /** fsync buffers to disk */
-  virtual void sync() = 0;
+  void sync();
 
   /** Test if folder is opened. */
-  bool isOpened() const { return (mOpenCount>0); }
+  bool isOpened() const;
 
   /** Mark all new messages as unread. */
-  virtual void markNewAsUnread();
+  void markNewAsUnread();
 
   /** Mark all new and unread messages as read. */
-  virtual void markUnreadAsRead();
+  void markUnreadAsRead();
 
   /** Create a new folder with the name of this object and open it.
       Returns zero on success and an error code equal to the
       c-library fopen call otherwise. */
-  virtual int create(bool imap = FALSE) = 0;
+  int create(bool imap = FALSE);
 
   /** Removes the folder physically from disk and empties the contents
     of the folder in memory. Note that the folder is closed during this
     process, whether there are others using it or not.
     @see KMFolder::removeContents */
-  virtual int remove();
+  void remove();
 
   /** Delete entire folder. Forces a close *but* opens the
     folder again afterwards. Returns errno(3) error code or zero on
     success.  @see KMFolder::expungeContents */
-  virtual int expunge();
+  int expunge();
 
-  /** Remove deleted messages from the folder. Returns zero on success
-    and an errno on failure. */
-  virtual int compact() = 0;
+  enum CompactOptions { CompactLater, CompactNow, CompactSilentlyNow };
+  /**
+   * Compact this folder. Options:
+   * CompactLater: schedule it as a background task
+   * CompactNow: do it now, and inform the user of the result (manual compaction)
+   * CompactSilentlyNow: do it now, and keep silent about it (e.g. for outbox)
+   */
+  void compact( CompactOptions options );
 
   /** Physically rename the folder. Returns zero on success and an errno
     on failure. */
-  virtual int rename(const QString& newName, KMFolderDir *aParent = 0);
+  int rename(const QString& newName, KMFolderDir *aParent = 0);
 
   /** Returns TRUE if a table of contents file is automatically created. */
-  bool autoCreateIndex() const { return mAutoCreateIndex; }
+  bool autoCreateIndex() const;
 
   /** Allow/disallow automatic creation of a table of contents file.
     Default is TRUE. */
-  virtual void setAutoCreateIndex(bool);
+  void setAutoCreateIndex(bool);
 
   /** Returns TRUE if the table of contents is dirty. This happens when
     a message is deleted from the folder. The toc will then be re-created
     when the folder is closed. */
-  bool dirty() const { return mDirty; }
+  bool dirty() const;
 
   /** Change the dirty flag. */
   void setDirty(bool f);
 
   /** Returns TRUE if the folder contains deleted messages */
-  bool needsCompacting() const { return needsCompact; }
-  virtual void setNeedsCompacting(bool f) { needsCompact = f; }
+  bool needsCompacting() const;
+  void setNeedsCompacting(bool f);
 
   /** If set to quiet the folder will not emit msgAdded(idx) signal.
     This is necessary because adding the messages to the listview
@@ -280,10 +298,10 @@ public:
     we only download headers it becomes a bottleneck. We therefore
     set the folder quiet() and rebuild the listview completely once
     the complete folder has been checked. */
-  virtual void quiet(bool beQuiet);
+  void quiet(bool beQuiet);
 
   /** Is the folder read-only? */
-  virtual bool isReadOnly() const = 0;
+  bool isReadOnly() const;
 
   /** Returns TRUE if the folder is a kmail system folder. These are
     the folders 'outbox', 'sent', 'trash'. The name of these
@@ -294,139 +312,129 @@ public:
 
   /** Returns the label of the folder for visualization. */
   virtual QString label() const;
-  void setLabel(const QString& lbl) { mLabel = lbl; }
+  void setLabel( const QString& l ) { mLabel = l; }
+
+  /** Set the label that is used as a system default */
+  virtual QString systemLabel() const { return mSystemLabel; }
+  void setSystemLabel( const QString& l ) { mSystemLabel = l; }
+
+  /** URL of the node for visualization purposes. */
+  virtual QString prettyURL() const;
 
   /** Type of the folder. Inherited. */
-  virtual const char* type() const;
-
-  virtual QCString protocol() const = 0;
+  const char* type() const;
 
   /** Returns TRUE if accounts are associated with this folder. */
-  bool hasAccounts() const { return (mAcctList != 0); }
+  bool hasAccounts() const;
 
   /** Returns TRUE if this folder is associated with a mailing-list. */
-  void setMailingList(bool enabled)
-  { mMailingListEnabled = enabled; writeConfig(); }
-  bool isMailingList() const { return mMailingListEnabled; }
+  void setMailingListEnabled( bool enabled );
+  bool isMailingListEnabled() const { return mMailingListEnabled; }
 
-  void setMailingListPostAddress(const QString &address)
-  { mMailingListPostingAddress = address; writeConfig(); }
-  QString mailingListPostAddress() const
-  { return mMailingListPostingAddress; }
-
-  void setMailingListAdminAddress(const QString &address)
-  { mMailingListAdminAddress = address; writeConfig(); }
-  QString mailingListAdminAddress() const
-  { return mMailingListAdminAddress; }
+  void setMailingList( const MailingList& mlist );
+  MailingList mailingList() const
+  { return mMailingList; }
+  QString mailingListPostAddress() const;
 
   void setIdentity(uint identity);
   uint identity() const { return mIdentity; }
 
   /** Get / set the name of the field that is used for the Sender/Receiver column in the headers (From/To) */
   QString whoField() const { return mWhoField; }
-  void setWhoField(const QString& aWhoField) { mWhoField = aWhoField; /*writeConfig();*/ }
+  void setWhoField(const QString& aWhoField);
 
   /** Get / set the user-settings for the WhoField (From/To/Empty) */
   QString userWhoField(void) { return mUserWhoField; }
   void setUserWhoField(const QString &whoField,bool writeConfig=true);
 
   /** A cludge to help make sure the count of unread messges is kept in sync */
-  virtual void correctUnreadMsgsCount();
+  void correctUnreadMsgsCount();
 
   /** Returns a string that can be used to identify this folder */
-  virtual QString idString() const;
+  QString idString() const;
 
   /**
    * Set whether this folder automatically expires messages.
    */
-  void
-  setAutoExpire(bool enabled) {
-    expireMessages = enabled;
-    writeConfig();
-  }
+  void setAutoExpire(bool enabled);
 
   /**
    * Does this folder automatically expire old messages?
    */
-  virtual bool    isAutoExpire() const { return expireMessages; }
+  bool isAutoExpire() const { return mExpireMessages; }
 
   /**
    * Set the maximum age for unread messages in this folder.
    * Age should not be negative. Units are set using
    * setUnreadExpireUnits().
    */
-  void
-  setUnreadExpireAge(int age) {
-    if (age >= 0) {
-      unreadExpireAge = age;
-      writeConfig();
-    }
-  }
+  void setUnreadExpireAge(int age);
 
   /**
    * Set units to use for expiry of unread messages.
    * Values are 1 = days, 2 = weeks, 3 = months.
    */
-  void
-  setUnreadExpireUnits(ExpireUnits units) {
-    if (units >= expireNever && units < expireMaxUnits) {
-      unreadExpireUnits = units;
-    }
-  }
+  void setUnreadExpireUnits(ExpireUnits units);
 
   /**
    * Set the maximum age for read messages in this folder.
    * Age should not be negative. Units are set using
    * setReadExpireUnits().
    */
-  void
-  setReadExpireAge(int age) {
-    if (age >= 0) {
-      readExpireAge = age;
-      writeConfig();
-    }
-  }
+  void setReadExpireAge(int age);
 
   /**
    * Set units to use for expiry of read messages.
    * Values are 1 = days, 2 = weeks, 3 = months.
    */
-  void
-  setReadExpireUnits(ExpireUnits units) {
-    if (units >= expireNever && units <= expireMaxUnits) {
-      readExpireUnits = units;
-    }
-  }
+  void setReadExpireUnits(ExpireUnits units);
 
   /**
    * Get the age at which unread messages are expired.
    * Units are determined by getUnreadExpireUnits().
    */
-  int getUnreadExpireAge() const { return unreadExpireAge; }
+  int getUnreadExpireAge() const { return mUnreadExpireAge; }
 
   /**
    * Get the age at which read messages are expired.
    * Units are determined by getReadExpireUnits().
    */
-  int getReadExpireAge() const { return readExpireAge; }
+  int getReadExpireAge() const { return mReadExpireAge; }
 
   /**
    * Units getUnreadExpireAge() is returned in.
    * 1 = days, 2 = weeks, 3 = months.
    */
-  ExpireUnits getUnreadExpireUnits() const { return unreadExpireUnits; }
+  ExpireUnits getUnreadExpireUnits() const { return mUnreadExpireUnits; }
 
   /**
    * Units getReadExpireAge() is returned in.
    * 1 = days, 2 = weeks, 3 = months.
    */
-  ExpireUnits getReadExpireUnits() const { return readExpireUnits; }
+  ExpireUnits getReadExpireUnits() const { return mReadExpireUnits; }
 
-  void expireOldMessages();
+  enum ExpireAction { ExpireDelete, ExpireMove };
+  /**
+   * What should expiry do? Delete or move to another folder?
+   */
+  ExpireAction expireAction() const { return mExpireAction; }
+  void setExpireAction( ExpireAction a );
+
+  /**
+   * If expiry should move to folder, return the ID of that folder
+   */
+  QString expireToFolderId() const { return mExpireToFolderId; }
+  void setExpireToFolderId( const QString& id );
+
+  /**
+   * Expire old messages in this folder.
+   * If immediate is true, do it immediately; otherwise schedule it for later
+   */
+  void expireOldMessages( bool immediate );
 
   /** Write index to index-file. Returns 0 on success and errno error on
     failure. */
-  virtual int writeIndex( bool createEmptyIndex = false ) = 0;
+  int writeIndex( bool createEmptyIndex = false );
 
   /** Inserts messages into the message dictionary.  Might be called
     during kernel initialization. */
@@ -445,20 +453,20 @@ public:
   void setRDict(KMMsgDictREntry *rentry);
 
   /** Returns the reverse-dictionary for this folder. */
-  KMMsgDictREntry *rDict() const { return mRDict; }
+  KMMsgDictREntry *rDict() const;
 
   /** Set the status of the message at index @p idx to @p status. */
-  virtual void setStatus(int idx, KMMsgStatus status, bool toggle=false);
+  void setStatus(int idx, KMMsgStatus status, bool toggle=false);
 
   /** Set the status of the message(s) in the QValueList @p ids to @p status. */
-  virtual void setStatus(QValueList<int>& ids, KMMsgStatus status, bool toggle=false);
+  void setStatus(QValueList<int>& ids, KMMsgStatus status, bool toggle=false);
 
   /** Icon related methods */
-  virtual bool useCustomIcons() const { return mUseCustomIcons; }
-  virtual void setUseCustomIcons(bool useCustomIcons) { mUseCustomIcons = useCustomIcons; }
-  virtual QString normalIconPath() const { return mNormalIconPath; }
-  virtual QString unreadIconPath() const { return mUnreadIconPath; }
-  virtual void setIconPaths(const QString &normalPath, const QString &unreadPath);
+  bool useCustomIcons() const { return mUseCustomIcons; }
+  void setUseCustomIcons(bool useCustomIcons) { mUseCustomIcons = useCustomIcons; }
+  QString normalIconPath() const { return mNormalIconPath; }
+  QString unreadIconPath() const { return mUnreadIconPath; }
+  void setIconPaths(const QString &normalPath, const QString &unreadPath);
 
   void removeJobs();
 
@@ -467,6 +475,28 @@ public:
   */
   static size_t crlf2lf( char* str, const size_t strLen );
 
+  void daysToExpire( int& unreadDays, int& readDays );
+
+  /**
+   * If this folder has a special trash folder set, return it. Otherwise
+   * return 0.
+   */
+  KMFolder* trashFolder() const;
+
+  /**
+   * Returns true if the replies to mails from this folder should be
+   * put in the same folder.
+   */
+  bool putRepliesInSameFolder() const { return mPutRepliesInSameFolder; }
+  void setPutRepliesInSameFolder( bool b ) { mPutRepliesInSameFolder = b; }
+
+  /**
+   * Returns true if the user doesn't want to get notified about new mail
+   * in this folder.
+   */
+  bool ignoreNewMail() const { return mIgnoreNewMail; }
+  void setIgnoreNewMail( bool b ) { mIgnoreNewMail = b; }
+
 signals:
   /** Emitted when the status, name, or associated accounts of this
     folder changed. */
@@ -478,7 +508,7 @@ signals:
 
   /** Emitted after an expunge. If not quiet, changed() will be
       emmitted first. */
-  void expunged();
+  void expunged( KMFolder* );
 
   /** Emitted when the icon paths are set. */
   void iconsChanged();
@@ -509,128 +539,59 @@ signals:
   /** Emitted when number of unread messages has changed. */
   void numUnreadMsgsChanged( KMFolder* );
 
-  // Emitted by KMFolderCachedIMAP to signal syncing
-  void syncRunning( KMFolder*, bool );
+  /** Emitted when a folder was removed */
+  void removed(KMFolder*, bool);
 
 public slots:
   /** Incrementally update the index if possible else call writeIndex */
-  virtual int updateIndex() = 0;
+  int updateIndex();
 
   /** Add the message to the folder after it has been retrieved from an IMAP
       server */
-  virtual void reallyAddMsg(KMMessage* aMsg);
+  void reallyAddMsg(KMMessage* aMsg);
 
   /** Add a copy of the message to the folder after it has been retrieved
       from an IMAP server */
-  virtual void reallyAddCopyOfMsg(KMMessage* aMsg);
+  void reallyAddCopyOfMsg(KMMessage* aMsg);
 
-protected slots:
-  virtual void removeJob( QObject* );
-protected:
-  virtual void addJob( FolderJob* ) const;
-  /**
-   * These two methods actually create the jobs. They have to be implemented
-   * in all folders.
-   * @see createJob
-   */
-  virtual FolderJob* doCreateJob( KMMessage *msg, FolderJob::JobType jt, KMFolder *folder,
-                                  QString partSpecifier, const AttachmentStrategy *as ) const = 0;
-  virtual FolderJob* doCreateJob( QPtrList<KMMessage>& msgList, const QString& sets,
-                                  FolderJob::JobType jt, KMFolder *folder ) const = 0;
+private:
+  FolderStorage* mStorage;
+  KMFolderDir* mParent;
+  KMFolderDir* mChild;
+  bool mIsSystemFolder;
 
-  /** Tell the folder that a header field that is usually used for
-    the index (subject, from, ...) has changed of given message.
-    This method is usually called from within KMMessage::setSubject/set... */
-  void headerOfMsgChanged(const KMMsgBase*, int idx);
-
-  /** Escape a leading dot */
-  virtual QString dotEscape(const QString&) const;
-
-  /** Load message from file and store it at given index. Returns 0
-    on failure. */
-  virtual KMMessage* readMsg(int idx) = 0;
-
-  /** Read index file and fill the message-info list mMsgList. */
-  virtual bool readIndex() = 0;
-
-  /** Called by KMFolder::remove() to delete the actual contents.
-    At the time of the call the folder has already been closed, and
-    the various index files deleted.  Returns 0 on success. */
-  virtual int removeContents() = 0;
-
-  /** Called by KMFolder::expunge() to delete the actual contents.
-    At the time of the call the folder has already been closed, and
-    the various index files deleted.  Returns 0 on success. */
-  virtual int expungeContents() = 0;
-
-  /** Write the config file */
-  virtual void writeConfig();
-
-  /** Read the config file */
-  virtual void readConfig();
-
-  virtual KMMsgBase* takeIndexEntry( int idx ) = 0;
-  virtual KMMsgInfo* setIndexEntry( int idx, KMMessage *msg ) = 0;
-  virtual void clearIndex(bool autoDelete=true, bool syncDict = false) = 0;
-  virtual void fillDictFromIndex(KMMsgDict *dict) = 0;
-  virtual void truncateIndex() = 0;
-
-  int mOpenCount;
-  int mQuiet;
-  bool mChanged;
-  /** is the automatic creation of a index file allowed ? */
-  bool mAutoCreateIndex;
-  /** if the index is dirty it will be recreated upon close() */
-  bool mDirty;
-  /** TRUE if the files of the folder are locked (writable) */
-  bool mFilesLocked;
   /** nationalized label or QString::null (then name() should be used) */
   QString mLabel;
-  /** name of the field that is used for "From" in listbox */
-  QString mWhoField, mUserWhoField;
-  bool mIsSystemFolder;
-  KMAcctList* mAcctList;
-
-  bool    mMailingListEnabled;
-  QString mMailingListPostingAddress;
-  QString mMailingListAdminAddress;
-  uint    mIdentity;
-
-  /** number of unread messages, -1 if not yet set */
-  int mUnreadMsgs, mGuessedUnreadMsgs;
-  int mTotalMsgs;
-  bool mWriteConfigEnabled;
-  /** sven: true if on destruct folder needs to be compacted. */
-  bool needsCompact;
-  /** false if index file is out of sync with mbox file */
-  bool mCompactable;
-  bool mNoContent;
-  KMFolderDir* mChild;
-  bool mConvertToUtf8;
+  QString mSystemLabel;
 
   /** Support for automatic expiry of old messages */
-  bool         expireMessages;          // TRUE if old messages are expired
-  int          unreadExpireAge;         // Given in unreadExpireUnits
-  int          readExpireAge;           // Given in readExpireUnits
-  ExpireUnits  unreadExpireUnits;
-  ExpireUnits  readExpireUnits;
-
-  int          daysToExpire(int num, ExpireUnits units);
-
-  /** Points at the reverse dictionary for this folder. */
-  KMMsgDictREntry *mRDict;
-  /** List of jobs created by this folder.
-   *  REMEBER to add jobs created via createJob
-   *  to this list.
-   */
-  mutable QPtrList<FolderJob> mJobList;
+  bool         mExpireMessages;          // TRUE if old messages are expired
+  int          mUnreadExpireAge;         // Given in unreadExpireUnits
+  int          mReadExpireAge;           // Given in readExpireUnits
+  ExpireUnits  mUnreadExpireUnits;
+  ExpireUnits  mReadExpireUnits;
+  ExpireAction mExpireAction;
+  QString      mExpireToFolderId;
 
   /** Icon related variables */
   bool mUseCustomIcons;
   QString mNormalIconPath;
   QString mUnreadIconPath;
-  QTimer *mDirtyTimer;
-  enum { mDirtyTimerInterval = 600000 }; // 10 minutes
+
+  /** Mailing list attributes */
+  bool                mMailingListEnabled;
+  MailingList         mMailingList;
+
+  uint mIdentity;
+
+  /** name of the field that is used for "From" in listbox */
+  QString mWhoField, mUserWhoField;
+
+  /** Should replies to messages in this folder be put in here? */
+  bool mPutRepliesInSameFolder;
+
+  /** Should new mail in this folder be ignored? */
+  bool mIgnoreNewMail;
 };
 
 #endif /*kmfolder_h*/
