@@ -9,13 +9,17 @@
 #include <qdir.h>
 #include <qregexp.h>
 
-#include "kfileio.h"
+#include <libkdepim/kfileio.h>
 #include "kmfoldermaildir.h"
 #include "kmfoldermgr.h"
+#include "kmfolder.h"
 #include "undostack.h"
 #include "maildirjob.h"
 #include "kcursorsaver.h"
+#include "jobscheduler.h"
 using KMail::MaildirJob;
+#include "compactionjob.h"
+
 #include <kio/netaccess.h>
 #include <kapplication.h>
 #include <kdebug.h>
@@ -31,6 +35,8 @@ using KMail::MaildirJob;
 #include <unistd.h>
 #include <assert.h>
 #include <limits.h>
+#include <unistd.h>
+#include <fcntl.h>
 
 #ifndef MAX_LINE
 #define MAX_LINE 4096
@@ -41,8 +47,8 @@ using KMail::MaildirJob;
 
 
 //-----------------------------------------------------------------------------
-KMFolderMaildir::KMFolderMaildir(KMFolderDir* aParent, const QString& aName)
-  : KMFolderIndex(aParent, aName)
+KMFolderMaildir::KMFolderMaildir(KMFolder* folder, const char* name)
+  : KMFolderIndex(folder, name)
 {
 }
 
@@ -51,26 +57,38 @@ KMFolderMaildir::KMFolderMaildir(KMFolderDir* aParent, const QString& aName)
 KMFolderMaildir::~KMFolderMaildir()
 {
   if (mOpenCount>0) close(TRUE);
-  if (kmkernel->undoStack()) kmkernel->undoStack()->folderDestroyed(this);
+  if (kmkernel->undoStack()) kmkernel->undoStack()->folderDestroyed( folder() );
 }
 
 //-----------------------------------------------------------------------------
 int KMFolderMaildir::canAccess()
 {
 
-  assert(!name().isEmpty());
+  assert(!folder()->name().isEmpty());
 
-  if (access(QFile::encodeName(location()), R_OK | W_OK | X_OK) != 0)
-    return 1;
+  QString sBadFolderName;
+  if (access(QFile::encodeName(location()), R_OK | W_OK | X_OK) != 0) {
+    sBadFolderName = location();
+  } else if (access(QFile::encodeName(location() + "/new"), R_OK | W_OK | X_OK) != 0) {
+    sBadFolderName = location() + "/new";
+  } else if (access(QFile::encodeName(location() + "/cur"), R_OK | W_OK | X_OK) != 0) {
+    sBadFolderName = location() + "/cur";
+  } else if (access(QFile::encodeName(location() + "/tmp"), R_OK | W_OK | X_OK) != 0) {
+    sBadFolderName = location() + "/tmp";
+  }
 
-  if (access(QFile::encodeName(location() + "/new"), R_OK | W_OK | X_OK) != 0)
-    return 1;
-
-  if (access(QFile::encodeName(location() + "/cur"), R_OK | W_OK | X_OK) != 0)
-    return 1;
-
-  if (access(QFile::encodeName(location() + "/tmp"), R_OK | W_OK | X_OK) != 0)
-    return 1;
+  if ( !sBadFolderName.isEmpty() ) {
+    int nRetVal = QFile::exists(sBadFolderName) ? EPERM : ENOENT;
+    KCursorSaver idle(KBusyPtr::idle());
+    if ( nRetVal == ENOENT )
+      KMessageBox::sorry(0, i18n("Error opening %1; this folder is missing.")
+                         .arg(sBadFolderName));
+    else
+      KMessageBox::sorry(0, i18n("Error opening %1; either this is not a valid "
+                                 "maildir folder, or you do not have sufficient access permissions.")
+                         .arg(sBadFolderName));
+    return nRetVal;
+  }
 
   return 0;
 }
@@ -81,30 +99,32 @@ int KMFolderMaildir::open()
   int rc = 0;
 
   mOpenCount++;
+  kmkernel->jobScheduler()->notifyOpeningFolder( folder() );
+
   if (mOpenCount > 1) return 0;  // already open
 
-  assert(!name().isEmpty());
+  assert(!folder()->name().isEmpty());
 
-  if (canAccess() != 0) {
-      KCursorSaver idle(KBusyPtr::idle());
-      KMessageBox::sorry(0, i18n("Error opening %1. Either this is not a valid "
-                                 "maildir folder or you don't have sufficient access permissions.")
-                         .arg(name()));
-      return EPERM;
+  rc = canAccess();
+  if ( rc != 0 ) {
+      return rc;
   }
 
-  if (!path().isEmpty())
+  if (!folder()->path().isEmpty())
   {
     if (KMFolderIndex::IndexOk != indexStatus()) // test if contents file has changed
     {
       QString str;
       mIndexStream = 0;
-      str = i18n("Folder `%1' changed. Recreating index.")
+      str = i18n("Folder `%1' changed; recreating index.")
 		  .arg(name());
       emit statusMsg(str);
     } else {
       mIndexStream = fopen(QFile::encodeName(indexLocation()), "r+"); // index file
-      updateIndexStreamPtr();
+      if ( mIndexStream ) {
+        fcntl(fileno(mIndexStream), F_SETFD, FD_CLOEXEC);
+        updateIndexStreamPtr();
+      }
     }
 
     if (!mIndexStream)
@@ -132,7 +152,7 @@ int KMFolderMaildir::create(bool imap)
   int rc;
   int old_umask;
 
-  assert(!name().isEmpty());
+  assert(!folder()->name().isEmpty());
   assert(mOpenCount == 0);
 
   // Make sure that neither a new, cur or tmp subfolder exists already.
@@ -166,14 +186,15 @@ int KMFolderMaildir::create(bool imap)
     return errno;
   }
 
-  if (!path().isEmpty())
+  if (!folder()->path().isEmpty())
   {
     old_umask = umask(077);
     mIndexStream = fopen(QFile::encodeName(indexLocation()), "w+"); //sven; open RW
-	updateIndexStreamPtr(TRUE);
+    updateIndexStreamPtr(TRUE);
     umask(old_umask);
 
     if (!mIndexStream) return errno;
+    fcntl(fileno(mIndexStream), F_SETFD, FD_CLOEXEC);
   }
   else
   {
@@ -197,12 +218,17 @@ void KMFolderMaildir::close(bool aForced)
 {
   if (mOpenCount <= 0) return;
   if (mOpenCount > 0) mOpenCount--;
+
   if (mOpenCount > 0 && !aForced) return;
-  if ((this != kmkernel->inboxFolder()) && isSystemFolder() && !aForced)
+
+#if 0 // removed hack that prevented closing system folders (see kmail-devel discussion about mail expiring)
+  if ( (folder() != kmkernel->inboxFolder())
+       && folder()->isSystemFolder() && !aForced)
   {
      mOpenCount = 1;
      return;
   }
+#endif
 
   if (mAutoCreateIndex)
   {
@@ -229,7 +255,7 @@ void KMFolderMaildir::sync()
 {
   if (mOpenCount > 0)
     if (!mIndexStream || fsync(fileno(mIndexStream))) {
-    kmkernel->emergencyExit( i18n("Couldn't sync maildir folder.") );
+    kmkernel->emergencyExit( i18n("Could not sync maildir folder.") );
     }
 }
 
@@ -252,23 +278,16 @@ int KMFolderMaildir::expungeContents()
   return 0;
 }
 
-//-----------------------------------------------------------------------------
-int KMFolderMaildir::compact()
+int KMFolderMaildir::compact( unsigned int startIndex, int nbMessages, const QStringList& entryList, bool& done )
 {
-  if (needsCompact == false)
-    return 0;
-
-  open();
-
   QString subdirNew(location() + "/new/");
   QString subdirCur(location() + "/cur/");
 
-  QDir d(subdirNew);
-  QStringList newFiles(d.entryList());
-
-  for (int i = 0; i < count(); i++)
-  {
-    KMMsgInfo *mi = (KMMsgInfo*)mMsgList[i];
+  unsigned int stopIndex = nbMessages == -1 ? mMsgList.count() :
+                           QMIN( mMsgList.count(), startIndex + nbMessages );
+  //kdDebug(5006) << "KMFolderMaildir: compacting from " << startIndex << " to " << stopIndex << endl;
+  for(unsigned int idx = startIndex; idx < stopIndex; ++idx) {
+    KMMsgInfo* mi = (KMMsgInfo*)mMsgList.at(idx);
     if (!mi)
       continue;
 
@@ -277,7 +296,7 @@ int KMFolderMaildir::compact()
       continue;
 
     // first, make sure this isn't in the 'new' subdir
-    if ( newFiles.contains( filename ) )
+    if ( entryList.contains( filename ) )
       moveInternal(subdirNew + filename, subdirCur + filename, mi);
 
     // construct a valid filename.  if it's already valid, then
@@ -292,20 +311,28 @@ int KMFolderMaildir::compact()
       setDirty( true );
     }
 
+#if 0
     // we can't have any New messages at this point
     if (mi->isNew())
     {
       mi->setStatus(KMMsgStatusUnread);
       setDirty( true );
     }
+#endif
   }
-  close();
-
-  needsCompact = false;
-
+  done = ( stopIndex == mMsgList.count() );
   return 0;
 }
 
+//-----------------------------------------------------------------------------
+int KMFolderMaildir::compact( bool silent )
+{
+  KMail::MaildirCompactionJob* job = new KMail::MaildirCompactionJob( folder(), true /*immediate*/ );
+  int rc = job->executeNow( silent );
+  // Note that job autodeletes itself.
+  return rc;
+}
+
 //-------------------------------------------------------------
 FolderJob*
 KMFolderMaildir::doCreateJob( KMMessage *msg, FolderJob::JobType jt,
@@ -351,7 +378,7 @@ if( fileD0.open( IO_WriteOnly ) ) {
   msgParent = aMsg->parent();
   if (msgParent)
   {
-    if (msgParent==this && !kmkernel->folderIsDraftOrOutbox(this))
+    if (msgParent==folder() && !kmkernel->folderIsDraftOrOutbox(folder()))
         return 0;
 
     idx = msgParent->find(aMsg);
@@ -377,8 +404,8 @@ if( fileD0.open( IO_WriteOnly ) ) {
   QString tmp_file(location() + "/tmp/");
   tmp_file += filename;
 
-  if (!kCStringToFile(msgText, tmp_file, false, false, false))
-    kmkernel->emergencyExit( "" ); // kCStringToFile already showed an errormessage
+  if (!KPIM::kCStringToFile(msgText, tmp_file, false, false, false))
+    kmkernel->emergencyExit( "" ); // KPIM::kCStringToFile already showed an errormessage
 
   QFile file(tmp_file);
   size = msgText.length();
@@ -387,7 +414,7 @@ if( fileD0.open( IO_WriteOnly ) ) {
   {
     opened = TRUE;
     rc = open();
-    kdDebug(5006) << "addMsg-open: " << rc << endl;
+    kdDebug(5006) << "KMFolderMaildir::addMsg-open: " << rc << " of folder: " << label() << endl;
     if (rc) return rc;
   }
 
@@ -407,18 +434,18 @@ if( fileD0.open( IO_WriteOnly ) ) {
   if (filename != aMsg->fileName())
     aMsg->setFileName(filename);
 
-  if (aMsg->isUnread() || aMsg->isNew() || this == kmkernel->outboxFolder())
+  if (aMsg->isUnread() || aMsg->isNew() || folder() == kmkernel->outboxFolder())
   {
     if (mUnreadMsgs == -1)
       mUnreadMsgs = 1;
     else
       ++mUnreadMsgs;
-    emit numUnreadMsgsChanged( this );
+    emit numUnreadMsgsChanged( folder() );
   }
   ++mTotalMsgs;
 
   // store information about the position in the folder file in the message
-  aMsg->setParent(this);
+  aMsg->setParent(folder());
   aMsg->setMsgSize(size);
   idx = mMsgList.append(&aMsg->toMsgBase());
   if (aMsg->getMsgSerNum() <= 0)
@@ -491,10 +518,9 @@ if( fileD1.open( IO_WriteOnly ) ) {
 KMMessage* KMFolderMaildir::readMsg(int idx)
 {
   KMMsgInfo* mi = (KMMsgInfo*)mMsgList[idx];
-  KMMessage *msg = new KMMessage(*mi);
-
+  KMMessage *msg = new KMMessage(*mi); // note that mi is deleted by the line below
+  mMsgList.set(idx,&msg->toMsgBase()); // done now so that the serial number can be computed
   msg->fromDwString(getDwString(idx));
-  mMsgList.set(idx,&msg->toMsgBase());
   return msg;
 }
 
@@ -543,7 +569,7 @@ QCString& KMFolderMaildir::getMsgString(int idx, QCString& mDest)
 
   QFileInfo fi( abs_file );
   mDest.resize(fi.size()+2);
-  mDest = kFileToString(abs_file, false, false);
+  mDest = KPIM::kFileToString(abs_file, false, false);
   size_t newMsgSize = crlf2lf( mDest.data(), fi.size() );
   mDest[newMsgSize] = '\0';
   return mDest;
@@ -568,7 +594,13 @@ void KMFolderMaildir::readFileHeaderIntern(const QString& dir, const QString& fi
 
   // open the file and get a pointer to it
   QFile f(file);
-  if (f.open(IO_ReadOnly) == false) return;
+  if ( f.open( IO_ReadOnly ) == false ) {
+    kdWarning(5006) << "The file '" << QFile::encodeName(dir) << "/" << file
+                    << "' could not be opened for reading the message. "
+                       "Please check ownership and permissions."
+                    << endl;
+    return;
+  }
 
   char line[MAX_LINE];
   bool atEof    = false;
@@ -577,7 +609,7 @@ void KMFolderMaildir::readFileHeaderIntern(const QString& dir, const QString& fi
 
   QCString dateStr, fromStr, toStr, subjStr;
   QCString xmarkStr, replyToIdStr, msgIdStr, referencesStr;
-  QCString statusStr, replyToAuxIdStr;
+  QCString statusStr, replyToAuxIdStr, uidStr;
 
   // iterate through this file until done
   while (!atEof)
@@ -650,7 +682,7 @@ void KMFolderMaildir::readFileHeaderIntern(const QString& dir, const QString& fi
           status |= KMMsgStatusFlag;
       }
 
-      KMMsgInfo *mi = new KMMsgInfo(this);
+      KMMsgInfo *mi = new KMMsgInfo(folder());
       mi->init( subjStr.stripWhiteSpace(),
                 fromStr.stripWhiteSpace(),
                 toStr.stripWhiteSpace(),
@@ -664,6 +696,8 @@ void KMFolderMaildir::readFileHeaderIntern(const QString& dir, const QString& fi
       dateStr = dateStr.stripWhiteSpace();
       if (!dateStr.isEmpty())
         mi->setDate(dateStr);
+      if ( !uidStr.isEmpty() )
+         mi->setUID( uidStr.toULong() );
       mi->setDirty(false);
       mMsgList.append(mi);
 
@@ -741,10 +775,16 @@ void KMFolderMaildir::readFileHeaderIntern(const QString& dir, const QString& fi
       replyToIdStr = QCString(line+12);
       lastStr = &replyToIdStr;
     }
+    else if (strncasecmp(line, "X-UID:", 6) == 0)
+    {
+      uidStr = QCString(line+6);
+      lastStr = &uidStr;
+    }
+
   }
 
   if (status & KMMsgStatusNew || status & KMMsgStatusUnread ||
-      (this == kmkernel->outboxFolder()))
+      (folder() == kmkernel->outboxFolder()))
   {
     mUnreadMsgs++;
    if (mUnreadMsgs == 0) ++mUnreadMsgs;
@@ -814,15 +854,15 @@ int KMFolderMaildir::createIndexFromContents()
 
   correctUnreadMsgsCount();
 
-  if (kmkernel->outboxFolder() == this && count() > 0)
+  if (kmkernel->outboxFolder() == folder() && count() > 0)
     KMessageBox::information(0, i18n("Your outbox contains messages which were "
-    "most likely not created by KMail.\nPlease remove them from there, if you "
-    "don't want KMail to send them."));
+    "most-likely not created by KMail;\nplease remove them from there if you "
+    "do not want KMail to send them."));
 
   needsCompact = true;
 
-  if (parent())
-    parent()->manager()->invalidateFolder(kmkernel->msgDict(), this);
+  if (folder()->parent())
+    folder()->parent()->manager()->invalidateFolder(kmkernel->msgDict(), folder());
   return 0;
 }
 
