@@ -8,8 +8,11 @@
 
 #include "kmfoldermbox.h"
 #include "kmfoldermgr.h"
+#include "kmfolder.h"
 #include "undostack.h"
 #include "kcursorsaver.h"
+#include "jobscheduler.h"
+#include "compactionjob.h"
 
 #include <kdebug.h>
 #include <klocale.h>
@@ -31,6 +34,8 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/file.h>
+#include "broadcaststatus.h"
+using KPIM::BroadcastStatus;
 
 #ifndef MAX_LINE
 #define MAX_LINE 4096
@@ -42,16 +47,17 @@
 // Regular expression to find the line that seperates messages in a mail
 // folder:
 #define MSG_SEPERATOR_START "From "
-#define MSG_SEPERATOR_REGEX "^From .*[0-9][0-9]:[0-9][0-9].*$"
-static short msgSepLen = strlen(MSG_SEPERATOR_START);
+#define MSG_SEPERATOR_START_LEN (sizeof(MSG_SEPERATOR_START) - 1)
+#define MSG_SEPERATOR_REGEX "^From .*[0-9][0-9]:[0-9][0-9]"
 
 
 //-----------------------------------------------------------------------------
-KMFolderMbox::KMFolderMbox(KMFolderDir* aParent, const QString& aName)
-  : KMFolderIndex(aParent, aName)
+KMFolderMbox::KMFolderMbox(KMFolder* folder, const char* name)
+  : KMFolderIndex(folder, name)
 {
   mStream         = 0;
   mFilesLocked    = false;
+  mReadOnly       = false;
   mLockType       = lock_none;
 }
 
@@ -60,7 +66,7 @@ KMFolderMbox::KMFolderMbox(KMFolderDir* aParent, const QString& aName)
 KMFolderMbox::~KMFolderMbox()
 {
   if (mOpenCount>0) close(true);
-  if (kmkernel->undoStack()) kmkernel->undoStack()->folderDestroyed(this);
+  if (kmkernel->undoStack()) kmkernel->undoStack()->folderDestroyed( folder() );
 }
 
 //-----------------------------------------------------------------------------
@@ -69,9 +75,11 @@ int KMFolderMbox::open()
   int rc = 0;
 
   mOpenCount++;
+  kmkernel->jobScheduler()->notifyOpeningFolder( folder() );
+
   if (mOpenCount > 1) return 0;  // already open
 
-  assert(!name().isEmpty());
+  assert(!folder()->name().isEmpty());
 
   mFilesLocked = false;
   mStream = fopen(QFile::encodeName(location()), "r+"); // messages file
@@ -86,7 +94,7 @@ int KMFolderMbox::open()
 
   lock();
 
-  if (!path().isEmpty())
+  if (!folder()->path().isEmpty())
   {
      KMFolderIndex::IndexStatus index_status = indexStatus();
      // test if index file exists and is up-to-date
@@ -137,7 +145,10 @@ int KMFolderMbox::open()
        emit statusMsg(str);
      } else {
        mIndexStream = fopen(QFile::encodeName(indexLocation()), "r+"); // index file
-       updateIndexStreamPtr();
+       if ( mIndexStream ) {
+         fcntl(fileno(mIndexStream), F_SETFD, FD_CLOEXEC);
+         updateIndexStreamPtr();
+       }
      }
 
      if (!mIndexStream)
@@ -154,13 +165,17 @@ int KMFolderMbox::open()
 
   mChanged = false;
 
+  fcntl(fileno(mStream), F_SETFD, FD_CLOEXEC);
+  if (mIndexStream)
+     fcntl(fileno(mIndexStream), F_SETFD, FD_CLOEXEC);
+
   return rc;
 }
 
 //----------------------------------------------------------------------------
 int KMFolderMbox::canAccess()
 {
-  assert(!name().isEmpty());
+  assert(!folder()->name().isEmpty());
 
   if (access(QFile::encodeName(location()), R_OK | W_OK) != 0) {
     kdDebug(5006) << "KMFolderMbox::access call to access function failed" << endl;
@@ -177,7 +192,7 @@ int KMFolderMbox::create(bool imap)
 
   Q_UNUSED(imap);
 
-  assert(!name().isEmpty());
+  assert(!folder()->name().isEmpty());
   assert(mOpenCount == 0);
 
   kdDebug(5006) << "Creating folder " << name() << endl;
@@ -194,14 +209,17 @@ int KMFolderMbox::create(bool imap)
 
   if (!mStream) return errno;
 
-  if (!path().isEmpty())
+  fcntl(fileno(mStream), F_SETFD, FD_CLOEXEC);
+
+  if (!folder()->path().isEmpty())
   {
     old_umask = umask(077);
     mIndexStream = fopen(QFile::encodeName(indexLocation()), "w+"); //sven; open RW
-        updateIndexStreamPtr(true);
+    updateIndexStreamPtr(true);
     umask(old_umask);
 
     if (!mIndexStream) return errno;
+    fcntl(fileno(mIndexStream), F_SETFD, FD_CLOEXEC);
   }
   else
   {
@@ -223,11 +241,14 @@ void KMFolderMbox::close(bool aForced)
   if (mOpenCount <= 0 || !mStream) return;
   if (mOpenCount > 0) mOpenCount--;
   if (mOpenCount > 0 && !aForced) return;
-  if ((this != kmkernel->inboxFolder()) && isSystemFolder() && !aForced)
+#if 0 // removed hack that prevented closing system folders (see kmail-devel discussion about mail expiring)
+  if ( (folder() != kmkernel->inboxFolder())
+        && folder()->isSystemFolder() && !aForced )
   {
       mOpenCount = 1;
       return;
   }
+#endif
 
   if (mAutoCreateIndex)
   {
@@ -283,6 +304,7 @@ int KMFolderMbox::lock()
   QCString cmd_str;
   assert(mStream != 0);
   mFilesLocked = false;
+  mReadOnly = false;
 
   switch( mLockType )
   {
@@ -293,6 +315,7 @@ int KMFolderMbox::lock()
       {
         kdDebug(5006) << "Cannot lock folder `" << location() << "': "
                   << strerror(errno) << " (" << errno << ")" << endl;
+        mReadOnly = true;
         return errno;
       }
 
@@ -306,7 +329,8 @@ int KMFolderMbox::lock()
                     << strerror(errno) << " (" << errno << ")" << endl;
           rc = errno;
           fl.l_type = F_UNLCK;
-          rc = fcntl(fileno(mIndexStream), F_SETLK, &fl);
+          /*rc =*/ fcntl(fileno(mIndexStream), F_SETLK, &fl);
+          mReadOnly = true;
           return rc;
         }
       }
@@ -324,6 +348,7 @@ int KMFolderMbox::lock()
       {
         kdDebug(5006) << "Cannot lock folder `" << location() << "': "
                   << strerror(rc) << " (" << rc << ")" << endl;
+        mReadOnly = true;
         return rc;
       }
       if( mIndexStream )
@@ -334,6 +359,7 @@ int KMFolderMbox::lock()
         {
           kdDebug(5006) << "Cannot lock index of folder `" << location() << "': "
                     << strerror(rc) << " (" << rc << ")" << endl;
+          mReadOnly = true;
           return rc;
         }
       }
@@ -346,6 +372,7 @@ int KMFolderMbox::lock()
       {
         kdDebug(5006) << "Cannot lock folder `" << location() << "': "
                   << strerror(rc) << " (" << rc << ")" << endl;
+        mReadOnly = true;
         return rc;
       }
       if( mIndexStream )
@@ -356,6 +383,7 @@ int KMFolderMbox::lock()
         {
           kdDebug(5006) << "Cannot lock index of folder `" << location() << "': "
                     << strerror(rc) << " (" << rc << ")" << endl;
+          mReadOnly = true;
           return rc;
         }
       }
@@ -368,6 +396,7 @@ int KMFolderMbox::lock()
       {
         kdDebug(5006) << "Cannot lock folder `" << location() << "': "
                   << strerror(rc) << " (" << rc << ")" << endl;
+        mReadOnly = true;
         return rc;
       }
       if( mIndexStream )
@@ -378,6 +407,7 @@ int KMFolderMbox::lock()
         {
           kdDebug(5006) << "Cannot lock index of folder `" << location() << "': "
                     << strerror(rc) << " (" << rc << ")" << endl;
+          mReadOnly = true;
           return rc;
         }
       }
@@ -505,6 +535,7 @@ int KMFolderMbox::createIndexFromContents()
   char status[8], xstatus[8];
   QCString subjStr, dateStr, fromStr, toStr, xmarkStr, *lastStr=0;
   QCString replyToIdStr, replyToAuxIdStr, referencesStr, msgIdStr;
+  QCString sizeServerStr, uidStr;
   bool atEof = false;
   bool inHeader = true;
   KMMsgInfo* mi;
@@ -534,6 +565,8 @@ int KMFolderMbox::createIndexFromContents()
   referencesStr = "";
   msgIdStr = "";
   needStatus = 3;
+  size_t sizeServer = 0;
+  ulong uid = 0;
 
 
   while (!atEof)
@@ -542,7 +575,7 @@ int KMFolderMbox::createIndexFromContents()
     if (!fgets(line, MAX_LINE, mStream)) atEof = true;
 
     if (atEof ||
-        (strncmp(line,MSG_SEPERATOR_START, msgSepLen)==0 &&
+        (memcmp(line, MSG_SEPERATOR_START, MSG_SEPERATOR_START_LEN)==0 &&
          regexp.search(line) >= 0))
     {
       size = pos - offs;
@@ -603,7 +636,7 @@ int KMFolderMbox::createIndexFromContents()
               replyToAuxIdStr.truncate( rightAngle + 1 );
           }
 
-          mi = new KMMsgInfo(this);
+          mi = new KMMsgInfo(folder());
           mi->init( subjStr.stripWhiteSpace(),
                     fromStr.stripWhiteSpace(),
                     toStr.stripWhiteSpace(),
@@ -611,7 +644,7 @@ int KMFolderMbox::createIndexFromContents()
                     xmarkStr.stripWhiteSpace(),
                     replyToIdStr, replyToAuxIdStr, msgIdStr,
                     KMMsgEncryptionStateUnknown, KMMsgSignatureStateUnknown,
-                    KMMsgMDNStateUnknown, offs, size );
+                    KMMsgMDNStateUnknown, offs, size, sizeServer, uid );
           mi->setStatus(status, xstatus);
           mi->setDate( dateStr.stripWhiteSpace() );
           mi->setDirty(false);
@@ -628,6 +661,8 @@ int KMFolderMbox::createIndexFromContents()
           dateStr = "";
           fromStr = "";
           subjStr = "";
+          sizeServer = 0;
+          uid = 0;
         }
         else num--,numStatus++;
       }
@@ -703,6 +738,18 @@ int KMFolderMbox::createIndexFromContents()
       subjStr = QCString(line+8);
       lastStr = &subjStr;
     }
+    else if (strncasecmp(line,"X-Length:",9)==0)
+    {
+      sizeServerStr = QCString(line+9);
+      sizeServer = sizeServerStr.toULong();
+      lastStr = &sizeServerStr;
+    }
+    else if (strncasecmp(line,"X-UID:",6)==0)
+    {
+      uidStr = QCString(line+6);
+      uid = uidStr.toULong();
+      lastStr = &uidStr;
+    }
   }
 
   if (mAutoCreateIndex)
@@ -714,14 +761,14 @@ int KMFolderMbox::createIndexFromContents()
 
   correctUnreadMsgsCount();
 
-  if (kmkernel->outboxFolder() == this && count() > 0)
+  if (kmkernel->outboxFolder() == folder() && count() > 0)
     KMessageBox::queuedMessageBox(0, KMessageBox::Information,
                                   i18n("Your outbox contains messages which were "
-    "most likely not created by KMail.\nPlease remove them from there, if you "
-    "don't want KMail to send them."));
+    "most-likely not created by KMail;\nplease remove them from there if you "
+    "do not want KMail to send them."));
 
-  if ( parent() )
-      parent()->manager()->invalidateFolder(kmkernel->msgDict(), this);
+  if ( folder()->parent() )
+      folder()->parent()->manager()->invalidateFolder( kmkernel->msgDict(), folder() );
   return 0;
 }
 
@@ -734,9 +781,9 @@ KMMessage* KMFolderMbox::readMsg(int idx)
   assert(mi!=0 && !mi->isMessage());
   assert(mStream != 0);
 
-  KMMessage* msg = new KMMessage(*mi);
+  KMMessage* msg = new KMMessage(*mi); // note that mi is deleted by the line below
+  mMsgList.set(idx,&msg->toMsgBase()); // done now so that the serial number can be computed
   msg->fromDwString( getDwString( idx ) );
-  mMsgList.set(idx,&msg->toMsgBase());
 
   return msg;
 }
@@ -866,7 +913,7 @@ DwString KMFolderMbox::getDwString(int idx)
 
 
 //-----------------------------------------------------------------------------
-int KMFolderMbox::addMsg(KMMessage* aMsg, int* aIndex_ret)
+int KMFolderMbox::addMsg( KMMessage* aMsg, int* aIndex_ret )
 {
   if (!canAddMsgNow(aMsg, aIndex_ret)) return 0;
   bool opened = false;
@@ -888,7 +935,7 @@ int KMFolderMbox::addMsg(KMMessage* aMsg, int* aIndex_ret)
   {
     opened = true;
     rc = open();
-    kdDebug(5006) << "addMsg-open: " << rc << endl;
+    kdDebug(5006) << "KMFolderMBox::addMsg-open: " << rc << " of folder: " << label() << endl;
     if (rc) return rc;
   }
 
@@ -896,9 +943,9 @@ int KMFolderMbox::addMsg(KMMessage* aMsg, int* aIndex_ret)
   msgParent = aMsg->parent();
   if (msgParent)
   {
-    if (msgParent==this)
+    if ( msgParent== folder() )
     {
-        if (kmkernel->folderIsDraftOrOutbox(this))
+        if (kmkernel->folderIsDraftOrOutbox( folder() ))
           //special case for Edit message.
           {
             kdDebug(5006) << "Editing message in outbox or drafts" << endl;
@@ -972,11 +1019,8 @@ if( fileD1.open( IO_WriteOnly ) ) {
     return error;
   }
 
-  QCString address( aMsg->fromEmail() );
-  if ( address.isEmpty() )
-    address = "unknown@unknown.invalid";
-  fprintf(mStream, "From %s %s\n", address.data(),
-          (const char *)aMsg->dateShortStr());
+  QCString messageSeparator( aMsg->mboxMessageSeparator() );
+  fwrite( messageSeparator.data(), messageSeparator.length(), 1, mStream );
   off_t offs = ftell(mStream);
   fwrite(msgText, len, 1, mStream);
   if (msgText[(int)len-1]!='\n') fwrite("\n\n", 1, 2, mStream);
@@ -1012,15 +1056,16 @@ if( fileD1.open( IO_WriteOnly ) ) {
 //  if (mAccount) aMsg->removeHeaderField("X-UID");
 
   if (aMsg->isUnread() || aMsg->isNew() ||
-      (this == kmkernel->outboxFolder())) {
+      (folder() == kmkernel->outboxFolder())) {
     if (mUnreadMsgs == -1) mUnreadMsgs = 1;
     else ++mUnreadMsgs;
-    emit numUnreadMsgsChanged( this );
+    if ( !mQuiet )
+      emit numUnreadMsgsChanged( folder() );
   }
   ++mTotalMsgs;
 
   // store information about the position in the folder file in the message
-  aMsg->setParent(this);
+  aMsg->setParent(folder());
   aMsg->setFolderOffset(offs);
   aMsg->setMsgSize(size);
   idx = mMsgList.append(&aMsg->toMsgBase());
@@ -1092,56 +1137,19 @@ if( fileD1.open( IO_WriteOnly ) ) {
   return 0;
 }
 
-
-//-----------------------------------------------------------------------------
-int KMFolderMbox::compact()
+int KMFolderMbox::compact( unsigned int startIndex, int nbMessages, FILE* tmpfile, off_t& offs, bool& done )
 {
-  QString tempName;
-  QString msgStr;
   int rc = 0;
-  int openCount = mOpenCount;
-
-  if (!needsCompact)
-    return 0;
-
-  if (!mCompactable) {
-    kdDebug(5006) << location() << " compaction skipped." << endl;
-    return 0;
-  }
-  kdDebug(5006) << "Compacting " << idString() << endl;
-
-  if (KMFolderIndex::IndexOk != indexStatus()) {
-      kdDebug(5006) << "Critical error: " << location() <<
-          " has been modified by an external application while KMail was running." << endl;
-      //      exit(1); backed out due to broken nfs
-  }
-
-  tempName = path() + "/." + name() + ".compacted";
-  mode_t old_umask = umask(077);
-  FILE *tmpfile = fopen(QFile::encodeName(tempName), "w");
-  umask(old_umask);
-  if (!tmpfile)
-    return errno;
-  open();
-
-  KMMsgInfo* mi;
-  size_t msize;
-  off_t folder_offset;
-  off_t offs=0;
-  int msgs=0;
   QCString mtext;
-  for(unsigned int idx = 0; idx < mMsgList.count(); idx++) {
-    if(!(msgs++ % 10)) {
-      msgStr = i18n("Compacting folder: one message done",
-                                "Compacting folder: %n messages done", msgs);
-      if (!kmkernel->shuttingDown())
-          emit statusMsg(msgStr);
-    }
-    mi = (KMMsgInfo*)mMsgList.at(idx);
-    msize = mi->msgSize();
+  unsigned int stopIndex = nbMessages == -1 ? mMsgList.count() :
+                           QMIN( mMsgList.count(), startIndex + nbMessages );
+  //kdDebug(5006) << "KMFolderMbox: compacting from " << startIndex << " to " << stopIndex << endl;
+  for(unsigned int idx = startIndex; idx < stopIndex; ++idx) {
+    KMMsgInfo* mi = (KMMsgInfo*)mMsgList.at(idx);
+    size_t msize = mi->msgSize();
     if (mtext.size() < msize + 2)
       mtext.resize(msize+2);
-    folder_offset = mi->folderOffset();
+    off_t folder_offset = mi->folderOffset();
 
     //now we need to find the separator! grr...
     for(off_t i = folder_offset-25; true; i -= 20) {
@@ -1165,7 +1173,7 @@ int KMFolderMbox::compact()
           }
           offs += folder_offset;
         } else {
-            rc = 666;
+          rc = 666;
         }
         break;
       } else {
@@ -1201,43 +1209,32 @@ int KMFolderMbox::compact()
     mi->setFolderOffset(offs);
     offs += msize;
   }
-  if (!rc)
-      rc = fflush(tmpfile);
-  if (!rc)
-      rc = fsync(fileno(tmpfile));
-  rc |= fclose(tmpfile);
-  if (!rc) {
-    bool autoCreate = mAutoCreateIndex;
-    QFileInfo inf(location());
-    QString box;
-    if (inf.isSymLink())
-      box = inf.readLink();
-    if (!box)
-      box = location();
-    ::rename(QFile::encodeName(tempName), QFile::encodeName(box));
-    writeIndex();
-    writeConfig();
-    mAutoCreateIndex = false;
-    close(true);
-    mAutoCreateIndex = autoCreate;
-    needsCompact = false;             // We are clean now
-  }
-  else
-  {
-    close();
-    kdDebug(5006) << "Error occurred while compacting" << endl;
-    kdDebug(5006) << location() << endl;
-    kdDebug(5006) << "Compaction aborted." << endl;
-  }
+  done = ( !rc && stopIndex == mMsgList.count() ); // finished without errors
+  return rc;
+}
+
+//-----------------------------------------------------------------------------
+int KMFolderMbox::compact( bool silent )
+{
+  // This is called only when the user explicitely requests compaction,
+  // so we don't check needsCompact.
+  int openCount = mOpenCount;
+
+  KMail::MboxCompactionJob* job = new KMail::MboxCompactionJob( folder(), true /*immediate*/ );
+  int rc = job->executeNow( silent );
+  // Note that job autodeletes itself.
 
   if (openCount > 0)
   {
     open();
     mOpenCount = openCount;
   }
+  // If this is the current folder, the changed signal will ultimately call
+  // KMHeaders::setFolderInfoStatus which will override the message, so save/restore it
+  QString statusMsg = BroadcastStatus::instance()->statusMsg();
   emit changed();
-  return 0;
-
+  BroadcastStatus::instance()->setStatusMsg( statusMsg );
+  return rc;
 }
 
 
