@@ -1,19 +1,27 @@
+// -*- mode: C++; c-file-style: "gnu" -*-
 // kmsearchpattern.cpp
 // Author: Marc Mutz <Marc@Mutz.com>
 // This code is under GPL!
 
 #include <config.h>
 
+#include "kmaddrbook.h"
 #include "kmsearchpattern.h"
 #include "kmmessage.h"
 #include "kmmsgindex.h"
 #include "kmmsgdict.h"
+#include "filterlog.h"
+using KMail::FilterLog;
+
+#include <libkdepim/email.h>
 
 #include <kglobal.h>
 #include <klocale.h>
 #include <kdebug.h>
 #include <kconfig.h>
 
+#include <kabc/stdaddressbook.h>
+
 #include <qregexp.h>
 
 #include <mimelib/string.h>
@@ -23,7 +31,9 @@
 
 static const char* funcConfigNames[] =
   { "contains", "contains-not", "equals", "not-equal", "regexp",
-    "not-regexp", "greater", "less-or-equal", "less", "greater-or-equal" };
+    "not-regexp", "greater", "less-or-equal", "less", "greater-or-equal",
+    "is-in-addressbook", "is-not-in-addressbook" , "is-in-category", "is-not-in-category",
+    "has-attachment", "has-no-attachment"};
 static const int numFuncConfigNames = sizeof funcConfigNames / sizeof *funcConfigNames;
 
 
@@ -104,12 +114,21 @@ KMSearchRule * KMSearchRule::createInstanceFromConfig( const KConfig * config, i
 }
 
 KMSearchRule::Function KMSearchRule::configValueToFunc( const char * str ) {
-  if ( !str ) return FuncContains;
+  if ( !str )
+    return FuncNone;
 
   for ( int i = 0 ; i < numFuncConfigNames ; ++i )
     if ( qstricmp( funcConfigNames[i], str ) == 0 ) return (Function)i;
 
-  return FuncContains;
+  return FuncNone;
+}
+
+QString KMSearchRule::functionToString( Function function )
+{
+  if ( function != FuncNone )
+    return funcConfigNames[int( function )];
+  else
+    return "invalid";
 }
 
 void KMSearchRule::writeConfig( KConfig * config, int aIdx ) const {
@@ -119,7 +138,7 @@ void KMSearchRule::writeConfig( KConfig * config, int aIdx ) const {
   static const QString & contents = KGlobal::staticQString( "contents" );
 
   config->writeEntry( field + cIdx, QString(mField) );
-  config->writeEntry( func + cIdx, funcConfigNames[(int)mFunction] );
+  config->writeEntry( func + cIdx, functionToString( mFunction ) );
   config->writeEntry( contents + cIdx, mContents );
 }
 
@@ -133,16 +152,14 @@ bool KMSearchRule::matches( const DwString & aStr, KMMessage & msg,
   return matches( &msg );
 }
 
-#ifndef NDEBUG
 const QString KMSearchRule::asString() const
 {
   QString result  = "\"" + mField + "\" <";
-  result += funcConfigNames[(int)mFunction];
+  result += functionToString( mFunction );
   result += "> \"" + mContents + "\"";
 
   return result;
 }
-#endif
 
 
 
@@ -158,8 +175,8 @@ KMSearchRuleString::KMSearchRuleString( const QCString & field,
 {
   if ( field.isEmpty() || field[0] == '<' )
     mBmHeaderField = 0;
-  else //TODO handle the unrealistic case of the message starting with mField
-    mBmHeaderField = new DwBoyerMoore(("\n" + field + ": ").data()); 	
+  else // make sure you handle the unrealistic case of the message starting with mField
+    mBmHeaderField = new DwBoyerMoore(("\n" + field + ": ").data());
 }
 
 KMSearchRuleString::KMSearchRuleString( const KMSearchRuleString & other )
@@ -215,7 +232,6 @@ bool KMSearchRuleString::matches( const DwString & aStr, KMMessage & msg,
   const int headerLen = ( aHeaderLen > -1 ? aHeaderLen : field().length() ) + 2 ; // +1 for ': '
 
   if ( headerField ) {
-    static const DwBoyerMoore lf( "\n" );
     static const DwBoyerMoore lflf( "\n\n" );
     static const DwBoyerMoore lfcrlf( "\n\r\n" );
 
@@ -223,33 +239,46 @@ bool KMSearchRuleString::matches( const DwString & aStr, KMMessage & msg,
     if ( endOfHeader == DwString::npos )
       endOfHeader = lfcrlf.FindIn( aStr, 0 );
     const DwString headers = ( endOfHeader == DwString::npos ) ? aStr : aStr.substr( 0, endOfHeader );
-    size_t start = headerField->FindIn( headers, 0, false );
+    // In case the searched header is at the beginning, we have to prepend 
+    // a newline - see the comment in KMSearchRuleString constructor
+    DwString fakedHeaders( "\n" );
+    size_t start = headerField->FindIn( fakedHeaders.append( headers ), 0, false );
+    // if the header field doesn't exist then return false for positive
+    // functions and true for negated functions (e.g. "does not
+    // contain"); note that all negated string functions correspond
+    // to an odd value
     if ( start == DwString::npos )
-      return false;
+      return ( ( function() & 1 ) == 1 );
     start += headerLen;
-    size_t stop = lf.FindIn( aStr, start );
+    size_t stop = aStr.find( '\n', start );
     char ch = '\0';
     while ( stop != DwString::npos && ( ch = aStr.at( stop + 1 ) ) == ' ' || ch == '\t' )
-      stop = lf.FindIn( aStr, stop + 1 );
+      stop = aStr.find( '\n', stop + 1 );
     const int len = stop == DwString::npos ? aStr.length() - start : stop - start ;
     const QCString codedValue( aStr.data() + start, len + 1 );
-    const QString msgContents = KMMsgBase::decodeRFC2047String( codedValue ).stripWhiteSpace();
+    const QString msgContents = KMMsgBase::decodeRFC2047String( codedValue ).stripWhiteSpace(); // FIXME: This needs to be changed for IDN support.
     return matchesInternal( msgContents );
   } else if ( field() == "<recipients>" ) {
     static const DwBoyerMoore to("\nTo: ");
-    bool res = matches( aStr, msg, &to, 2 );
-    if ( !res ) {
-      static const DwBoyerMoore cc("\nCc: ");
-      res = matches( aStr, msg, &cc, 2 );
+    static const DwBoyerMoore cc("\nCc: ");
+    static const DwBoyerMoore bcc("\nBcc: ");
+    // <recipients> "contains" "foo" is true if any of the fields contains
+    // "foo", while <recipients> "does not contain" "foo" is true if none
+    // of the fields contains "foo"
+    if ( ( function() & 1 ) == 0 ) {
+      // positive function, e.g. "contains"
+      return ( matches( aStr, msg, &to, 2 ) ||
+               matches( aStr, msg, &cc, 2 ) ||
+               matches( aStr, msg, &bcc, 3 ) );
     }
-    if ( !res ) {
-      static const DwBoyerMoore bcc("\nBcc: ");
-      res = matches( aStr, msg, &bcc, 3 );
+    else {
+      // negated function, e.g. "does not contain"
+      return ( matches( aStr, msg, &to, 2 ) &&
+               matches( aStr, msg, &cc, 2 ) &&
+               matches( aStr, msg, &bcc, 3 ) );
     }
-    return res;
-  } else {
-    return KMSearchRule::matches( aStr, msg, aHeaderField, aHeaderLen );
   }
+  return false;
 }
 
 bool KMSearchRuleString::matches( const KMMessage * msg ) const
@@ -260,13 +289,19 @@ bool KMSearchRuleString::matches( const KMMessage * msg ) const
     return false;
 
   QString msgContents;
+  // Show the value used to compare the rules against in the log.
+  // Overwrite the value for complete messages and all headers!
+  bool logContents = true;
 
   if( field() == "<message>" ) {
     msgContents = msg->asString();
+    logContents = false;
   } else if ( field() == "<body>" ) {
-    msgContents = msg->bodyDecoded();
+    msgContents = msg->bodyToUnicode();
+    logContents = false;
   } else if ( field() == "<any header>" ) {
     msgContents = msg->headerAsString();
+    logContents = false;
   } else if ( field() == "<recipients>" ) {
     // (mmutz 2001-11-05) hack to fix "<recipients> !contains foo" to
     // meet user's expectations. See FAQ entry in KDE 2.2.2's KMail
@@ -276,14 +311,47 @@ bool KMSearchRuleString::matches( const KMMessage * msg ) const
       // "equality" mean for recipients.
       return matchesInternal( msg->headerField("To") )
           || matchesInternal( msg->headerField("Cc") )
-          || matchesInternal( msg->headerField("Bcc") );
+          || matchesInternal( msg->headerField("Bcc") )
+          // sometimes messages have multiple Cc headers
+          || matchesInternal( msg->cc() );
 
-    msgContents = msg->headerField("To") + msg->headerField("Cc")
-                + msg->headerField("Bcc");
+    msgContents = msg->headerField("To");
+    if ( !msg->headerField("Cc").compare( msg->cc() ) )
+      msgContents += ", " + msg->headerField("Cc");
+    else
+      msgContents += ", " + msg->cc();
+    msgContents += ", " + msg->headerField("Bcc");
   }  else {
-    msgContents = msg->headerField( field() );
+    // make sure to treat messages with multiple header lines for
+    // the same header correctly
+    msgContents = msg->headerFields( field() ).join( " " );
   }
-  return matchesInternal( msgContents );
+
+  if ( function() == FuncIsInAddressbook ||
+       function() == FuncIsNotInAddressbook ) {
+    // I think only the "from"-field makes sense.
+    msgContents = msg->headerField( field() );
+    if ( msgContents.isEmpty() )
+      return false;
+  }
+
+  // these two functions need the kmmessage therefore they don't call matchesInternal
+  if ( function() == FuncHasAttachment )
+    return ( msg->toMsgBase().attachmentState() == KMMsgHasAttachment );
+  if ( function() == FuncHasNoAttachment )
+    return ( ((KMMsgAttachmentState) msg->toMsgBase().attachmentState()) == KMMsgHasNoAttachment );
+
+  bool rc = matchesInternal( msgContents );
+  if ( FilterLog::instance()->isLogging() ) {
+    QString msg = ( rc ? "<font color=#00FF00>1 = </font>"
+                       : "<font color=#FF0000>0 = </font>" );
+    msg += FilterLog::recode( asString() );
+    // only log headers bcause messages and bodies can be pretty large
+    if ( logContents )
+      msg += " (<i>" + FilterLog::recode( msgContents ) + "</i>)";
+    FilterLog::instance()->add( msg, FilterLog::ruleResult );
+  }
+  return rc;
 }
 
 // helper, does the actual comparing
@@ -325,6 +393,68 @@ bool KMSearchRuleString::matchesInternal( const QString & msgContents ) const
 
   case FuncIsGreaterOrEqual:
       return ( QString::compare( msgContents.lower(), contents().lower() ) >= 0 );
+
+  case FuncIsInAddressbook: {
+    KABC::AddressBook *stdAb = KABC::StdAddressBook::self();
+    QStringList addressList =
+      KPIM::splitEmailAddrList( msgContents.lower() );
+    for( QStringList::ConstIterator it = addressList.begin();
+         ( it != addressList.end() );
+         ++it ) {
+      if ( !stdAb->findByEmail( KPIM::getEmailAddr( *it ) ).isEmpty() )
+        return true;
+    }
+    return false;
+  }
+
+  case FuncIsNotInAddressbook: {
+    KABC::AddressBook *stdAb = KABC::StdAddressBook::self();
+    QStringList addressList =
+      KPIM::splitEmailAddrList( msgContents.lower() );
+    for( QStringList::ConstIterator it = addressList.begin();
+         ( it != addressList.end() );
+         ++it ) {
+      if ( stdAb->findByEmail( KPIM::getEmailAddr( *it ) ).isEmpty() )
+        return true;
+    }
+    return false;
+  }
+
+  case FuncIsInCategory: {
+    QString category = contents();
+    QStringList addressList =  KPIM::splitEmailAddrList( msgContents.lower() );
+    KABC::AddressBook *stdAb = KABC::StdAddressBook::self();
+
+    for( QStringList::ConstIterator it = addressList.begin();
+      it != addressList.end(); ++it ) {
+        KABC::Addressee::List addresses = stdAb->findByEmail( KPIM::getEmailAddr( *it ) );
+
+          for ( KABC::Addressee::List::Iterator itAd = addresses.begin(); itAd != addresses.end(); ++itAd )
+              if ( (*itAd).hasCategory(category) )
+                return true;
+
+      }
+      return false;
+    }
+
+    case FuncIsNotInCategory: {
+      QString category = contents();
+      QStringList addressList =  KPIM::splitEmailAddrList( msgContents.lower() );
+      KABC::AddressBook *stdAb = KABC::StdAddressBook::self();
+
+      for( QStringList::ConstIterator it = addressList.begin();
+        it != addressList.end(); ++it ) {
+          KABC::Addressee::List addresses = stdAb->findByEmail( KPIM::getEmailAddr( *it ) );
+
+            for ( KABC::Addressee::List::Iterator itAd = addresses.begin(); itAd != addresses.end(); ++itAd )
+                if ( (*itAd).hasCategory(category) )
+                  return false;
+
+      }
+      return true;
+    }
+  default:
+    ;
   }
 
   return false;
@@ -370,11 +500,19 @@ bool KMSearchRuleNumerical::matches( const KMMessage * msg ) const
     numericalValue = contents().toInt();
     msgContents.setNum( numericalMsgContents );
   }
-  return matchesInternal( numericalValue, numericalMsgContents, msgContents );
+  bool rc = matchesInternal( numericalValue, numericalMsgContents, msgContents );
+  if ( FilterLog::instance()->isLogging() ) {
+    QString msg = ( rc ? "<font color=#00FF00>1 = </font>"
+                       : "<font color=#FF0000>0 = </font>" );
+    msg += FilterLog::recode( asString() );
+    msg += " ( <i>" + QString::number( numericalMsgContents ) + "</i> )";
+    FilterLog::instance()->add( msg, FilterLog::ruleResult );
+  }
+  return rc;
 }
 
-bool KMSearchRuleNumerical::matchesInternal( unsigned long numericalValue,
-    unsigned long numericalMsgContents, const QString & msgContents ) const
+bool KMSearchRuleNumerical::matchesInternal( long numericalValue,
+    long numericalMsgContents, const QString & msgContents ) const
 {
   switch ( function() ) {
   case KMSearchRule::FuncEquals:
@@ -412,6 +550,15 @@ bool KMSearchRuleNumerical::matchesInternal( unsigned long numericalValue,
 
   case FuncIsGreaterOrEqual:
       return ( numericalMsgContents >= numericalValue );
+
+  case FuncIsInAddressbook:  // since email-addresses are not numerical, I settle for false here
+    return false;
+
+  case FuncIsNotInAddressbook:
+    return false;
+
+  default:
+    ;
   }
 
   return false;
@@ -425,44 +572,54 @@ bool KMSearchRuleNumerical::matchesInternal( unsigned long numericalValue,
 //
 //==================================================
 
+
 KMSearchRuleStatus::KMSearchRuleStatus( const QCString & field,
                                         Function func, const QString & aContents )
           : KMSearchRule(field, func, aContents)
 {
   // the values are always in english, both from the conf file as well as
   // the patternedit gui
-  if ( ! aContents.compare("new") )
-    mStatus = KMMsgStatusNew;
-  if ( ! aContents.compare("unread") )
-    mStatus = KMMsgStatusUnread;
-  if ( ! aContents.compare("read") )
-    mStatus = KMMsgStatusRead;
-  if ( ! aContents.compare("old") )
-    mStatus = KMMsgStatusOld;
-  if ( ! aContents.compare("deleted") )
-    mStatus = KMMsgStatusDeleted;
-  if ( ! aContents.compare("replied") )
-    mStatus = KMMsgStatusReplied;
- if ( ! aContents.compare("forwarded") )
-    mStatus = KMMsgStatusForwarded;
- if ( ! aContents.compare("queued") )
-    mStatus = KMMsgStatusQueued;
- if ( ! aContents.compare("sent") )
-    mStatus = KMMsgStatusSent;
- if ( ! aContents.compare("important") )
-    mStatus = KMMsgStatusFlag;
- if ( ! aContents.compare("watched") )
-    mStatus = KMMsgStatusWatched;
- if ( ! aContents.compare("ignored") )
-    mStatus = KMMsgStatusIgnored;
- /*
-  if ( ! aContents.compare("todo") )
-    mStatus = KMMsgStatusTodo;
-  */
- if ( ! aContents.compare("spam") )
-    mStatus = KMMsgStatusSpam;
-if ( ! aContents.compare("ham") )
-    mStatus = KMMsgStatusHam;
+  mStatus = statusFromEnglishName( aContents );
+}
+
+KMMsgStatus KMSearchRuleStatus::statusFromEnglishName(
+      const QString & aStatusString )
+{
+  KMMsgStatus status = 0;
+  if ( ! aStatusString.compare("important") )
+    status = KMMsgStatusFlag;
+  if ( ! aStatusString.compare("new") )
+    status = KMMsgStatusNew;
+  if ( ! aStatusString.compare("unread") )
+    status = KMMsgStatusUnread | KMMsgStatusNew;
+  if ( ! aStatusString.compare("read") )
+    status = KMMsgStatusRead;
+  if ( ! aStatusString.compare("old") )
+    status = KMMsgStatusOld;
+  if ( ! aStatusString.compare("deleted") )
+    status = KMMsgStatusDeleted;
+  if ( ! aStatusString.compare("replied") )
+    status = KMMsgStatusReplied;
+  if ( ! aStatusString.compare("forwarded") )
+    status = KMMsgStatusForwarded;
+  if ( ! aStatusString.compare("queued") )
+    status = KMMsgStatusQueued;
+  if ( ! aStatusString.compare("sent") )
+    status = KMMsgStatusSent;
+  if ( ! aStatusString.compare("watched") )
+    status = KMMsgStatusWatched;
+  if ( ! aStatusString.compare("ignored") )
+    status = KMMsgStatusIgnored;
+  if ( ! aStatusString.compare("todo") )
+    status = KMMsgStatusTodo;
+  if ( ! aStatusString.compare("spam") )
+    status = KMMsgStatusSpam;
+  if ( ! aStatusString.compare("ham") )
+     status = KMMsgStatusHam;
+  if ( ! aStatusString.compare("has an attachment") )
+     status = KMMsgStatusHasAttach;
+
+  return status;
 }
 
 bool KMSearchRuleStatus::isEmpty() const
@@ -481,17 +638,18 @@ bool KMSearchRuleStatus::matches( const KMMessage * msg ) const
 {
 
   KMMsgStatus msgStatus = msg->status();
+  bool rc = false;
 
   switch ( function() ) {
     case FuncEquals: // fallthrough. So that "<status> 'is' 'read'" works
     case FuncContains:
       if (msgStatus & mStatus)
-        return true;
+        rc = true;
       break;
     case FuncNotEqual: // fallthrough. So that "<status> 'is not' 'read'" works
     case FuncContainsNot:
       if (! (msgStatus & mStatus) )
-        return true;
+        rc = true;
       break;
     // FIXME what about the remaining funcs, how can they make sense for
     // stati?
@@ -499,7 +657,13 @@ bool KMSearchRuleStatus::matches( const KMMessage * msg ) const
       break;
   }
 
-  return false;
+  if ( FilterLog::instance()->isLogging() ) {
+    QString msg = ( rc ? "<font color=#00FF00>1 = </font>"
+                       : "<font color=#FF0000>0 = </font>" );
+    msg += FilterLog::recode( asString() );
+    FilterLog::instance()->add( msg, FilterLog::ruleResult );
+  }
+  return rc;
 }
 
 // ----------------------------------------------------------------------------
@@ -524,10 +688,11 @@ KMSearchPattern::~KMSearchPattern()
 {
 }
 
-bool KMSearchPattern::matches( const KMMessage * msg ) const {
-
+bool KMSearchPattern::matches( const KMMessage * msg ) const
+{
   if ( isEmpty() )
-    return false;
+    return true;
+
   QPtrListIterator<KMSearchRule> it( *this );
   switch ( mOperator ) {
   case OpAnd: // all rules must match
@@ -548,7 +713,8 @@ bool KMSearchPattern::matches( const KMMessage * msg ) const {
 bool KMSearchPattern::matches( const DwString & aStr ) const
 {
   if ( isEmpty() )
-    return false;
+    return true;
+
   KMMessage msg;
   QPtrListIterator<KMSearchRule> it( *this );
   switch ( mOperator ) {
@@ -569,6 +735,9 @@ bool KMSearchPattern::matches( const DwString & aStr ) const
 
 bool KMSearchPattern::matches( Q_UINT32 serNum ) const
 {
+  if ( isEmpty() )
+    return true;
+
   bool res;
   int idx = -1;
   KMFolder *folder = 0;
@@ -577,7 +746,9 @@ bool KMSearchPattern::matches( Q_UINT32 serNum ) const
     return false;
   }
 
-  folder->open();
+  bool opened = folder->isOpened();
+  if ( !opened )
+    folder->open();
   KMMsgBase *msgBase = folder->getMsgBase(idx);
   if (requiresBody()) {
     bool unGet = !msgBase->isMessage();
@@ -588,7 +759,8 @@ bool KMSearchPattern::matches( Q_UINT32 serNum ) const
   } else {
     res = matches( folder->getDwString(idx) );
   }
-  folder->close();
+  if ( !opened )
+    folder->close();
   return res;
 }
 
@@ -700,18 +872,18 @@ void KMSearchPattern::init() {
   mName = '<' + i18n("name used for a virgin filter","unknown") + '>';
 }
 
-#ifndef NDEBUG
 QString KMSearchPattern::asString() const {
-  QString result = "Match ";
-  result += ( mOperator == OpOr ) ? "any" : "all";
-  result += " of the following:\n";
+  QString result;
+  if ( mOperator == OpOr )
+    result = i18n("(match any of the following)");
+  else
+    result = i18n("(match all of the following)");
 
   for ( QPtrListIterator<KMSearchRule> it( *this ) ; it.current() ; ++it )
-    result += (*it)->asString() + '\n';
+    result += "\n\t" + FilterLog::recode( (*it)->asString() );
 
   return result;
 }
-#endif
 
 const KMSearchPattern & KMSearchPattern::operator=( const KMSearchPattern & other ) {
   if ( this == &other )
