@@ -57,13 +57,12 @@ using KRecentAddress::RecentAddresses;
 #include "knconfigmanager.h"
 #include "knaccountmanager.h"
 #include "knnntpaccount.h"
-#include <kpgp.h>
 #include "knarticlefactory.h"
 #include <kstatusbar.h>
 #include <klocale.h>
 #include <qpopupmenu.h>
 #include <spellingfilter.h>
-#include <qcursor.h>
+#include <kstdguiitem.h>
 
 KNLineEdit::KNLineEdit(KNComposer::ComposerView *_composerView, bool useCompletion,
                        QWidget *parent, const char *name)
@@ -72,6 +71,35 @@ KNLineEdit::KNLineEdit(KNComposer::ComposerView *_composerView, bool useCompleti
 {
 }
 
+
+QPopupMenu *KNLineEdit::createPopupMenu()
+{
+    QPopupMenu *menu = KLineEdit::createPopupMenu();
+    if ( !menu )
+        return 0;
+
+    menu->insertSeparator();
+    menu->insertItem( i18n( "Edit Recent Addresses..." ),
+                      this, SLOT( editRecentAddresses() ) );
+
+    return menu;
+}
+
+void KNLineEdit::editRecentAddresses()
+{
+  KRecentAddress::RecentAddressDialog dlg( this );
+  dlg.setAddresses( RecentAddresses::self( knGlobals.config() )->addresses() );
+  if ( dlg.exec() ) {
+    RecentAddresses::self( knGlobals.config() )->clear();
+    QStringList addrList = dlg.addresses();
+    QStringList::Iterator it;
+    for ( it = addrList.begin(); it != addrList.end(); ++it )
+      RecentAddresses::self( knGlobals.config() )->add( *it );
+
+    loadAddresses();
+  }
+}
+
 void KNLineEdit::loadAddresses()
 {
     KNLineEditInherited::loadAddresses();
@@ -210,6 +238,7 @@ KNComposer::KNComposer(KNLocalArticle *a, const QString &text, const QString &si
   KStdAction::selectAll(this, SLOT(slotSelectAll()), actionCollection());
 
   KStdAction::find(v_iew->e_dit, SLOT(slotFind()), actionCollection());
+  KStdAction::findNext(v_iew->e_dit, SLOT(slotSearchAgain()), actionCollection());
 
   KStdAction::replace(v_iew->e_dit, SLOT(slotReplace()), actionCollection());
 
@@ -233,7 +262,7 @@ KNComposer::KNComposer(KNLocalArticle *a, const QString &text, const QString &si
   a_ctRemoveAttachment = new KAction(i18n("&Remove"), 0, this,
                                     SLOT(slotRemoveAttachment()), actionCollection(), "remove_attachment");
 
-  a_ctAttachmentProperties  = new KAction(i18n("&Properties..."), 0, this,
+  a_ctAttachmentProperties  = new KAction(i18n("&Properties"), 0, this,
                                           SLOT(slotAttachmentProperties()), actionCollection(), "attachment_properties");
 
   //options menu
@@ -245,7 +274,7 @@ KNComposer::KNComposer(KNLocalArticle *a, const QString &text, const QString &si
                    SLOT(slotToggleDoMail()), actionCollection(), "send_mail");
 
   a_ctSetCharset = new KSelectAction(i18n("Set &Charset"), 0, actionCollection(), "set_charset");
-  a_ctSetCharset->setItems(knGlobals.cfgManager->postNewsTechnical()->composerCharsets());
+  a_ctSetCharset->setItems(knGlobals.configManager()->postNewsTechnical()->composerCharsets());
   a_ctSetCharset->setShortcutConfigurable(false);
   connect(a_ctSetCharset, SIGNAL(activated(const QString&)),
   this, SLOT(slotSetCharset(const QString&)));
@@ -315,7 +344,7 @@ KNComposer::KNComposer(KNLocalArticle *a, const QString &text, const QString &si
   setConfig(false);
 
   if (firstEdit) {   // now we place the cursor at the end of the quoted text / below the attribution line
-    if (knGlobals.cfgManager->postNewsComposer()->cursorOnTop())
+    if (knGlobals.configManager()->postNewsComposer()->cursorOnTop())
       v_iew->e_dit->setCursorPosition(1,0);
     else
       v_iew->e_dit->setCursorPosition(v_iew->e_dit->numLines()-1,0);
@@ -336,7 +365,7 @@ KNComposer::KNComposer(KNLocalArticle *a, const QString &text, const QString &si
     v_iew->t_o->setFocus();
   }
 
-  if(firstEdit && knGlobals.cfgManager->postNewsComposer()->appendOwnSignature())
+  if(firstEdit && knGlobals.configManager()->postNewsComposer()->appendOwnSignature())
     slotAppendSig();
 
   if (createCopy && (m_ode==news)) {
@@ -353,7 +382,7 @@ KNComposer::KNComposer(KNLocalArticle *a, const QString &text, const QString &si
   applyMainWindowSettings(conf);
 
   // starting the external editor
-  if(knGlobals.cfgManager->postNewsComposer()->useExternalEditor())
+  if(knGlobals.configManager()->postNewsComposer()->useExternalEditor())
     slotExternalEditor();
 }
 
@@ -473,16 +502,16 @@ void KNComposer::slotSelectAll()
 void KNComposer::setConfig(bool onlyFonts)
 {
   if (!onlyFonts) {
-    v_iew->e_dit->setWordWrap(knGlobals.cfgManager->postNewsComposer()->wordWrap()?
+    v_iew->e_dit->setWordWrap(knGlobals.configManager()->postNewsComposer()->wordWrap()?
                               QMultiLineEdit::FixedColumnWidth : QMultiLineEdit::NoWrap);
-    v_iew->e_dit->setWrapColumnOrWidth(knGlobals.cfgManager->postNewsComposer()->maxLineLength());
-    a_ctWordWrap->setChecked(knGlobals.cfgManager->postNewsComposer()->wordWrap());
+    v_iew->e_dit->setWrapColumnOrWidth(knGlobals.configManager()->postNewsComposer()->maxLineLength());
+    a_ctWordWrap->setChecked(knGlobals.configManager()->postNewsComposer()->wordWrap());
 
     Kpgp::Module *pgp = Kpgp::Module::getKpgp();
     a_ctPGPsign->setEnabled(pgp->usePGP());
   }
 
-  QFont fnt=knGlobals.cfgManager->appearance()->composerFont();
+  QFont fnt=knGlobals.configManager()->appearance()->composerFont();
   v_iew->s_ubject->setFont(fnt);
   v_iew->t_o->setFont(fnt);
   v_iew->g_roups->setFont(fnt);
@@ -524,7 +553,7 @@ bool KNComposer::hasValidData()
   // header checks
 
   if (v_iew->s_ubject->text().isEmpty()) {
-    KMessageBox::sorry(this, i18n("Please enter a subject!"));
+    KMessageBox::sorry(this, i18n("Please enter a subject."));
     return false;
   }
   if (!n_eeds8Bit && !KMime::isUsAscii(v_iew->s_ubject->text()))
@@ -532,7 +561,7 @@ bool KNComposer::hasValidData()
 
   if (m_ode != mail) {
     if (v_iew->g_roups->text().isEmpty()) {
-      KMessageBox::sorry(this, i18n("Please enter a newsgroup!"));
+      KMessageBox::sorry(this, i18n("Please enter a newsgroup."));
       return false;
     }
 
@@ -541,7 +570,7 @@ bool KNComposer::hasValidData()
     bool followUp = !v_iew->f_up2->currentText().isEmpty();
 
     if (groupCount>12) {
-      KMessageBox::sorry(this, i18n("You are crossposting to more than 12 newsgroups.\nPlease remove all newsgroups in which your article is off-topic!"));
+      KMessageBox::sorry(this, i18n("You are crossposting to more than 12 newsgroups.\nPlease remove all newsgroups in which your article is off-topic."));
       return false;
     }
 
@@ -556,7 +585,7 @@ bool KNComposer::hasValidData()
         return false;
 
     if (fupCount>12) {
-      KMessageBox::sorry(this, i18n("You are directing replies to more than 12 newsgroups.\nPlease remove some newsgroups from the \"Followup-To\" header!"));
+      KMessageBox::sorry(this, i18n("You are directing replies to more than 12 newsgroups.\nPlease remove some newsgroups from the \"Followup-To\" header."));
       return false;
     }
 
@@ -568,7 +597,7 @@ bool KNComposer::hasValidData()
 
   if (m_ode != news) {
     if (v_iew->t_o->text().isEmpty() ) {
-      KMessageBox::sorry(this, i18n("Please enter the email address!"));
+      KMessageBox::sorry(this, i18n("Please enter the email address."));
       return false;
     }
     if (!n_eeds8Bit && !KMime::isUsAscii(v_iew->t_o->text()))
@@ -620,24 +649,24 @@ bool KNComposer::hasValidData()
   }
 
   if (n_eeds8Bit && (c_harset.lower()=="us-ascii")) {
-    KMessageBox::sorry(this, i18n("Your message contains characters that aren't included\nin the \"us-ascii\" character set. Please choose\na suitable character set in the \"Options\" menu!"));
+    KMessageBox::sorry(this, i18n("Your message contains characters which are not included\nin the \"us-ascii\" character set; please choose\na suitable character set from the \"Options\" menu."));
     return false;
   }
 
   if (empty) {
-    KMessageBox::sorry(this, i18n("You can't post an empty message!"));
+    KMessageBox::sorry(this, i18n("You cannot post an empty message."));
     return false;
   }
 
   if ((textLines>1)&&(notQuoted==1)) {
     if (hasAttributionLine)
-      if (!(KMessageBox::warningYesNo( this, i18n("Your article seems to consist entirely of quoted text.\nDo you want to re-edit the article or send it anyway?"),
+      if (!(KMessageBox::warningYesNo( this, i18n("Your article seems to consist entirely of quoted text;\ndo you want to re-edit the article or send it anyway?"),
                                        QString::null, i18n("&Send"),i18n("edit article","&Edit")) == KMessageBox::Yes))
         return false;
   } else {
     if (notQuoted==0) {
-      KMessageBox::sorry(this, i18n("You can't post an article consisting\n"
-			      "entirely of quoted text!"));
+      KMessageBox::sorry(this, i18n("You cannot post an article consisting\n"
+			      "entirely of quoted text."));
       return false;
     }
   }
@@ -656,17 +685,17 @@ bool KNComposer::hasValidData()
       return false;
   } else
     if (sigLength>4)
-       KMessageBox::information(this, i18n("Your signature exceeds the widely accepted limit of 4 lines.\nPlease consider shortening your signature.\nOtherwise, you will probably annoy your readers"),
+       KMessageBox::information(this, i18n("Your signature exceeds the widely-accepted limit of 4 lines:\nplease consider shortening your signature;\notherwise, you will probably annoy your readers."),
                                 QString::null,"longSignatureWarning");
 
   // check if article can be signed
   if ( a_ctPGPsign->isChecked() ) {
     // try to get the signing key
-    QCString signingKey = knGlobals.cfgManager->identity()->signingKey();
-    KNNntpAccount *acc = knGlobals.accManager->account( a_rticle->serverId() );
+    QCString signingKey = knGlobals.configManager()->identity()->signingKey();
+    KNNntpAccount *acc = knGlobals.accountManager()->account( a_rticle->serverId() );
     if ( acc ) {
       KMime::Headers::Newsgroups *grps = a_rticle->newsgroups();
-      KNGroup *grp = knGlobals.grpManager->group( grps->firstGroup(), acc );
+      KNGroup *grp = knGlobals.groupManager()->group( grps->firstGroup(), acc );
       if ( grp && grp->identity() && grp->identity()->hasSigningKey() )
         signingKey = grp->identity()->signingKey();
       else if ( acc->identity() && acc->identity()->hasSigningKey() )
@@ -676,12 +705,12 @@ bool KNComposer::hasValidData()
     // the article can only be signed if we have a key
     if (signingKey.isEmpty()) {
           if ( KMessageBox::warningContinueCancel( this,
-                   i18n("You haven't configured your preferred "
-                        "signing key yet.\n"
-                        "Please specify it in the global "
+                   i18n("You have not configured your preferred "
+                        "signing key yet;\n"
+                        "please specify it in the global "
                         "identity configuration,\n"
                         "in the account properties or in the "
-                        "group properties!\n"
+                        "group properties.\n"
                         "The article will be sent unsigned." ),
                    QString::null, i18n( "Send Unsigned" ),
                    "sendUnsignedDialog" )
@@ -759,7 +788,7 @@ bool KNComposer::applyChanges()
   }
 
   //set text
-  KNConfig::PostNewsTechnical *pnt=knGlobals.cfgManager->postNewsTechnical();
+  KNConfig::PostNewsTechnical *pnt=knGlobals.configManager()->postNewsTechnical();
   if (v_alidated) {
     if (n_eeds8Bit) {
       text->contentType()->setCharset(c_harset);
@@ -788,11 +817,11 @@ bool KNComposer::applyChanges()
   // Sign article if needed
   if ( a_ctPGPsign->isChecked() ) {
       // first get the signing key
-      QCString signingKey = knGlobals.cfgManager->identity()->signingKey();
-      KNNntpAccount *acc = knGlobals.accManager->account( a_rticle->serverId() );
+      QCString signingKey = knGlobals.configManager()->identity()->signingKey();
+      KNNntpAccount *acc = knGlobals.accountManager()->account( a_rticle->serverId() );
       if ( acc ) {
           KMime::Headers::Newsgroups *grps = a_rticle->newsgroups();
-          KNGroup *grp = knGlobals.grpManager->group( grps->firstGroup(), acc );
+          KNGroup *grp = knGlobals.groupManager()->group( grps->firstGroup(), acc );
           if ( grp && grp->identity() && grp->identity()->hasSigningKey() )
               signingKey = grp->identity()->signingKey();
           else if ( acc->identity() && acc->identity()->hasSigningKey() )
@@ -812,6 +841,8 @@ bool KNComposer::applyChanges()
           if( block.clearsign( signingKey, codec->name() ) == Kpgp::Ok ) {
               QCString result = block.text();
               tmp = codec->toUnicode(result.data(), result.length() );
+          } else {
+              return false;
           }
       }
   }
@@ -837,7 +868,7 @@ void KNComposer::closeEvent(QCloseEvent *e)
   }
   else {
     switch ( KMessageBox::warningYesNoCancel( this, i18n("Do you want to save this article in the draft folder?"),
-                                              QString::null, i18n("&Save"), i18n("&Discard"))) {
+                                              QString::null, KStdGuiItem::save(), KStdGuiItem::discard())) {
       case KMessageBox::Yes :
         r_esult=CRsave;
         break;
@@ -891,9 +922,9 @@ void KNComposer::initData(const QString &text)
   if(textContent)
     c_harset=textContent->contentType()->charset();
   else
-    c_harset=knGlobals.cfgManager->postNewsTechnical()->charset();
+    c_harset=knGlobals.configManager()->postNewsTechnical()->charset();
 
-  a_ctSetCharset->setCurrentItem(knGlobals.cfgManager->postNewsTechnical()->indexForCharset(c_harset));
+  a_ctSetCharset->setCurrentItem(knGlobals.configManager()->postNewsTechnical()->indexForCharset(c_harset));
 
   // initialize the message type select action
   if (a_rticle->doPost() && a_rticle->doMail())
@@ -1125,14 +1156,14 @@ void KNComposer::slotToggleDoMail()
 {
   if (a_ctDoMail->isChecked()) {
     if (a_uthorDislikesMailCopies) {
-      if (!(KMessageBox::warningContinueCancel(this, i18n("The poster doesn't want a mail copy of your reply (Mail-Copies-To: nobody).\nPlease respect his request."),
+      if (!(KMessageBox::warningContinueCancel(this, i18n("The poster does not want a mail copy of your reply (Mail-Copies-To: nobody);\nplease respect their request."),
                                                QString::null, i18n("&Send Copy")) == KMessageBox::Continue)) {
         a_ctDoMail->setChecked(false); //revert
         return;
       }
     }
 
-    if (knGlobals.cfgManager->postNewsTechnical()->useExternalMailer()) {
+    if (knGlobals.configManager()->postNewsTechnical()->useExternalMailer()) {
       QString s = v_iew->e_dit->textLine(0);
       if (!s.contains(i18n("<posted & mailed>")))
         v_iew->e_dit->insertAt(i18n("<posted & mailed>\n\n"),0,0);
@@ -1192,8 +1223,7 @@ void KNComposer::slotToggleWordWrap()
 
 void KNComposer::slotUndoRewrap()
 {
-  if (KMessageBox::warningContinueCancel( this, i18n("This will replace all text you have written!"),
-                                         QString::null, QString::null) == KMessageBox::Continue) {
+  if (KMessageBox::warningContinueCancel( this, i18n("This will replace all text you have written.")) == KMessageBox::Continue) {
     v_iew->e_dit->setText(u_nwraped);
     slotAppendSig();
   }
@@ -1204,7 +1234,7 @@ void KNComposer::slotExternalEditor()
   if(e_xternalEditor)   // in progress...
     return;
 
-  QString editorCommand=knGlobals.cfgManager->postNewsComposer()->externalEditor();
+  QString editorCommand=knGlobals.configManager()->postNewsComposer()->externalEditor();
 
   if(editorCommand.isEmpty())
     KMessageBox::sorry(this, i18n("No editor configured.\nPlease do this in the settings dialog."));
@@ -1445,13 +1475,13 @@ void KNComposer::slotGroupsBtnClicked()
   KNNntpAccount *nntp=0;
 
   if(id!=-1)
-    nntp=knGlobals.accManager->account(id);
+    nntp=knGlobals.accountManager()->account(id);
 
   if(!nntp)
-    nntp=knGlobals.accManager->first();
+    nntp=knGlobals.accountManager()->first();
 
   if(!nntp) {
-    KMessageBox::error(this, i18n("You have no valid news accounts configured!"));
+    KMessageBox::error(this, i18n("You have no valid news accounts configured."));
     v_iew->g_roups->clear();
     return;
   }
@@ -1462,8 +1492,8 @@ void KNComposer::slotGroupsBtnClicked()
   KNGroupSelectDialog *dlg=new KNGroupSelectDialog(this, nntp, v_iew->g_roups->text().remove(QRegExp("\\s")));
 
   connect(dlg, SIGNAL(loadList(KNNntpAccount*)),
-    knGlobals.grpManager, SLOT(slotLoadGroupList(KNNntpAccount*)));
-  connect(knGlobals.grpManager, SIGNAL(newListReady(KNGroupListData*)),
+    knGlobals.groupManager(), SLOT(slotLoadGroupList(KNNntpAccount*)));
+  connect(knGlobals.groupManager(), SIGNAL(newListReady(KNGroupListData*)),
     dlg, SLOT(slotReceiveList(KNGroupListData*)));
 
   if(dlg->exec())
@@ -1869,7 +1899,7 @@ void KNComposer::ComposerView::showAttachmentView()
             parent(), SLOT(slotAttachmentEdit(QListViewItem*)));
 
     //buttons
-    a_ttAddBtn=new QPushButton(i18n("A&dd"),a_ttWidget);
+    a_ttAddBtn=new QPushButton(i18n("A&dd..."),a_ttWidget);
     connect(a_ttAddBtn, SIGNAL(clicked()), parent(), SLOT(slotAttachFile()));
     topL->addWidget(a_ttAddBtn, 0,1);
 
@@ -2028,16 +2058,12 @@ bool KNComposer::Editor::eventFilter(QObject*o, QEvent* e)
           //Save the cursor position
           int parIdx = 1, txtIdx = 1;
           getCursorPosition(&parIdx, &txtIdx);
-
-          //Put in our replacement
-          QString txtContents = text();
-          QString newContents = txtContents.left(firstSpace) + m_replacements[word][id] +
-                                txtContents.right( txtContents.length() - lastSpace );
-          setText( newContents );
-
-          //Restore the cursor position
-          if( txtIdx > lastSpace )
-            txtIdx += newContents.length() - txtContents.length();
+          setSelection(para, firstSpace, para, lastSpace);
+          insert(m_replacements[word][id]);
+          // Restore the cursor position; if the cursor was behind the
+          // misspelled word then adjust the cursor position
+          if ( para == parIdx && txtIdx >= lastSpace )
+            txtIdx += m_replacements[word][id].length() - word.length();
           setCursorPosition(parIdx, txtIdx);
         }
         //Cancel original event
@@ -2122,6 +2148,10 @@ void KNComposer::Editor::slotFind()
   search();
 }
 
+void KNComposer::Editor::slotSearchAgain()
+{
+    repeatSearch();
+}
 
 void KNComposer::Editor::slotReplace()
 {
