@@ -2,7 +2,7 @@
     knconfig.h
 
     KNode, the KDE newsreader
-    Copyright (c) 1999-2001 the KNode authors.
+    Copyright (c) 1999-2004 the KNode authors.
     See file AUTHORS for details
 
     This program is free software; you can redistribute it and/or modify
@@ -20,11 +20,12 @@
 #include <qasciidict.h>
 
 #include <kdialogbase.h>
+#include <kcmodule.h>
 
 #include "knwidgets.h"
 
 class QRadioButton;
-class QMultiLineEdit;
+class QTextEdit;
 class QCheckBox;
 class QButtonGroup;
 
@@ -69,12 +70,12 @@ class Base {
 };
 
 
-class BaseWidget : public QWidget {
+class BaseWidget : public KCModule {
 
   Q_OBJECT
 
   public:
-    BaseWidget(QWidget *p=0, const char *n=0) : QWidget(p, n), d_irty(false) {}
+    BaseWidget(QWidget *p=0, const char *n=0) : KCModule(p, n), d_irty(false) {}
     ~BaseWidget() {}
 
     void show()             { d_irty=true; QWidget::show(); }
@@ -82,11 +83,11 @@ class BaseWidget : public QWidget {
     bool dirty()const            { return d_irty; }
     void setDirty(bool b)   { d_irty=b; }
 
-    virtual void apply() {}
-
   protected:
     bool d_irty;
 
+  protected slots:
+    void slotEmitChanged() { emit changed( true ); }
 };
 
 
@@ -165,7 +166,8 @@ class IdentityWidget : public BaseWidget {
     IdentityWidget(Identity *d, QWidget *p=0, const char *n=0);
     ~IdentityWidget();
 
-    void apply();
+    void load();
+    void save();
 
   protected:
     QLabel          *f_ileName;
@@ -180,7 +182,7 @@ class IdentityWidget : public BaseWidget {
     QCheckBox       *s_igGenerator;
     QPushButton     *c_hooseBtn,
                     *e_ditBtn;
-    QMultiLineEdit  *s_igEditor;
+    QTextEdit       *s_igEditor;
     QButtonGroup    *b_uttonGroup;
     Kpgp::SecretKeyRequester
                     *s_igningKey;
@@ -205,6 +207,7 @@ class NntpAccountListWidget : public BaseWidget {
     NntpAccountListWidget(QWidget *p=0, const char *n=0);
     ~NntpAccountListWidget();
 
+    void load();
 
   protected:
     class LBoxItem : public KNListBoxItem {
@@ -287,7 +290,8 @@ Q_OBJECT
     SmtpAccountWidget(QWidget *p=0, const char *n=0);
     ~SmtpAccountWidget();
 
-    void apply();
+    void load();
+    void save();
 
   protected slots:
     void useExternalMailerToggled(bool b);
@@ -320,7 +324,7 @@ class Appearance : public Base {
                         quoted2=5, quoted3=6, url=7, unreadThread=8, readThread=9,
                         unreadArticle=10, readArticle=11, activeItem=12, selectedItem=13 };
 
-    enum HexIndex     { quoted1Hex=0, quoted2Hex=1, quoted3Hex=2, headerHex=3 };
+    enum HexIndex     { quoted1Hex=0, quoted2Hex=1, quoted3Hex=2 };
 
     enum FontIndex    { article=0, articleFixed=1, composer=2, groupList=3, articleList=4 };
 
@@ -350,9 +354,8 @@ class Appearance : public Base {
     QColor activeItemColor();
     QColor selectedItemColor();
 
-    const char* headerDecoHexcode()       { return h_excodes[headerHex]; }
-    const char* quotedTextHexcode(int i)  { return h_excodes[i]; }
-    void updateHexcodes();
+    QString headerDecoHexcode()       { return headerDecoColor().name(); }
+    QString quotedTextHexcode(int i);
 
     QFont articleFont();
     QFont articleFixedFont();
@@ -381,7 +384,6 @@ class Appearance : public Base {
     QString c_olorNames[COL_CNT];
     QFont   f_onts[FNT_CNT];
     QString f_ontNames[FNT_CNT];
-    char    h_excodes[HEX_CNT][8];
     QPixmap i_cons[ICON_CNT];
 
 };
@@ -392,10 +394,12 @@ class AppearanceWidget : public BaseWidget {
   Q_OBJECT
 
   public:
-    AppearanceWidget(Appearance *d, QWidget *p=0, const char *n=0);
+    AppearanceWidget(QWidget *p=0, const char *n=0);
     ~AppearanceWidget();
 
-    void apply();
+    void load();
+    void save();
+    void defaults();
 
     //===================================================================================
     // code taken from KMail, Copyright (C) 2000 Espen Sand, espen@kde.org
@@ -442,9 +446,7 @@ class AppearanceWidget : public BaseWidget {
               *f_List;
     QCheckBox *c_olorCB,
               *f_ontCB;
-    QPushButton *c_olDefBtn,
-                *c_olChngBtn,
-                *f_ntDefBtn,
+    QPushButton *c_olChngBtn,
                 *f_ntChngBtn;
 
     Appearance *d_ata;
@@ -453,14 +455,12 @@ class AppearanceWidget : public BaseWidget {
     //colors
     void slotColCheckBoxToggled(bool b);
     void slotColItemSelected(QListBoxItem *);   // show color dialog for the entry
-    void slotColDefaultBtnClicked();
     void slotColChangeBtnClicked();
     void slotColSelectionChanged();
 
     //fonts
     void slotFontCheckBoxToggled(bool b);
     void slotFontItemSelected(QListBoxItem *);  // show font dialog for the entry
-    void slotFontDefaultBtnClicked();
     void slotFontChangeBtnClicked();
     void slotFontSelectionChanged();
 
@@ -526,7 +526,8 @@ class ReadNewsGeneralWidget : public BaseWidget {
     ReadNewsGeneralWidget(ReadNewsGeneral *d, QWidget *p=0, const char *n=0);
     ~ReadNewsGeneralWidget();
 
-    void apply();
+    void load();
+    void save();
 
   protected:
     QCheckBox   *a_utoCB,
@@ -582,7 +583,8 @@ class ReadNewsNavigationWidget : public BaseWidget {
     ReadNewsNavigationWidget(ReadNewsNavigation *d, QWidget *p=0, const char *n=0);
     ~ReadNewsNavigationWidget();
 
-    void apply();
+    void load();
+    void save();
 
   protected:
     QCheckBox   *e_muKMailCB,
@@ -655,7 +657,8 @@ class ReadNewsViewerWidget : public BaseWidget {
     ReadNewsViewerWidget(ReadNewsViewer *d, QWidget *p=0, const char *n=0);
     ~ReadNewsViewerWidget();
 
-    void apply();
+    void load();
+    void save();
 
   protected:
     QCheckBox   *d_ecoCB,
@@ -715,7 +718,8 @@ class DisplayedHeadersWidget : public BaseWidget {
     DisplayedHeadersWidget(DisplayedHeaders *d, QWidget *p=0, const char *n=0);
     ~DisplayedHeadersWidget();
 
-    void apply();
+    void load();
+    void save();
 
   protected:
 
@@ -804,7 +808,8 @@ class ScoringWidget : public BaseWidget {
     ScoringWidget(Scoring *d, QWidget *p=0, const char *n=0);
     ~ScoringWidget();
 
-    void apply();
+    void load();
+    void save();
 
   private:
     KScoringEditorWidget *ksc;
@@ -823,7 +828,8 @@ class FilterListWidget : public BaseWidget {
     FilterListWidget(QWidget *p=0, const char *n=0);
     ~FilterListWidget();
 
-    void apply();
+    void load();
+    void save();
 
     void addItem(KNArticleFilter *f);
     void removeItem(KNArticleFilter *f);
@@ -950,7 +956,8 @@ class PostNewsTechnicalWidget : public BaseWidget {
     PostNewsTechnicalWidget(PostNewsTechnical *d, QWidget *p=0, const char *n=0);
     ~PostNewsTechnicalWidget();
 
-    void apply();
+    void load();
+    void save();
 
   protected:
     QComboBox   *c_harset,
@@ -1039,7 +1046,8 @@ class PostNewsComposerWidget : public BaseWidget {
     PostNewsComposerWidget(PostNewsComposer *d, QWidget *p=0, const char *n=0);
     ~PostNewsComposerWidget();
 
-    void apply();
+    void load();
+    void save();
 
   protected:
     KIntSpinBox *m_axLen;
@@ -1066,7 +1074,7 @@ class PostNewsSpellingWidget : public BaseWidget {
     PostNewsSpellingWidget(QWidget *p=0, const char *n=0);
     ~PostNewsSpellingWidget();
 
-    void apply();
+    void save();
 
   protected:
      KSpellConfig *c_onf;
@@ -1082,7 +1090,9 @@ class PrivacyWidget : public BaseWidget {
   public:
     PrivacyWidget(QWidget *p=0, const char *n=0);
     ~PrivacyWidget();
-    void apply();   // overrides BaseWidget::apply()
+
+    void load();
+    void save();
 
   protected:
     Kpgp::Config *c_onf;
@@ -1131,10 +1141,11 @@ class CleanupWidget : public BaseWidget {
   Q_OBJECT
 
   public:
-    CleanupWidget(Cleanup *d, QWidget *p=0, const char *n=0);
+    CleanupWidget(QWidget *p=0, const char *n=0);
     ~CleanupWidget();
 
-    void apply();
+    void load();
+    void save();
 
   protected:
     QCheckBox   *f_olderCB,
