@@ -15,13 +15,14 @@
 #include "knmainwidget.h"
 
 #include <qhbox.h>
+#include <qlayout.h>
+#include <ktoolbar.h>
 
 #include <kinputdialog.h>
 #include <kaccel.h>
 #include <kxmlguiclient.h>
 #include <kconfig.h>
 #include <kmessagebox.h>
-#include <kkeydialog.h>
 #include <kedittoolbar.h>
 #include <kstdaction.h>
 #include <kdebug.h>
@@ -30,12 +31,17 @@
 #include <kstatusbar.h>
 #include <klocale.h>
 #include <kapplication.h>
+
+#include "broadcaststatus.h"
 #include "recentaddresses.h"
+using KPIM::BroadcastStatus;
 using KRecentAddress::RecentAddresses;
+
 //GUI
 #include "knmainwidget.h"
 #include "knarticlewidget.h"
 #include "knarticlewindow.h"
+#include "kncollectionview.h"
 #include "kncollectionviewitem.h"
 #include "knhdrviewitem.h"
 
@@ -58,6 +64,8 @@ using KRecentAddress::RecentAddresses;
 #include "knmemorymanager.h"
 #include <kcmdlineargs.h>
 
+#include <klistviewsearchline_pimcopy.h>
+
 KNGlobals knGlobals;
 
 KNMainWidget::KNMainWidget( KXMLGUIClient* client, bool detachable, QWidget* parent,
@@ -70,8 +78,7 @@ KNMainWidget::KNMainWidget( KXMLGUIClient* client, bool detachable, QWidget* par
   knGlobals.topWidget=this;
 
   //------------------------------- <CONFIG> ----------------------------------
-  c_fgManager = new KNConfigManager();
-  knGlobals.cfgManager=c_fgManager;
+  c_fgManager = knGlobals.configManager();
   //------------------------------- </CONFIG> ----------------------------------
 
   //-------------------------------- <GUI> ------------------------------------
@@ -89,7 +96,7 @@ KNMainWidget::KNMainWidget( KXMLGUIClient* client, bool detachable, QWidget* par
   }
   KDockWidgetHeader *header = new KDockWidgetHeader(a_rtDock, "artDockHeader");
   a_rtDock->setHeader(header);
-  a_rtView=new KNArticleWidget( actionCollection(), a_rtDock ,"artView");
+  a_rtView = new KNArticleWidget(actionCollection(), knGlobals.guiClient, a_rtDock ,"artView");
   header->setDragPanel(new KNDockWidgetHeaderDrag(a_rtView, header, a_rtDock));
   knGlobals.artWidget=a_rtView;
   a_rtDock->setWidget(a_rtView);
@@ -111,28 +118,11 @@ KNMainWidget::KNMainWidget( KXMLGUIClient* client, bool detachable, QWidget* par
   }
   header = new KDockWidgetHeader(c_olDock, "colDockHeader");
   c_olDock->setHeader(header);
-  c_olView=new KNListView( this, "collectionView");
+  c_olView = new KNCollectionView(this, "collectionView");
   header->setDragPanel(new KNDockWidgetHeaderDrag(c_olView, header, c_olDock));
   c_olDock->setWidget(c_olView);
   c_olDock->manualDock(a_rtDock, KDockWidget::DockLeft, 3000);
 
-  c_olView->setAcceptDrops(true);
-  c_olView->setDragEnabled(true);
-  c_olView->addAcceptableDropMimetype("x-knode-drag/article", false);
-  c_olView->addAcceptableDropMimetype("x-knode-drag/folder", true);
-  c_olView->setSelectionModeExt(KListView::Single);
-  c_olView->setFrameStyle(QFrame::Panel | QFrame::Plain);
-  c_olView->setLineWidth(1);
-  c_olView->setTreeStepSize(12);
-  c_olView->setRootIsDecorated(true);
-  c_olView->setShowSortIndicator(true);
-  c_olView->addColumn(i18n("Name"),162);
-  c_olView->addColumn(i18n("Total"),36);
-  c_olView->addColumn(i18n("Unread"),48);
-  c_olView->setColumnAlignment(1,AlignCenter);
-  c_olView->setColumnAlignment(2,AlignCenter);
-  c_olView->setAlternateBackground(QColor());
-
   connect(c_olDock, SIGNAL(iMBeingClosed()), SLOT(slotGroupDockHidden()));
   connect(c_olDock, SIGNAL(hasUndocked()), SLOT(slotGroupDockHidden()));
   connect(c_olView, SIGNAL(focusChangeRequest(QWidget *)), SLOT(slotDockWidgetFocusChangeRequest(QWidget *)));
@@ -161,11 +151,34 @@ KNMainWidget::KNMainWidget( KXMLGUIClient* client, bool detachable, QWidget* par
   }
   header = new KDockWidgetHeader(h_drDock, "headerDockHeader");
   h_drDock->setHeader(header);
-  h_drView=new KNListView(h_drDock, "hdrView");
+  QWidget *dummy = new QWidget(h_drDock);
+  QVBoxLayout *vlay = new QVBoxLayout(dummy);
+  h_drView=new KNListView(dummy, "hdrView");
   header->setDragPanel(new KNDockWidgetHeaderDrag(h_drView, header, h_drDock));
-  h_drDock->setWidget(h_drView);
+  h_drDock->setWidget(dummy);
   h_drDock->manualDock(a_rtDock, KDockWidget::DockTop, 5000);
 
+  q_uicksearch = new KToolBar(dummy, "search toolbar");
+  KAction *resetQuickSearch = new KAction( i18n( "Reset Quick Search" ),
+                                           QApplication::reverseLayout()
+                                           ? "clear_left"
+                                           : "locationbar_erase",
+                                           0, actionCollection(),
+                                           "reset_quicksearch" );
+  resetQuickSearch->plug( q_uicksearch );
+  resetQuickSearch->setWhatsThis( i18n( "<b>Reset Quick Search</b><br>"
+                                        "Resets the quick search so that "
+                                        "all messages are shown again." ) );
+
+  QLabel *lbl = new QLabel(i18n("&Search:"), q_uicksearch, "kde toolbar widget");
+  s_earchLineEdit = new KPIM::KListViewSearchLine(q_uicksearch, h_drView, "KListViewSearchLine");
+  q_uicksearch->setStretchableWidget(s_earchLineEdit);
+  lbl->setBuddy(s_earchLineEdit);
+  connect( resetQuickSearch, SIGNAL( activated() ), s_earchLineEdit, SLOT( clear() ));
+
+  vlay->addWidget(q_uicksearch);
+  vlay->addWidget(h_drView);
+
   h_drView->setAcceptDrops(false);
   h_drView->setDragEnabled(true);
   h_drView->setSelectionModeExt(KListView::Extended);
@@ -209,43 +222,37 @@ KNMainWidget::KNMainWidget( KXMLGUIClient* client, bool detachable, QWidget* par
   //-------------------------------- <CORE> ------------------------------------
 
   //Network
-  n_etAccess=new KNNetAccess();
+  n_etAccess = knGlobals.netAccess();
   connect(n_etAccess, SIGNAL(netActive(bool)), this, SLOT(slotNetworkActive(bool)));
-  knGlobals.netAccess=n_etAccess;
 
   //Filter Manager
-  f_ilManager=new KNFilterManager(a_ctArtFilter, a_ctArtFilterKeyb);
-  knGlobals.filManager=f_ilManager;
+  f_ilManager = knGlobals.filterManager();
+  f_ilManager->setMenuAction(a_ctArtFilter, a_ctArtFilterKeyb);
 
   //Article Manager
-  a_rtManager=new KNArticleManager(h_drView, f_ilManager);
-  knGlobals.artManager=a_rtManager;
+  a_rtManager = knGlobals.articleManager();
+  a_rtManager->setView(h_drView);
 
   //Group Manager
-  g_rpManager=new KNGroupManager(a_rtManager);
-  knGlobals.grpManager=g_rpManager;
+  g_rpManager = knGlobals.groupManager();
 
   //Folder Manager
-  f_olManager=new KNFolderManager(c_olView, a_rtManager);
-  knGlobals.folManager=f_olManager;
+  f_olManager = knGlobals.folderManager();
 
   //Account Manager
-  a_ccManager=new KNAccountManager(g_rpManager, c_olView);
-  knGlobals.accManager=a_ccManager;
+  a_ccManager = knGlobals.accountManager();
 
   //Article Factory
   a_rtFactory=new KNArticleFactory();
   knGlobals.artFactory=a_rtFactory;
 
   // Score Manager
-  s_coreManager = new KNScoringManager();
-  knGlobals.scoreManager = s_coreManager;
+  s_coreManager = knGlobals.scoringManager();
   //connect(s_coreManager, SIGNAL(changedRules()), SLOT(slotReScore()));
   connect(s_coreManager, SIGNAL(finishedEditing()), SLOT(slotReScore()));
 
   // Memory Manager
-  m_emManager = new KNMemoryManager();
-  knGlobals.memManager = m_emManager;
+  m_emManager = knGlobals.memoryManager();
 
   // create a global pgp instance
   p_gp = new Kpgp::Module();
@@ -307,9 +314,6 @@ KNMainWidget::~KNMainWidget()
   delete c_fgManager;
   kdDebug(5003) << "KNMainWidget::~KNMainWidget() : Config deleted" << endl;
 
-  delete s_coreManager;
-  kdDebug(5003) << "KNMainWidget::~KNMainWidget() : Score Manager deleted" << endl;
-
   delete m_emManager;
   kdDebug(5003) << "KNMainWidget::~KNMainWidget() : Memory Manager deleted" << endl;
 
@@ -326,10 +330,6 @@ void KNMainWidget::initStatusBar()
   //statusbar
   KMainWindow *mainWin = dynamic_cast<KMainWindow*>(topLevelWidget());
   KStatusBar *sb =  mainWin ? mainWin->statusBar() : 0;
-  p_rogBar = new KNProgress( sb->sizeHint().height()-4, 1000, 0, sb );
-  knGlobals.progressBar = p_rogBar;
-  s_tatusMain = new KStatusBarLabel( QString::null, SB_MAIN, sb );
-  s_tatusMain->setAlignment( AlignLeft | AlignVCenter );
   s_tatusFilter = new KStatusBarLabel( QString::null, SB_FILTER, sb );
   s_tatusFilter->setAlignment( AlignLeft | AlignVCenter );
   s_tatusGroup = new KStatusBarLabel( QString::null, SB_GROUP, sb );
@@ -346,10 +346,10 @@ void KNMainWidget::setStatusMsg(const QString& text, int id)
     return;
   bar->clear();
   if (text.isEmpty() && (id==SB_MAIN))
-    if (knGlobals.netAccess->currentMsg().isEmpty())
-      s_tatusMain->setText(i18n(" Ready"));
+    if (knGlobals.netAccess()->currentMsg().isEmpty())
+      BroadcastStatus::instance()->setStatusMsg(i18n(" Ready"));
     else
-      s_tatusMain->setText(knGlobals.netAccess->currentMsg());
+      BroadcastStatus::instance()->setStatusMsg(knGlobals.netAccess()->currentMsg());
   else if ( id == SB_GROUP ) {
     int statusWidth = s_tatusGroup->width();
     QString mtext = text;
@@ -361,7 +361,7 @@ void KNMainWidget::setStatusMsg(const QString& text, int id)
   else {
     switch(id) {
       case SB_MAIN:
-        s_tatusMain->setText(text); break;
+        BroadcastStatus::instance()->setStatusMsg(text); break;
       case SB_GROUP:
         s_tatusGroup->setText(text); break;
       case SB_FILTER:
@@ -477,7 +477,7 @@ void KNMainWidget::openURL(const KURL &url)
 
     // lets see if we already have an account for this host...
     for(acc=a_ccManager->first(); acc; acc=a_ccManager->next())
-      if( acc->server()==host && (port==0 || acc->port()==port) )
+      if( acc->server().lower()==host.lower() && (port==0 || acc->port()==port) )
         break;
 
     if(!acc) {
@@ -575,7 +575,7 @@ void KNMainWidget::configChanged()
   h_drView->setPalette(p);
   h_drView->setAlternateBackground(app->alternateBackgroundColor());
 
-  if (knGlobals.cfgManager->readNewsGeneral()->showScore()) {
+  if (knGlobals.configManager()->readNewsGeneral()->showScore()) {
     if (!h_drView->header()->isResizeEnabled(2)) {
       h_drView->header()->setResizeEnabled(true,2);
       h_drView->header()->setLabel(2,i18n("Score"),42);
@@ -584,7 +584,7 @@ void KNMainWidget::configChanged()
     h_drView->header()->setLabel(2,QString::null,0);
     h_drView->header()->setResizeEnabled(false,2);
   }
-  if (knGlobals.cfgManager->readNewsGeneral()->showLines()) {
+  if (knGlobals.configManager()->readNewsGeneral()->showLines()) {
     if (!h_drView->header()->isResizeEnabled(3)) {
       h_drView->header()->setResizeEnabled(true,3);
       h_drView->header()->setLabel(3,i18n("Lines"),42);
@@ -621,7 +621,7 @@ void KNMainWidget::initActions()
   a_ctNavReadThrough->plugAccel(a_ccel);
 
   //collection-view - accounts
-  a_ctAccProperties         = new KAction(i18n("Account &Properties..."), "configure", 0, this,
+  a_ctAccProperties         = new KAction(i18n("Account &Properties"), "configure", 0, this,
                               SLOT(slotAccProperties()), actionCollection(), "account_properties");
   a_ctAccRename             = new KAction(i18n("&Rename Account"), "text", 0, this,
                               SLOT(slotAccRename()), actionCollection(), "account_rename");
@@ -631,13 +631,15 @@ void KNMainWidget::initActions()
                               SLOT(slotAccExpireAll()), actionCollection(), "account_expire_all");
   a_ctAccGetNewHdrs         = new KAction(i18n("&Get New Articles in All Groups"), "mail_get", 0, this,
                               SLOT(slotAccGetNewHdrs()), actionCollection(), "account_dnlHeaders");
+  a_ctAccGetNewHdrsAll      = new KAction(i18n("&Get New Articles in All Accounts"), "mail_get", 0, this,
+                                          SLOT(slotAccGetNewHdrsAll()), actionCollection(), "account_dnlAllHeaders");
   a_ctAccDelete             = new KAction(i18n("&Delete Account"), "editdelete", 0, this,
                               SLOT(slotAccDelete()), actionCollection(), "account_delete");
   a_ctAccPostNewArticle     = new KAction(i18n("&Post to Newsgroup..."), "mail_new", Key_P , this,
                               SLOT(slotAccPostNewArticle()), actionCollection(), "article_postNew");
 
   //collection-view - groups
-  a_ctGrpProperties         = new KAction(i18n("Group &Properties..."), "configure", 0, this,
+  a_ctGrpProperties         = new KAction(i18n("Group &Properties"), "configure", 0, this,
                               SLOT(slotGrpProperties()), actionCollection(), "group_properties");
   a_ctGrpRename             = new KAction(i18n("Rename &Group"), "text", 0, this,
                               SLOT(slotGrpRename()), actionCollection(), "group_rename");
@@ -662,10 +664,6 @@ void KNMainWidget::initActions()
                        "configure", 0, this,
                        SLOT(slotSettings()), actionCollection(),
                        "knode_configure_knode" );
-   (void) new KAction( i18n("Configure S&hortcuts..."),
-                       "configure_shortcuts", 0, this,
-                       SLOT(slotConfKeys()), actionCollection(),
-                       "knode_configure_shortcuts" );
 
   //collection-view - folder
   a_ctFolNew                = new KAction(i18n("&New Folder"), "folder_new", 0, this,
@@ -708,7 +706,7 @@ void KNMainWidget::initActions()
   a_ctArtFilterKeyb->plugAccel(a_ccel);
   a_ctArtSearch             = new KAction(i18n("&Search Articles..."),"mail_find" , Key_F4 , this,
                               SLOT(slotArtSearch()), actionCollection(), "article_search");
-  a_ctArtFind               = new KAction(i18n("F&ind in Article..."),KStdAccel::shortcut(KStdAccel::Find) , this,
+  a_ctArtFind               = new KAction(i18n("F&ind in Article..."),"find", KStdAccel::shortcut(KStdAccel::Find) , this,
                               SLOT(slotArtFind()), actionCollection(), "find_in_article");
   a_ctArtRefreshList        = new KAction(i18n("&Refresh List"),"reload", KStdAccel::shortcut(KStdAccel::Reload), this,
                               SLOT(slotArtRefreshList()), actionCollection(), "view_Refresh");
@@ -720,6 +718,10 @@ void KNMainWidget::initActions()
                               SLOT(slotArtToggleThread()), actionCollection(), "thread_toggle");
   a_ctArtToggleShowThreads  = new KToggleAction(i18n("Show T&hreads"), 0 , this,
                               SLOT(slotArtToggleShowThreads()), actionCollection(), "view_showThreads");
+#if KDE_IS_VERSION(3,2,90)
+  a_ctArtToggleShowThreads->setCheckedState(i18n("Hide T&hreads"));
+#endif
+
   a_ctArtToggleShowThreads->setChecked(c_fgManager->readNewsGeneral()->showThreads());
 
   //header-view - remote articles
@@ -773,6 +775,14 @@ void KNMainWidget::initActions()
                                SLOT(slotToggleHeaderView()), actionCollection(), "settings_show_headerView");
   a_ctToggleArticleViewer    = new KToggleAction(i18n("Show &Article Viewer"), CTRL+Key_J, this,
                                SLOT(slotToggleArticleViewer()), actionCollection(), "settings_show_articleViewer");
+  a_ctToggleQuickSearch      = new KToggleAction(i18n("Show Quick Search"), QString::null, this,
+                               SLOT(slotToggleQuickSearch()), actionCollection(), "settings_show_quickSearch");
+#if KDE_IS_VERSION(3,2,90)
+  a_ctToggleGroupView->setCheckedState(i18n("Hide &Group View"));
+  a_ctToggleHeaderView->setCheckedState(i18n("Hide &Header View"));
+  a_ctToggleArticleViewer->setCheckedState(i18n("Hide &Article Viewer"));
+  a_ctToggleQuickSearch->setCheckedState(i18n("Hide Quick Search"));
+#endif
   a_ctSwitchToGroupView      = new KAction(i18n("Switch to Group View"), Key_G , this,
                                SLOT(slotSwitchToGroupView()), actionCollection(), "switch_to_group_view");
   a_ctSwitchToGroupView->plugAccel(a_ccel);
@@ -798,14 +808,14 @@ bool KNMainWidget::firstStart()
   QString group = emailConf.readEntry("Profile","Default");
 
   emailConf.setGroup(QString("PROFILE_%1").arg(group));
-  KNConfig::Identity *id=knGlobals.cfgManager->identity();
+  KNConfig::Identity *id=knGlobals.configManager()->identity();
   id->setName(emailConf.readEntry("FullName"));
   id->setEmail(emailConf.readEntry("EmailAddress").latin1());
   id->setOrga(emailConf.readEntry("Organization"));
   id->setReplyTo(emailConf.readEntry("ReplyAddr"));
   id->save();
 
-  KNServerInfo *smtp=knGlobals.accManager->smtp();
+  KNServerInfo *smtp=knGlobals.accountManager()->smtp();
   smtp->setServer(emailConf.readEntry("OutgoingServer").latin1());
   smtp->setPort(25);
   conf->setGroup("MAILSERVER");
@@ -856,6 +866,10 @@ void KNMainWidget::readOptions()
     }
   }
 
+  if (conf->readBoolEntry("quicksearch", true))
+    a_ctToggleQuickSearch->setChecked(true);
+  else
+    q_uicksearch->hide();
   int sortCol=conf->readNumEntry("sortCol",4);
   bool sortAsc=conf->readBoolEntry("sortAscending", false);
   bool sortByThreadChangeDate=conf->readBoolEntry("sortByThreadChangeDate", false);
@@ -914,7 +928,7 @@ void KNMainWidget::saveOptions()
   conf->writeEntry("sortByThreadChangeDate", h_drView->sortByThreadChangeDate());
   conf->writeEntry("account_sortCol", c_olView->sortColumn());
   conf->writeEntry("account_sortAscending", c_olView->ascending());
-
+  conf->writeEntry("quicksearch", q_uicksearch->isShown());
   //saveMainWindowSettings(KGlobal::config(),"mainWindow_options");
 
   // store dock configuration
@@ -1012,7 +1026,6 @@ void KNMainWidget::fontChange( const QFont & )
 
 void KNMainWidget::paletteChange( const QPalette & )
 {
-  c_fgManager->appearance()->updateHexcodes();
   KNArticleWidget::configChanged();
   configChanged();
 }
@@ -1087,7 +1100,6 @@ void KNMainWidget::closeCurrentThread()
   }
 }
 
-
 void KNMainWidget::slotArticleSelected(QListViewItem *i)
 {
   kdDebug(5003) << "KNMainWidget::slotArticleSelected(QListViewItem *i)" << endl;
@@ -1160,6 +1172,7 @@ void KNMainWidget::slotCollectionSelected(QListViewItem *i)
   KNGroup *selectedGroup=0;
   KNFolder *selectedFolder=0;
 
+  s_earchLineEdit->clear();
   h_drView->clear();
   slotArticleSelected(0);
 
@@ -1220,6 +1233,7 @@ void KNMainWidget::slotCollectionSelected(QListViewItem *i)
     a_ctAccSubscribe->setEnabled(enabled);
     a_ctAccExpireAll->setEnabled(enabled);
     a_ctAccGetNewHdrs->setEnabled(enabled);
+    a_ctAccGetNewHdrsAll->setEnabled(enabled);
     a_ctAccDelete->setEnabled(enabled);
     a_ctAccPostNewArticle->setEnabled(enabled);
   }
@@ -1696,6 +1710,13 @@ void KNMainWidget::slotAccDelete()
   }
 }
 
+void KNMainWidget::slotAccGetNewHdrsAll()
+{
+  kdDebug(5003) << "KNMainWindow::slotAccGetNewHdrsAll()" << endl;
+
+  for (KNNntpAccount *account = a_ccManager->first(); account; account = a_ccManager->next() )
+    g_rpManager->checkAll(account);
+}
 
 void KNMainWidget::slotAccPostNewArticle()
 {
@@ -1831,8 +1852,8 @@ void KNMainWidget::slotFolDelete()
   if(f_olManager->currentFolder()->isStandardFolder())
     KMessageBox::sorry(knGlobals.topWidget, i18n("You cannot delete a standard folder."));
 
-  else if( KMessageBox::Yes==KMessageBox::questionYesNo(knGlobals.topWidget,
-      i18n("Do you really want to delete this folder and all its children?")) ) {
+  else if( KMessageBox::Continue==KMessageBox::warningContinueCancel(knGlobals.topWidget,
+      i18n("Do you really want to delete this folder and all its children?"),"",KGuiItem(i18n("&Delete"),"editdelete")) ) {
 
     if(!f_olManager->deleteFolder(f_olManager->currentFolder()))
       KMessageBox::sorry(knGlobals.topWidget,
@@ -1882,8 +1903,8 @@ void KNMainWidget::slotFolEmpty()
       i18n("This folder cannot be emptied at the moment\nbecause some of its articles are currently in use.") );
       return;
     }
-    if( KMessageBox::Yes == KMessageBox::questionYesNo(
-        this, i18n("Do you really want to delete all articles in %1?").arg(f_olManager->currentFolder()->name())) )
+    if( KMessageBox::Continue == KMessageBox::warningContinueCancel(
+        this, i18n("Do you really want to delete all articles in %1?").arg(f_olManager->currentFolder()->name()),"",KGuiItem(i18n("&Delete"),"editdelete")) )
       f_olManager->emptyFolder(f_olManager->currentFolder());
   }
 }
@@ -2243,6 +2264,13 @@ void KNMainWidget::slotToggleArticleViewer()
   slotCheckDockWidgetStatus();
 }
 
+void KNMainWidget::slotToggleQuickSearch()
+{
+  if (q_uicksearch->isHidden())
+    q_uicksearch->show();
+  else
+    q_uicksearch->hide();
+}
 
 void KNMainWidget::slotSwitchToGroupView()
 {
@@ -2267,14 +2295,6 @@ void KNMainWidget::slotSwitchToArticleViewer()
 }
 
 
-void KNMainWidget::slotConfKeys()
-{
-  //KKeyDialog::configureKeys(actionCollection(), xmlFile(), true, this);
-  KKeyDialog::configure( actionCollection(),
-			 true /*allow one-letter shortcuts*/
-    );
-}
-
 void KNMainWidget::slotSettings()
 {
   c_fgManager->configure();
@@ -2287,7 +2307,7 @@ KActionCollection* KNMainWidget::actionCollection() const
 
 KXMLGUIFactory* KNMainWidget::factory() const
 {
-  kdDebug()<<"m_guiclient is "<< m_GUIClient
+  kdDebug(5003)<<"m_guiclient is "<< m_GUIClient
            <<", the factory is " << m_GUIClient->factory() <<endl;
   return m_GUIClient->factory();
 }
@@ -2305,6 +2325,7 @@ FetchArticleIdDlg::FetchArticleIdDlg(QWidget *parent, const char */*name*/ )
   label->setBuddy(edit);
   edit->setFocus();
   enableButtonOK( false );
+  setButtonOKText(i18n("&Fetch"));
   connect( edit, SIGNAL(textChanged( const QString & )), this, SLOT(slotTextChanged(const QString & )));
   KNHelper::restoreWindowSize("fetchArticleWithID", this, QSize(325,66));
 }
