@@ -2,7 +2,7 @@
     knode.h
 
     KNode, the KDE newsreader
-    Copyright (c) 1999-2001 the KNode authors.
+    Copyright (c) 1999-2004 the KNode authors.
     See file AUTHORS for details
 
     This program is free software; you can redistribute it and/or modify
@@ -29,6 +29,13 @@
 
 class KURL;
 
+namespace KPIM { 
+  class StatusbarProgressWidget; 
+  class ProgressDialog; 
+}
+using KPIM::StatusbarProgressWidget;
+using KPIM::ProgressDialog;
+
 class KNMainWidget;
 class KNListView;
 
@@ -46,11 +53,16 @@ public:
 public slots:
   void slotConfToolbar();
   void slotNewToolbarConfig();
+  void slotConfKeys();
 protected:
   bool queryClose();
 private:
   void setupStatusBar();
   KNMainWidget *m_mainWidget;
+  StatusbarProgressWidget *mLittleProgress;
+  ProgressDialog *mProgressDialog;
+private slots:
+  void slotShowStatusMsg( const QString& );
 };
 
 #endif // KNODE_H
