@@ -3,6 +3,7 @@
 
   Copyright (c) 2002 Mike Pilone <mpilone@slac.com>
   Copyright (c) 2002 Don Sanders <sanders@kde.org>
+  Copyright (c) 2004 Cornelius Schumacher <schumacher@kde.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -25,8 +26,27 @@
 
 #include "actionmanager.h"
 
-#include <qapplication.h>
-#include <qtimer.h>
+#include "alarmclient.h"
+#include "calendarview.h"
+#include "kocore.h"
+#include "kodialogmanager.h"
+#include "koglobals.h"
+#include "koprefs.h"
+#include "koviewmanager.h"
+#include "kowindowlist.h"
+#include "kprocess.h"
+#include "konewstuff.h"
+#include "history.h"
+#include "kogroupware.h"
+#include "resourceview.h"
+#include "importdialog.h"
+#include "eventarchiver.h"
+#include "stdcalendar.h"
+
+#include <libkcal/htmlexport.h>
+#include <libkcal/calendarlocal.h>
+#include <libkcal/calendarresources.h>
+#include <libkcal/resourcelocal.h>
 
 #include <dcopclient.h>
 #include <kaction.h>
@@ -40,35 +60,21 @@
 #include <kxmlguiclient.h>
 #include <kwin.h>
 #include <knotifyclient.h>
+#include <kstdguiitem.h>
+#include <kdeversion.h>
 
-#include <libkcal/htmlexport.h>
-#include <libkcal/calendarlocal.h>
-#include <libkcal/calendarresources.h>
-#include <libkcal/resourcelocal.h>
+#include <qapplication.h>
+#include <qtimer.h>
 
-#include "alarmclient.h"
-#include "calendarview.h"
-#include "kocore.h"
-#include "kodialogmanager.h"
-#include "koglobals.h"
-#include "koprefs.h"
-#include "koviewmanager.h"
-#include "kowindowlist.h"
-#include "korganizer.h"
-#include "kprocess.h"
-#include "konewstuff.h"
-#include "history.h"
-#include "kogroupware.h"
-#include "resourceview.h"
-#include "resourceimportdialog.h"
 
 KOWindowList *ActionManager::mWindowList = 0;
 
 ActionManager::ActionManager( KXMLGUIClient *client, CalendarView *widget,
                               QObject *parent, KOrg::MainWindow *mainWindow,
                               bool isPart )
-  : QObject( parent ), KCalendarIface(), mRecent( 0 ), mCalendar( 0 ),
-    mCalendarResources( 0 ), mIsClosing( false )
+  : QObject( parent ), KCalendarIface(), mRecent( 0 ),
+    mResourceButtonsAction( 0 ), mCalendar( 0 ),
+    mCalendarResources( 0 ), mResourceView( 0 ), mIsClosing( false )
 {
   mGUIClient = client;
   mACollection = mGUIClient->actionCollection();
@@ -103,6 +109,9 @@ ActionManager::~ActionManager()
 // see the Note: below for why this method is necessary
 void ActionManager::ActionManager::init()
 {
+  // Construct the groupware object
+  KOGroupware::create( mCalendarView, mCalendarResources );
+
   // add this instance of the window to the static list.
   if ( !mWindowList ) {
     mWindowList = new KOWindowList;
@@ -125,6 +134,12 @@ void ActionManager::ActionManager::init()
     mAutoSaveTimer->start( 1000 * 60 * KOPrefs::instance()->mAutoSaveInterval );
   }
 
+  mAutoArchiveTimer = new QTimer( this );
+  connect( mAutoArchiveTimer, SIGNAL( timeout() ), SLOT( slotAutoArchive() ) );
+  // First auto-archive should be in 5 minutes (like in kmail).
+  if ( KOPrefs::instance()->mAutoArchive )
+    mAutoArchiveTimer->start( 5 * 60 * 1000, true ); // singleshot
+
   setTitle();
 
   connect( mCalendarView, SIGNAL( modifiedChanged( bool ) ), SLOT( setTitle() ) );
@@ -153,14 +168,15 @@ void ActionManager::createCalendarLocal()
 
 void ActionManager::createCalendarResources()
 {
-  mCalendarResources = KOCore::self()->calendarResources();
+  mCalendarResources = KOrg::StdCalendar::self();
 
   CalendarResourceManager *manager = mCalendarResources->resourceManager();
 
   kdDebug(5850) << "CalendarResources used by KOrganizer:" << endl;
   CalendarResourceManager::Iterator it;
   for( it = manager->begin(); it != manager->end(); ++it ) {
-    (*it)->dump();
+    kdDebug() << "  " << (*it)->resourceName() << endl;
+//    (*it)->dump();
   }
 
   setDestinationPolicy();
@@ -168,18 +184,30 @@ void ActionManager::createCalendarResources()
   mCalendarView->setCalendar( mCalendarResources );
   mCalendarView->readSettings();
 
-  // Construct the groupware object
-  KOGroupware::create( mCalendarView, mCalendarResources );
-
   ResourceViewFactory factory( mCalendarResources, mCalendarView );
   mCalendarView->addExtension( &factory );
+  mResourceView = factory.resourceView();
 
   connect( mCalendarResources, SIGNAL( calendarChanged() ),
            mCalendarView, SLOT( slotCalendarChanged() ) );
+  connect( mCalendarResources, SIGNAL( signalErrorMessage( const QString & ) ),
+           mCalendarView, SLOT( showErrorMessage( const QString & ) ) );
 
   connect( mCalendarView, SIGNAL( configChanged() ),
            SLOT( updateConfig() ) );
 
+  mResourceButtonsAction = new KToggleAction( i18n("Show Resource Buttons"), 0,
+                                              this,
+                                              SLOT( toggleResourceButtons() ),
+                                              mACollection,
+                                              "show_resourcebuttons" );
+
+  KConfig *config = KOGlobals::self()->config();
+  config->setGroup("Settings");
+  mResourceButtonsAction->setChecked(
+      config->readBoolEntry( "ResourceButtonsVisible", true ) );
+  toggleResourceButtons();
+
   initCalendar( mCalendarResources );
 }
 
@@ -200,19 +228,18 @@ void ActionManager::initActions()
     if ( mMainWindow->hasDocument() ) {
       new KAction( i18n("&New"), "filenew", CTRL+Key_N, this,
                    SLOT( file_new() ), mACollection, "korganizer_openNew" );
-      new KAction( i18n("&Open"), "fileopen", CTRL+Key_O, this,
-                   SLOT( file_open() ), mACollection, "korganizer_open" );
+      KStdAction::open( this, SLOT( file_open() ), mACollection, "korganizer_open" );
       mRecent = new KRecentFilesAction( i18n("Open &Recent"), 0, 0, this,
                                         SLOT( file_openRecent( const KURL & ) ),
                                         mACollection, "korganizer_openRecent" );
       new KAction( i18n("Re&vert"), "revert", 0, this,
                    SLOT( file_revert() ), mACollection, "korganizer_revert" );
-      new KAction( i18n("Save &As..."), "filesaveas", 0, this,
-                   SLOT( file_saveas() ), mACollection, "korganizer_saveAs" );
-      new KAction( i18n("&Close"), "fileclose", CTRL+Key_W, this,
+      KStdAction::saveAs( this, SLOT( file_saveas() ), mACollection,
+                   "korganizer_saveAs" );
+      KStdAction::close( this,
                    SLOT( file_close() ), mACollection, "korganizer_close" );
     }
-    new KAction( i18n("&Save"), "filesave", CTRL+Key_S, this,
+    KStdAction::save( this,
                  SLOT( file_save() ), mACollection, "korganizer_save" );
   } else {
     KStdAction::openNew(this, SLOT(file_new()), mACollection);
@@ -242,6 +269,9 @@ void ActionManager::initActions()
                                         SLOT(toggleFilterView()),
                                         mACollection,
                                         "show_filter");
+#if KDE_IS_VERSION(3,2,90)
+  mFilterViewAction->setCheckedState(i18n("Hide Filter"));
+#endif
 
   KStdAction::tipOfDay( this, SLOT( showTip() ), mACollection,
                         "help_tipofday" );
@@ -269,16 +299,14 @@ void ActionManager::initActions()
 #endif
 
   if (mIsPart) {
-    new KAction(i18n("&Print..."), "fileprint", CTRL+Key_P, mCalendarView,
-                SLOT(print()), mACollection, "korganizer_print" );
+    KStdAction::print(mCalendarView, SLOT(print()), mACollection, "korganizer_print" );
   } else {
     KStdAction::print(mCalendarView, SLOT(print()), mACollection);
   }
 
 #if 1
   if (mIsPart) {
-    new KAction(i18n("Print Previe&w..."), "filequickprint", 0, mCalendarView,
-                SLOT(printPreview()), mACollection, "korganizer_quickprint" );
+    KStdAction::printPreview(mCalendarView, SLOT(printPreview()), mACollection, "korganizer_quickprint" );
   } else {
     KStdAction::printPreview(mCalendarView, SLOT(printPreview()),
                              mACollection);
@@ -404,28 +432,11 @@ void ActionManager::initActions()
                          mCalendarView,SLOT(deleteIncidence()),
                          mACollection, "delete_incidence");
 
-#if 0
-  action = new KAction(i18n("T&ake over Event"), 0,
-                       mCalendarView,SLOT(takeOverEvent()),
-                       mACollection, "takeover_event");
-  connect(mCalendarView,SIGNAL(eventsSelected(bool)),
-          action,SLOT(setEnabled(bool)));
-  (void)new KAction(i18n("T&ake over Calendar"), 0,
-                    mCalendarView,SLOT(takeOverCalendar()),
-                    mACollection, "takeover_calendar");
-
-  action = new KAction(i18n("&Mail Appointment"), "mail_generic", 0,
-                    mCalendarView,SLOT(action_mail()),
-                    mACollection, "mail_appointment");
-  connect(mCalendarView,SIGNAL(eventsSelected(bool)),
-          action,SLOT(setEnabled(bool)));
-#endif
-
   action = new KAction(i18n("&Make Sub-To-Do Independent"), 0,
                     mCalendarView,SLOT(todo_unsub()),
                     mACollection, "unsub_todo");
   action->setEnabled(false);
-  connect(mCalendarView,SIGNAL(todoSelected(bool)),
+  connect(mCalendarView,SIGNAL(subtodoSelected(bool)),
           action,SLOT(setEnabled(bool)));
 
   // Schedule menu.
@@ -452,7 +463,7 @@ void ActionManager::initActions()
   action->setEnabled(false);
   connect(mCalendarView,SIGNAL(groupEventsSelected(bool)),
           action,SLOT(setEnabled(bool)));
-  action = new KAction(i18n("Cancel"),0,
+  action = new KAction(KStdGuiItem::cancel(),0,
                        mCalendarView,SLOT(schedule_cancel()),
                        mACollection,"cancel");
   action->setEnabled(false);
@@ -475,16 +486,17 @@ void ActionManager::initActions()
   action->setEnabled(false);
   connect(mCalendarView,SIGNAL(groupEventsSelected(bool)),
           action,SLOT(setEnabled(bool)));
-  action = new KAction(i18n("Publish Free Busy Information"),0,
-                       mCalendarView,SLOT(schedule_publish_freebusy()),
-                       mACollection,"publish_freebusy");
-  action->setEnabled(true);
-/*  action = new KAction(i18n("Decline Counter"),0,
-                       mCalendarView,SLOT(schedule_declinecounter()),
-                       mACollection,"declinecounter");
-  connect(mCalendarView,SIGNAL(eventsSelected(bool)),
-          action,SLOT(setEnabled(bool)));
-*/
+
+  action = new KAction( i18n("Mail Free Busy Information"), 0,
+                        mCalendarView, SLOT( mailFreeBusy() ),
+                        mACollection, "mail_freebusy" );
+  action->setEnabled( true );
+
+  action = new KAction( i18n("Upload Free Busy Information"), 0,
+                        mCalendarView, SLOT( uploadFreeBusy() ),
+                        mACollection, "upload_freebusy" );
+  action->setEnabled( true );
+
   if ( !mIsPart ) {
       action = new KAction(i18n("Addressbook"),"contents",0,
                            mCalendarView,SLOT(openAddressbook()),
@@ -572,12 +584,21 @@ void ActionManager::readSettings()
 void ActionManager::writeSettings()
 {
   kdDebug(5850) << "ActionManager::writeSettings" << endl;
+
   KConfig *config = KOGlobals::self()->config();
   mCalendarView->writeSettings();
 
   config->setGroup( "Settings" );
   config->writeEntry( "Filter Visible", mFilterViewAction->isChecked() );
+  if ( mResourceButtonsAction ) {
+    config->writeEntry( "ResourceButtonsVisible",
+                        mResourceButtonsAction->isChecked() );
+  }
   if ( mRecent ) mRecent->saveEntries( config );
+
+  if ( mCalendarResources ) {
+    mCalendarResources->resourceManager()->writeConfig();
+  }
 }
 
 void ActionManager::file_new()
@@ -590,10 +611,11 @@ void ActionManager::file_open()
   KURL url;
   QString defaultPath = locateLocal("data","korganizer/");
   url = KFileDialog::getOpenURL(defaultPath,i18n("*.vcs *.ics|Calendar Files"),
-                                mCalendarView->topLevelWidget());
+                                dialogParent());
 
   if (url.isEmpty()) return;
 
+  // is that URL already opened somewhere else? Activate that window
   KOrg::MainWindow *korg=ActionManager::findInstance(url);
   if ((0 != korg)&&(korg != mMainWindow)) {
     KWin::setActiveWindow(korg->topLevelWidget()->winId());
@@ -615,6 +637,7 @@ void ActionManager::file_openRecent(const KURL& url)
   if (!url.isEmpty()) {
     KOrg::MainWindow *korg=ActionManager::findInstance(url);
     if ((0 != korg)&&(korg != mMainWindow)) {
+		  // already open in a different windows, activate that one
       KWin::setActiveWindow(korg->topLevelWidget()->winId());
       return;
     }
@@ -624,7 +647,7 @@ void ActionManager::file_openRecent(const KURL& url)
 
 void ActionManager::file_import()
 {
-  // eventually, we will need a dialog box to select import type, etc.
+  // TODO: eventually, we will need a dialog box to select import type, etc.
   // for now, hard-coded to ical file, $HOME/.calendar.
   int retVal = -1;
   QString progPath;
@@ -633,7 +656,7 @@ void ActionManager::file_import()
   QString homeDir = QDir::homeDirPath() + QString::fromLatin1("/.calendar");
 
   if (!QFile::exists(homeDir)) {
-    KMessageBox::error(mCalendarView->topLevelWidget(),
+    KMessageBox::error(dialogParent(),
                        i18n("You have no ical file in your home directory.\n"
                             "Import cannot proceed.\n"));
     return;
@@ -656,25 +679,25 @@ void ActionManager::file_import()
     // now we need to MERGE what is in the iCal to the current calendar.
     mCalendarView->openCalendar(tmpfn.name(),1);
     if (!retVal)
-      KMessageBox::information(mCalendarView->topLevelWidget(),
+      KMessageBox::information(dialogParent(),
                                i18n("KOrganizer successfully imported and "
                                     "merged your .calendar file from ical "
                                     "into the currently opened calendar."));
     else
-      KMessageBox::information(mCalendarView->topLevelWidget(),
+      KMessageBox::information(dialogParent(),
                            i18n("KOrganizer encountered some unknown fields while "
                                 "parsing your .calendar ical file, and had to "
-                                "discard them. Please check to see that all "
+                                "discard them; please check to see that all "
                                 "your relevant data was correctly imported."),
-                                 i18n("ICal Import Successful With Warning"));
+                                 i18n("ICal Import Successful with Warning"));
   } else if (retVal == -1) {
-    KMessageBox::error(mCalendarView->topLevelWidget(),
+    KMessageBox::error(dialogParent(),
                          i18n("KOrganizer encountered an error parsing your "
-                              ".calendar file from ical. Import has failed."));
+                              ".calendar file from ical; import has failed."));
   } else if (retVal == -2) {
-    KMessageBox::error(mCalendarView->topLevelWidget(),
-                         i18n("KOrganizer doesn't think that your .calendar "
-                              "file is a valid ical calendar. Import has failed."));
+    KMessageBox::error(dialogParent(),
+                         i18n("KOrganizer does not think that your .calendar "
+                              "file is a valid ical calendar; import has failed."));
   }
   tmpfn.unlink();
 }
@@ -683,7 +706,7 @@ void ActionManager::file_merge()
 {
   KURL url = KFileDialog::getOpenURL(locateLocal("data","korganizer/"),
                                      i18n("*.vcs *.ics|Calendar Files"),
-                                     mCalendarView->topLevelWidget());
+                                     dialogParent());
   openURL(url,true);
 }
 
@@ -711,12 +734,19 @@ void ActionManager::file_save()
   if ( mMainWindow->hasDocument() ) {
     if (mURL.isEmpty()) {
       file_saveas();
+      return;
     } else {
       saveURL();
     }
   } else {
     mCalendarView->calendar()->save();
   }
+
+  // export to HTML
+  if ( KOPrefs::instance()->mHtmlWithSave &&
+       !KOPrefs::instance()->mHtmlExportFile.isNull() ) {
+    exportToHTML();
+  }
 }
 
 void ActionManager::file_close()
@@ -744,33 +774,61 @@ bool ActionManager::openURL(const KURL &url,bool merge)
     return false;
   }
 
-  QString tmpFile;
-  if( KIO::NetAccess::download( url, tmpFile, view() ) ) {
-    kdDebug(5850) << "--- Downloaded to " << tmpFile << endl;
-    bool success = mCalendarView->openCalendar(tmpFile,merge);
-    if (merge) {
-      KIO::NetAccess::removeTempFile(tmpFile);
-      if (success)
-        mMainWindow->showStatusMessage(i18n("Merged calendar '%1'.").arg(url.prettyURL()));
+  if ( url.isLocalFile() ) {
+    mURL = url;
+    mFile = url.path();
+    if ( !KStandardDirs::exists( mFile ) ) {
+      mMainWindow->showStatusMessage( i18n("New calendar '%1'.")
+                                      .arg( url.prettyURL() ) );
+      mCalendarView->setModified();
     } else {
-      if (success) {
-        KIO::NetAccess::removeTempFile(mFile);
-        mURL = url;
-        mFile = tmpFile;
-        KConfig *config = KOGlobals::self()->config();
-        config->setGroup("General");
-        setTitle();
-        kdDebug(5850) << "-- Add recent URL: " << url.prettyURL() << endl;
-        if ( mRecent ) mRecent->addURL(url);
-        mMainWindow->showStatusMessage(i18n("Opened calendar '%1'.").arg(mURL.prettyURL()));
+      bool success = mCalendarView->openCalendar( mFile, merge );
+      if ( success ) {
+        showStatusMessageOpen( url, merge );
       }
     }
-    return success;
+    setTitle();
   } else {
-    QString msg;
-    msg = i18n("Cannot download calendar from '%1'.").arg(url.prettyURL());
-    KMessageBox::error(mCalendarView->topLevelWidget(),msg);
-    return false;
+    QString tmpFile;
+    if( KIO::NetAccess::download( url, tmpFile, view() ) ) {
+      kdDebug(5850) << "--- Downloaded to " << tmpFile << endl;
+      bool success = mCalendarView->openCalendar( tmpFile, merge );
+      if (merge) {
+        KIO::NetAccess::removeTempFile( tmpFile );
+        if ( success )
+          showStatusMessageOpen( url, merge );
+      } else {
+        if ( success ) {
+          KIO::NetAccess::removeTempFile( mFile );
+          mURL = url;
+          mFile = tmpFile;
+          KConfig *config = KOGlobals::self()->config();
+          config->setGroup( "General" );
+          setTitle();
+          kdDebug(5850) << "-- Add recent URL: " << url.prettyURL() << endl;
+          if ( mRecent ) mRecent->addURL( url );
+          showStatusMessageOpen( url, merge );
+        }
+      }
+      return success;
+    } else {
+      QString msg;
+      msg = i18n("Cannot download calendar from '%1'.").arg( url.prettyURL() );
+      KMessageBox::error( dialogParent(), msg );
+      return false;
+    }
+  }
+  return true;
+}
+
+void ActionManager::showStatusMessageOpen( const KURL &url, bool merge )
+{
+  if ( merge ) {
+    mMainWindow->showStatusMessage( i18n("Merged calendar '%1'.")
+                                    .arg( url.prettyURL() ) );
+  } else {
+    mMainWindow->showStatusMessage( i18n("Opened calendar '%1'.")
+                                    .arg( url.prettyURL() ) );
   }
 }
 
@@ -793,7 +851,7 @@ bool ActionManager::saveURL()
 
   if ( ext == ".vcs" ) {
     int result = KMessageBox::warningContinueCancel(
-        mCalendarView->topLevelWidget(),
+        dialogParent(),
         i18n("Your calendar will be saved in iCalendar format. Use "
               "'Export vCalendar' to save in vCalendar format."),
         i18n("Format Conversion"), i18n("Proceed"), "dontaskFormatConversion",
@@ -822,7 +880,7 @@ bool ActionManager::saveURL()
     if ( !KIO::NetAccess::upload( mFile, mURL, view() ) ) {
       QString msg = i18n("Cannot upload calendar to '%1'")
                     .arg( mURL.prettyURL() );
-      KMessageBox::error( mCalendarView->topLevelWidget() ,msg );
+      KMessageBox::error( dialogParent() ,msg );
       return false;
     }
   }
@@ -835,63 +893,65 @@ bool ActionManager::saveURL()
 
   mMainWindow->showStatusMessage(i18n("Saved calendar '%1'.").arg(mURL.prettyURL()));
 
-  // export to HTML
-  if ( KOPrefs::instance()->mHtmlWithSave==true &&
-        !KOPrefs::instance()->mHtmlExportFile.isNull() ) {
-    KURL dest( KOPrefs::instance()->mHtmlExportFile );
-    KCal::HtmlExport mExport( mCalendarView->calendar() );
-    mExport.setEmail( KOPrefs::instance()->email() );
-    mExport.setFullName( KOPrefs::instance()->fullName() );
+  return true;
+}
 
-    KConfig *cfg = KOGlobals::self()->config();
-    cfg->setGroup( "HtmlExport" );
+void ActionManager::exportToHTML()
+{
+  KURL dest( KOPrefs::instance()->mHtmlExportFile );
+  KCal::HtmlExport mExport( mCalendarView->calendar() );
+  //@TODO: Make these items translatable after 3.3.
+  mExport.setTitle( "KOrganizer Calendar" );
+  mExport.setTitleTodo( "KOrganizer To-Do List" );
+  mExport.setCredit( "KOrganizer", "http://korganizer.kde.org" );
+  mExport.setEmail( KOPrefs::instance()->email() );
+  mExport.setFullName( KOPrefs::instance()->fullName() );
 
-    mExport.setMonthViewEnabled( cfg->readBoolEntry( "Month", false ) );
-    mExport.setEventsEnabled( cfg->readBoolEntry( "Event", true ) );
-    mExport.setTodosEnabled( cfg->readBoolEntry( "Todo", true ) );
-    mExport.setCategoriesEventEnabled( cfg->readBoolEntry( "CategoriesEvent", false ) );
-    mExport.setAttendeesEventEnabled( cfg->readBoolEntry( "AttendeesEvent", false ) );
-    mExport.setExcludePrivateEventEnabled( cfg->readBoolEntry( "ExcludePrivateEvent", true ) );
-    mExport.setExcludeConfidentialEventEnabled( cfg->readBoolEntry( "ExcludeConfidentialEvent", true ) );
-    mExport.setCategoriesTodoEnabled( cfg->readBoolEntry( "CategoriesTodo", false ) );
-    mExport.setAttendeesTodoEnabled( cfg->readBoolEntry( "AttendeesTodo", false ) );
-    mExport.setExcludePrivateTodoEnabled( cfg->readBoolEntry( "ExcludePrivateTodo", true ) );
-    mExport.setExcludeConfidentialTodoEnabled( cfg->readBoolEntry( "ExcludeConfidentialTodo", true ) );
-    mExport.setDueDateEnabled( cfg->readBoolEntry( "DueDates", true ) );
-    QDate qd1;
-    qd1 = QDate::currentDate();
-    QDate qd2;
-    qd2 = QDate::currentDate();
-    if ( mExport.monthViewEnabled() )
-      qd2.addMonths( 1 );
-    else
-      qd2.addDays( 7 );
+  KConfig *cfg = KOGlobals::self()->config();
+  cfg->setGroup( "HtmlExport" );
 
-    mExport.setDateRange( qd1, qd2 );
-    QDate cdate=qd1;
-    while (cdate<=qd2)
-    {
-      if ( !KOCore::self()->holiday(cdate).isEmpty() )
-        mExport.addHoliday( cdate, KOCore::self()->holiday(cdate) );
-      cdate = cdate.addDays(1);
-    }
+  mExport.setMonthViewEnabled( cfg->readBoolEntry( "Month", false ) );
+  mExport.setEventsEnabled( cfg->readBoolEntry( "Event", true ) );
+  mExport.setTodosEnabled( cfg->readBoolEntry( "Todo", true ) );
+  mExport.setCategoriesEventEnabled( cfg->readBoolEntry( "CategoriesEvent", false ) );
+  mExport.setAttendeesEventEnabled( cfg->readBoolEntry( "AttendeesEvent", false ) );
+  mExport.setExcludePrivateEventEnabled( cfg->readBoolEntry( "ExcludePrivateEvent", true ) );
+  mExport.setExcludeConfidentialEventEnabled( cfg->readBoolEntry( "ExcludeConfidentialEvent", true ) );
+  mExport.setCategoriesTodoEnabled( cfg->readBoolEntry( "CategoriesTodo", false ) );
+  mExport.setAttendeesTodoEnabled( cfg->readBoolEntry( "AttendeesTodo", false ) );
+  mExport.setExcludePrivateTodoEnabled( cfg->readBoolEntry( "ExcludePrivateTodo", true ) );
+  mExport.setExcludeConfidentialTodoEnabled( cfg->readBoolEntry( "ExcludeConfidentialTodo", true ) );
+  mExport.setDueDateEnabled( cfg->readBoolEntry( "DueDates", true ) );
+  QDate qd1;
+  qd1 = QDate::currentDate();
+  QDate qd2;
+  qd2 = QDate::currentDate();
+  if ( mExport.monthViewEnabled() )
+    qd2.addMonths( 1 );
+  else
+    qd2.addDays( 7 );
 
-    if ( dest.isLocalFile() ) {
-      mExport.save( dest.path() );
-    } else {
-      KTempFile tf;
-      QString tfile = tf.name();
-      tf.close();
-      mExport.save( tfile );
-      if (!KIO::NetAccess::upload( tfile, dest, view() ) ) {
-        KNotifyClient::event ( view()->winId(),
-                               i18n("Could not upload file.") );
-      }
-      tf.unlink();
-    }
+  mExport.setDateRange( qd1, qd2 );
+  QDate cdate=qd1;
+  while (cdate<=qd2) {
+    if ( !KOCore::self()->holiday(cdate).isEmpty() )
+      mExport.addHoliday( cdate, KOCore::self()->holiday(cdate) );
+    cdate = cdate.addDays(1);
   }
 
-  return true;
+  if ( dest.isLocalFile() ) {
+    mExport.save( dest.path() );
+  } else {
+    KTempFile tf;
+    QString tfile = tf.name();
+    tf.close();
+    mExport.save( tfile );
+    if (!KIO::NetAccess::upload( tfile, dest, view() ) ) {
+      KNotifyClient::event ( view()->winId(),
+                            i18n("Could not upload file.") );
+    }
+    tf.unlink();
+  }
 }
 
 bool ActionManager::saveAsURL(const KURL &url)
@@ -952,7 +1012,7 @@ bool ActionManager::saveModifiedURL()
     return saveURL();
   } else {
     int result = KMessageBox::warningYesNoCancel(
-        mCalendarView->topLevelWidget(),
+        dialogParent(),
         i18n("The calendar has been modified.\nDo you want to save it?"),
         QString::null,
         KStdGuiItem::save(), KStdGuiItem::discard());
@@ -981,7 +1041,7 @@ KURL ActionManager::getSaveURL()
 {
   KURL url = KFileDialog::getSaveURL(locateLocal("data","korganizer/"),
                                      i18n("*.vcs *.ics|Calendar Files"),
-                                     mCalendarView->topLevelWidget());
+                                     dialogParent());
 
   if (url.isEmpty()) return url;
 
@@ -1084,20 +1144,20 @@ void ActionManager::configureDateTime()
           SLOT(configureDateTimeFinished(KProcess *)));
 
   if (!proc->start()) {
-      KMessageBox::sorry(mCalendarView->topLevelWidget(),
-        i18n("Couldn't start control module for date and time format."));
+      KMessageBox::sorry(dialogParent(),
+        i18n("Could not start control module for date and time format."));
       delete proc;
   }
 }
 
 void ActionManager::showTip()
 {
-  KTipDialog::showTip(mCalendarView->topLevelWidget(),QString::null,true);
+  KTipDialog::showTip(dialogParent(),QString::null,true);
 }
 
 void ActionManager::showTipOnStart()
 {
-  KTipDialog::showTip(mCalendarView->topLevelWidget());
+  KTipDialog::showTip(dialogParent());
 }
 
 KOrg::MainWindow *ActionManager::findInstance( const KURL &url )
@@ -1121,6 +1181,15 @@ void ActionManager::toggleFilterView()
   mCalendarView->showFilter(visible);
 }
 
+void ActionManager::toggleResourceButtons()
+{
+  bool visible = mResourceButtonsAction->isChecked();
+
+  kdDebug() << "RESOURCE VIEW " << int( mResourceView ) << endl;
+
+  if ( mResourceView ) mResourceView->showButtons( visible );
+}
+
 bool ActionManager::openURL(QString url)
 {
   return openURL(KURL(url));
@@ -1141,25 +1210,16 @@ QString ActionManager::getCurrentURLasString() const
   return mURL.url();
 }
 
-bool ActionManager::deleteEvent(QString uid)
+bool ActionManager::editIncidence( const QString& uid )
+{
+  return mCalendarView->editIncidence(uid);
+}
+
+bool ActionManager::deleteEvent( const QString& uid )
 {
   return mCalendarView->deleteEvent(uid);
 }
 
-bool ActionManager::eventRequest(QString request, QCString receiver,
-                                 QString ical)
-{
-  if( !KOGroupware::instance() ) return false;
-  return KOGroupware::instance()->incomingEventRequest(request, receiver,
-                                                       ical);
-}
-
-bool ActionManager::eventReply( QString ical )
-{
-  if( !KOGroupware::instance() ) return false;
-  return KOGroupware::instance()->incidenceAnswer( ical );
-}
-
 void ActionManager::configureDateTimeFinished(KProcess *proc)
 {
   delete proc;
@@ -1259,6 +1319,12 @@ void ActionManager::openEventEditor( QString summary, QString description,
   mCalendarView->newEvent( summary, description, attachment );
 }
 
+void ActionManager::openEventEditor( QString summary, QString description,
+                                     QString attachment, QStringList attendees )
+{
+  mCalendarView->newEvent( summary, description, attachment, attendees );
+}
+
 void ActionManager::openTodoEditor( QString text )
 {
   mCalendarView->newTodo( text );
@@ -1270,6 +1336,12 @@ void ActionManager::openTodoEditor( QString summary, QString description,
   mCalendarView->newTodo( summary, description, attachment );
 }
 
+void ActionManager::openTodoEditor( QString summary, QString description,
+                                    QString attachment, QStringList attendees )
+{
+  mCalendarView->newTodo( summary, description, attachment, attendees );
+}
+
 void ActionManager::showTodoView()
 {
   mCalendarView->viewManager()->showTodoView();
@@ -1280,6 +1352,16 @@ void ActionManager::showEventView()
   mCalendarView->viewManager()->showEventView();
 }
 
+void ActionManager::goDate( QDate date )
+{
+  mCalendarView->goDate( date );
+}
+
+void ActionManager::goDate( QString date )
+{
+  goDate( KGlobal::locale()->readDate(date) );
+}
+
 void ActionManager::updateUndoAction( const QString &text )
 {
   if ( text.isNull() ) {
@@ -1313,7 +1395,6 @@ bool ActionManager::queryClose()
   if ( mCalendar ) {
     close = saveModifiedURL();
   } else if ( mCalendarResources ) {
-    mCalendarResources->resourceManager()->writeConfig();
     if ( !mIsClosing ) {
       kdDebug(5850) << "!mIsClosing" << endl;
       if ( !saveResourceCalendar() ) return false;
@@ -1325,10 +1406,12 @@ bool ActionManager::queryClose()
 //               mMainWindow, SLOT( close() ) );
     }
     if ( mCalendarResources->isSaving() ) {
-      kdDebug(5850) << "KOrganizer::queryClose(): isSaving" << endl;
+      kdDebug(5850) << "ActionManager::queryClose(): isSaving" << endl;
       close = false;
+      KMessageBox::information( dialogParent(),
+          i18n("Unable to exit. Saving still in progress.") );
     } else {
-      kdDebug(5850) << "KOrganizer::queryClose(): close = true" << endl;
+      kdDebug(5850) << "ActionManager::queryClose(): close = true" << endl;
       close = true;
     }
   } else {
@@ -1365,21 +1448,69 @@ bool ActionManager::saveResourceCalendar()
     if ( (*it)->readOnly() ) continue;
     if ( !(*it)->save() ) {
       int result = KMessageBox::warningContinueCancel( view(),
-        i18n("Saving '%1' failed. Please check, if the resource is "
+        i18n("Saving of '%1' failed. Check that the resource is "
              "properly configured.\nIgnore problem and continue without "
              "saving or cancel save?").arg( (*it)->resourceName() ),
-        i18n("Save Error"), i18n("Don't save") );
+        i18n("Save Error"), KStdGuiItem::dontSave() );
       if ( result == KMessageBox::Cancel ) return false;
     }
   }
   return true;
 }
 
-void ActionManager::importResource( const QString &url )
+void ActionManager::importCalendar( const KURL &url )
 {
-  ResourceImportDialog *dialog;
-  dialog = new ResourceImportDialog( url, mMainWindow->topLevelWidget() );
+  if ( !url.isValid() ) {
+    KMessageBox::error( dialogParent(),
+                        i18n("URL '%1' is invalid.").arg( url.prettyURL() ) );
+    return;
+  }
+
+  ImportDialog *dialog;
+  dialog = new ImportDialog( url, mMainWindow->topLevelWidget() );
+  connect( dialog, SIGNAL( dialogFinished( ImportDialog * ) ),
+           SLOT( slotImportDialogFinished( ImportDialog * ) ) );
+  connect( dialog, SIGNAL( openURL( const KURL &, bool ) ),
+           SLOT( openURL( const KURL &, bool ) ) );
+  connect( dialog, SIGNAL( newWindow( const KURL & ) ),
+           SIGNAL( actionNew( const KURL & ) ) );
+  if ( mResourceView ) {
+    connect( dialog, SIGNAL( resourceAdded( ResourceCalendar * ) ),
+             mResourceView, SLOT( addResourceItem( ResourceCalendar * ) ) );
+  }
+
   dialog->show();
 }
 
+void ActionManager::slotImportDialogFinished( ImportDialog *dlg )
+{
+  dlg->deleteLater();
+  mCalendarView->updateView();
+}
+
+void ActionManager::slotAutoArchivingSettingsModified()
+{
+  if ( KOPrefs::instance()->mAutoArchive )
+    mAutoArchiveTimer->start( 4 * 60 * 60 * 1000, true ); // check again in 4 hours
+  else
+    mAutoArchiveTimer->stop();
+}
+
+void ActionManager::slotAutoArchive()
+{
+  if ( !mCalendarView->calendar() ) // can this happen?
+    return;
+  mAutoArchiveTimer->stop();
+  EventArchiver archiver;
+  connect( &archiver, SIGNAL( eventsDeleted() ), mCalendarView, SLOT( updateView() ) );
+  archiver.runAuto( mCalendarView->calendar(), mCalendarView, false /*no gui*/ );
+  // restart timer with the correct delay (especially useful for the first time)
+  slotAutoArchivingSettingsModified();
+}
+
+QWidget *ActionManager::dialogParent()
+{
+  return mCalendarView->topLevelWidget();
+}
+
 #include "actionmanager.moc"
