@@ -24,21 +24,20 @@
 #ifndef KOEVENTEDITOR_H
 #define KOEVENTEDITOR_H
 
-#include <kdialogbase.h>
-
-#include <qdatetime.h>
-
-#include <libkcal/calendar.h>
-
-#include "koeditorgeneralevent.h"
-#include "koeditordetails.h"
-#include "koeditorrecurrence.h"
 #include "koincidenceeditor.h"
 
+class KOEditorGeneralEvent;
+class KOEditorRecurrence;
+class KOEditorFreeBusy;
+
 class SaveTemplateDialog;
 
-class KOEditorGantt;
+class KOEditorFreeBusy;
 
+namespace KCal {
+class Calendar;
+class Event;
+}
 using namespace KCal;
 
 /**
@@ -55,7 +54,8 @@ class KOEventEditor : public KOIncidenceEditor
     virtual ~KOEventEditor(void);
 
     void init();
-
+    /** This event has been modified externally */
+    void modified (int change=0);
     void reload();
 
     /**
@@ -71,10 +71,15 @@ class KOEventEditor : public KOIncidenceEditor
     */
     void newEvent( const QString &summary, const QString &description,
                    const QString &attachment );
+    /**
+      Edit new event.
+    */
+    void newEvent( const QString &summary, const QString &description,
+                   const QString &attachment, const QStringList &attendees );
     /**
       Edit an existing event.
     */
-    void editEvent( Event * );
+    void editIncidence( Incidence * );
 
     /**
       Set widgets to default values
@@ -94,12 +99,6 @@ class KOEventEditor : public KOIncidenceEditor
     QObject *typeAheadReceiver() const;
 
   signals:
-    void eventAdded( Event * );
-    void eventChanged( Event *oldEvent, Event *newEvent );
-    void eventToBeDeleted( Event * );
-    void eventDeleted( Event * );
-    void deleteAttendee( Incidence * );
-
     void focusReceivedSignal();
 
   protected slots:
@@ -114,7 +113,7 @@ class KOEventEditor : public KOIncidenceEditor
     QString type() { return "Event"; }
     void setupGeneral();
     void setupRecurrence();
-    void setupGanttTab();
+    void setupFreeBusy();
 
     /** Check if the input is valid. */
     bool validateInput();
@@ -129,7 +128,7 @@ class KOEventEditor : public KOIncidenceEditor
 
     KOEditorGeneralEvent *mGeneral;
     KOEditorRecurrence   *mRecurrence;
-    KOEditorGantt        *mGantt;
+    KOEditorFreeBusy     *mFreeBusy;
 };
 
 #endif
