@@ -37,17 +37,23 @@
 KOEventPopupMenu::KOEventPopupMenu()
 {
   mCurrentIncidence = 0;
+  mCurrentDate = QDate();
   mHasAdditionalItems = false;
 
   insertItem (i18n("&Show"),this,SLOT(popupShow()));
   mEditOnlyItems.append(insertItem (i18n("&Edit..."),this,SLOT(popupEdit())));
   mEditOnlyItems.append(insertItem (KOGlobals::self()->smallIcon("editdelete"),i18n("&Delete"),
                                    this,SLOT(popupDelete())));
+  mEditOnlyItems.append( insertSeparator() );
+  mEditOnlyItems.append( insertItem( QIconSet( KOGlobals::self()->smallIcon("bell") ),
+                                     i18n("Toggle Alarm"), this,
+                                     SLOT( popupAlarm() ) ) );
 }
 
-void KOEventPopupMenu::showIncidencePopup(Incidence *incidence)
+void KOEventPopupMenu::showIncidencePopup( Incidence *incidence, const QDate &qd )
 {
   mCurrentIncidence = incidence;
+  mCurrentDate = qd;
   
   if (mCurrentIncidence) {
     // Enable/Disabled menu items only valid for editable events.
@@ -87,3 +93,8 @@ void KOEventPopupMenu::popupDelete()
 {
   if (mCurrentIncidence) emit deleteIncidenceSignal(mCurrentIncidence);
 }
+
+void KOEventPopupMenu::popupAlarm()
+{
+  if (mCurrentIncidence) emit toggleAlarmSignal( mCurrentIncidence );
+}
