@@ -36,13 +36,14 @@
 
 #include "koglobals.h"
 #include "koprefs.h"
+#include "kolistview.h"
 
 #include "searchdialog.h"
 #include "searchdialog.moc"
 
 SearchDialog::SearchDialog(Calendar *calendar,QWidget *parent)
   : KDialogBase(Plain,i18n("Find Events"),User1|Close,User1,parent,0,false,false,
-                i18n("&Find"))
+                KGuiItem( i18n("&Find"), "find") )
 {
   mCalendar = calendar;
 
@@ -69,7 +70,7 @@ SearchDialog::SearchDialog(Calendar *calendar,QWidget *parent)
   mJournalsCheck = new QCheckBox( i18n("&Journal entries"), itemsGroup );
   mEventsCheck->setChecked( true );
   mTodosCheck->setChecked( true );
-  
+
   // Date range
   QGroupBox *rangeGroup = new QGroupBox( 1, Horizontal, i18n( "Date Range" ),
                                         topFrame );
@@ -79,15 +80,15 @@ SearchDialog::SearchDialog(Calendar *calendar,QWidget *parent)
   QHBoxLayout *rangeLayout = new QHBoxLayout( rangeWidget, 0, spacingHint() );
 
   mStartDate = new KDateEdit( rangeWidget );
-  rangeLayout->addWidget( new QLabel( mStartDate, i18n("&From:"), rangeWidget ) );
+  rangeLayout->addWidget( new QLabel( mStartDate, i18n("Fr&om:"), rangeWidget ) );
   rangeLayout->addWidget( mStartDate );
-  
+
   mEndDate = new KDateEdit( rangeWidget );
   rangeLayout->addWidget( new QLabel( mEndDate, i18n("&To:"), rangeWidget ) );
   mEndDate->setDate( QDate::currentDate().addDays( 365 ) );
   rangeLayout->addWidget( mEndDate );
 
-  mInclusiveCheck = new QCheckBox( i18n("Events have to be &completely included"),
+  mInclusiveCheck = new QCheckBox( i18n("E&vents have to be completely included"),
                                   rangeGroup );
   mInclusiveCheck->setChecked( false );
   mIncludeUndatedTodos = new QCheckBox( i18n("Include todos &without due date"), rangeGroup );
@@ -101,7 +102,7 @@ SearchDialog::SearchDialog(Calendar *calendar,QWidget *parent)
   mSummaryCheck->setChecked( true );
   mDescriptionCheck = new QCheckBox( i18n("Desc&riptions"), subjectGroup );
   mCategoryCheck = new QCheckBox( i18n("Cate&gories"), subjectGroup );
-  
+
 
   // Results list view
   listView = new KOListView( mCalendar, topFrame );
@@ -177,28 +178,45 @@ void SearchDialog::search( const QRegExp &re )
 {
   QDate startDt = mStartDate->date();
   QDate endDt = mEndDate->date();
-  
-  Event::List events = mCalendar->events( startDt, endDt,
-                                          mInclusiveCheck->isChecked() );
+
+  Event::List events;
+  if (mEventsCheck->isChecked()) {
+    events = mCalendar->events( startDt, endDt, mInclusiveCheck->isChecked() );
+  }
   Todo::List todos;
-  if ( mIncludeUndatedTodos ) 
-    todos = mCalendar->todos();
-  else {
-    QDate dt = startDt; 
+  if (mTodosCheck->isChecked()) {
+    if ( mIncludeUndatedTodos->isChecked() ) {
+      Todo::List alltodos = mCalendar->todos();
+      Todo::List::iterator it;
+      Todo *todo;
+      for (it=alltodos.begin(); it!=alltodos.end(); ++it) {
+        todo = *it;
+        if ( (!todo->hasStartDate() && !todo->hasDueDate() ) || // undated
+             ( todo->hasStartDate() && (todo->dtStart()>=startDt) && (todo->dtStart()<=endDt) ) || // start dt in range
+             ( todo->hasDueDate() && (todo->dtDue().date()>=startDt) && (todo->dtDue()<=endDt) ) || // due dt in range
+             ( todo->hasCompletedDate() && (todo->completed().date()>=startDt) && (todo->completed()<=endDt) ) ) { // completed dt in range
+          todos.append( todo );
+        }
+      } 
+    } else {
+      QDate dt = startDt;
+      while ( dt <= endDt ) {
+        todos += mCalendar->todos( dt );
+        dt = dt.addDays( 1 );
+      }
+    }
+  }
+
+  Journal::List journals;
+  if (mJournalsCheck->isChecked()) {
+    QDate dt = startDt;
     while ( dt <= endDt ) {
-      todos += mCalendar->todos( dt );
+      Journal* j=mCalendar->journal( dt );
+      if (j) journals.append( j );
       dt = dt.addDays( 1 );
     }
   }
-  
-  Journal::List journals; 
-  QDate dt = startDt; 
-  while ( dt <= endDt ) {
-    Journal* j=mCalendar->journal( dt );
-    if (j) journals.append( j );
-    dt = dt.addDays( 1 );
-  }
-  
+
   Incidence::List allIncidences = Calendar::mergeIncidenceList( events, todos, journals );
 
   mMatchedEvents.clear();
