@@ -30,49 +30,27 @@
 ** Bug reports and questions can be sent to kde-pim@kde.org
 */
 
-static const char *vcalconduitbase_id = "$Id: vcal-conduitbase.cc,v 1.48 2003/10/19 20:38:58 adridg Exp $";
+static const char *vcalconduitbase_id = "$Id: vcal-conduitbase.cc,v 1.70.2.1 2004/09/29 09:20:54 adridg Exp $";
 
 #include <options.h>
-#include <unistd.h>
 
-#include <qdatetime.h>
 #include <qtimer.h>
 #include <qfile.h>
 
-#include <pilotUser.h>
-#include <kconfig.h>
 #include <kmessagebox.h>
+#include <kio/netaccess.h>
 
-#include <libkcal/calendar.h>
-#include <libkcal/calendarlocal.h>
-#include <libkcal/incidence.h>
-#include <libkcal/calendarresources.h>
+#include "libkcal/calendar.h"
+#include "libkcal/calendarlocal.h"
+#include "libkcal/calendarresources.h"
 #include <kstandarddirs.h>
-#include <ksimpleconfig.h>
 
+#include "pilotSerialDatabase.h"
+#include "pilotLocalDatabase.h"
+#include "pilotDateEntry.h"
 
-/*
-** KDE 2.2 uses class KORecurrence in a different header file.
-*/
-#ifdef KDE2
-#include <korecurrence.h>
-#define Recurrence_t KCal::KORecurrence
-#define DateList_t QDateList
-#define DateListIterator_t QDateListIterator
-#else
-#include <libkcal/recurrence.h>
-#define Recurrence_t KCal::Recurrence
-#define DateList_t KCal::DateList
-#define DateListIterator_t KCal::DateList::ConstIterator
-#endif
-
-#include <pilotSerialDatabase.h>
-#include <pilotLocalDatabase.h>
-#include <pilotDateEntry.h>
-
-#include "vcal-factorybase.h"
 #include "vcal-conduitbase.moc"
-
+#include "vcalconduitSettings.h"
 
 
 
@@ -164,18 +142,10 @@ there are two special cases: a full and a first sync.
 	FUNCTIONSETUP;
 	DEBUGCONDUIT<<vcalconduitbase_id<<endl;
 
-	if (!fConfig)
-	{
-		kdWarning() << k_funcinfo
-			<< ": No configuration set for vcal-conduit"
-			<< endl;
-		return false;
-	}
-
 	readConfig();
 
-	// don't do a first sync by default in any case, only when explicitly requested, or the backup
-	// database or the alendar are empty.
+	// don't do a first sync by default in any case, only when explicitly
+	// requested, or the backup database or the alendar are empty.
 	fFirstSync = false;
 
 	// TODO: Check Full sync and First sync
@@ -186,30 +156,31 @@ there are two special cases: a full and a first sync.
 
 #ifdef DEBUG
 	DEBUGCONDUIT<<fname<<": fullsync="<<isFullSync()<<", firstSync="<<isFirstSync()<<endl;
-	DEBUGCONDUIT<<fname<<": syncAction="<<fSyncDirection<<
-		", conflictResolution = "<<fConflictResolution<<", archive = "<<archive<<endl;
+	DEBUGCONDUIT<<fname<<": syncAction="<<getSyncDirection()<<
+		", conflictResolution = "<<getConflictResolution()<<", archive = "<<config()->syncArchived()<<endl;
 #endif
 
-	addSyncLogEntry(i18n("Syncing with file \"%1\"").arg(fCalendarFile));
 	pilotindex=0;
-	switch (fSyncDirection)
+	switch (getSyncDirection())
 	{
 	case SyncAction::eCopyPCToHH:
 		// TODO: Clear the palm and backup database??? Or just add the new items ignore
 		// the Palm->PC side and leave the existing items on the palm?
+		emit logMessage(i18n("Copying records to Pilot ..."));
 		QTimer::singleShot(0, this, SLOT(slotPCRecToPalm()));
 		break;
 	case SyncAction::eCopyHHToPC:
 		// TODO: Clear the backup database and the calendar, update fP
 		//       or just add the palm items and leave the PC ones there????
 	default:
+		emit logMessage(i18n("Copying records to PC ..."));
 		QTimer::singleShot(0, this, SLOT(slotPalmRecToPC()));
 	}
 	return true;
 
 error:
 
-	emit logError(i18n("Couldn't open the calendar databases."));
+	emit logError(i18n("Could not open the calendar databases."));
 
 	KPILOT_DELETE(fCalendar);
 	KPILOT_DELETE(fP);
@@ -220,14 +191,9 @@ error:
 
 /* virtual */ void VCalConduitBase::readConfig()
 {
-	fConfig->setGroup(configGroup());
-
-	fCalendarFile = fConfig->readPathEntry(VCalConduitFactoryBase::calendarFile);
-	SyncAction::eConflictResolution res=(SyncAction::eConflictResolution)fConfig->readNumEntry(
-		VCalConduitFactoryBase::conflictResolution, SyncAction::eUseGlobalSetting);
-	if (res!=SyncAction::eUseGlobalSetting) fConflictResolution=res;
-	archive = fConfig->readBoolEntry(VCalConduitFactoryBase::archive);
-	fCalendarType = (eCalendarTypeEnum)fConfig->readNumEntry(VCalConduitFactoryBase::calendarType, 0);
+	config()->readConfig();
+	SyncAction::ConflictResolution res=(SyncAction::ConflictResolution)(config()->conflictResolution());
+	setConflictResolution(res);
 }
 
 
@@ -241,83 +207,127 @@ error:
 	korgcfg.setGroup( "Time & Date" );
 	QString tz(korgcfg.readEntry( "TimeZoneId" ) );
 #ifdef DEBUG
-	DEBUGCONDUIT << fname<<": KOrganizer's time zone = "<<tz<<endl;
+	DEBUGCONDUIT << fname << ": KOrganizer's time zone = " << tz << endl;
 #endif
 
+	// Need a subclass ptr. for the ResourceCalendar methods
+	KCal::CalendarResources *rescal = 0L;
 
 #ifdef DEBUG
-	DEBUGCONDUIT << fname << ": Using calendar file " << fCalendarFile << endl;
-	DEBUGCONDUIT << "fCalendarType="<<fCalendarType<<endl;
-	DEBUGCONDUIT << "eCalendarLocal would be "<<eCalendarLocal<<
-			", eCalendarResources would be "<<eCalendarResource<<endl;
+	DEBUGCONDUIT << fname << ": Got calendar type " << config()->calendarType() << endl;
 #endif
 
-	switch(fCalendarType)
+	switch(config()->calendarType())
 	{
-		case eCalendarLocal:
+		case VCalConduitSettings::eCalendarLocal:
+		{
 #ifdef DEBUG
-			DEBUGCONDUIT<<"Using CalendarLocal!"<<endl;
+			DEBUGCONDUIT << fname
+				<< "Using CalendarLocal, file="
+				<< config()->calendarFile() << endl;
 #endif
-			if (fCalendarFile.isEmpty() )
+			if (config()->calendarFile().isEmpty() )
 			{
 #ifdef DEBUG
-				DEBUGCONDUIT<<"empty calendar file name, cannot open"<<endl;
+				DEBUGCONDUIT << fname
+					<< "Empty calendar file name."
+					<< endl;
 #endif
 				emit logError(i18n("You selected to sync with the a iCalendar file, "
 						"but did not give a filename. Please select a valid file name in "
 						"the conduit's configuration dialog"));
 				return false;
 			}
-//			fCalendar = new KCal::CalendarLocal(tz);
-			fCalendar = new KCal::CalendarLocal();
+
+			fCalendar = new KCal::CalendarLocal(tz);
 			if ( !fCalendar)
 			{
-				kdWarning() << k_funcinfo <<
-				    "Cannot initialize calendar object for file "<<fCalendarFile<<endl;
+				kdWarning() << k_funcinfo
+					<< "Cannot initialize calendar object for file "
+					<< config()->calendarFile() << endl;
 				return false;
 			}
 #ifdef DEBUG
-			DEBUGCONDUIT<<"Calendar's timezone: "<<fCalendar->timeZoneId()<<endl;
-			DEBUGCONDUIT<<"Calendar is local time: "<<fCalendar->isLocalTime()<<endl;
+			DEBUGCONDUIT << fname
+				<< "Calendar's timezone: "
+				<< fCalendar->timeZoneId() << endl;
+			DEBUGCONDUIT << fname
+				<< "Calendar is local time: "
+				<< fCalendar->isLocalTime() << endl;
 #endif
+			emit logMessage(i18n("Using %1 time zone: %2")
+				.arg(fCalendar->isLocalTime() ?
+					i18n("non-local") : i18n("local"))
+				.arg(tz));
+
+			KURL kurl(config()->calendarFile());
+			if(!KIO::NetAccess::download(config()->calendarFile(), fCalendarFile, 0L) &&
+				!kurl.isLocalFile())
+			{
+				emit logError(i18n("You chose to sync with the file \"%1\", which "
+							"cannot be opened. Please make sure to supply a "
+							"valid file name in the conduit's configuration dialog. "
+							"Aborting the conduit.").arg(config()->calendarFile()));
+				KIO::NetAccess::removeTempFile(fCalendarFile);
+				return false;
+			}
 
 			// if there is no calendar yet, use a first sync..
 			// the calendar is initialized, so nothing more to do...
 			if (!dynamic_cast<KCal::CalendarLocal*>(fCalendar)->load(fCalendarFile) )
 			{
 #ifdef DEBUG
-				DEBUGCONDUIT << "calendar file "<<fCalendarFile<<
-						" could not be opened. Will create a new one"<<endl;
+				DEBUGCONDUIT << fname
+					<< "Calendar file "
+					<< fCalendarFile
+					<< " could not be opened. "
+					   "Will create a new one"
+					<< endl;
 #endif
-				// Try to create empty file. if it fails, no valid file name was given.
+				// Try to create empty file. if it fails,
+				// no valid file name was given.
 				QFile fl(fCalendarFile);
 				if (!fl.open(IO_WriteOnly | IO_Append))
 				{
 #ifdef DEBUG
-					DEBUGCONDUIT<<"Invalid calendar file name "<<fCalendarFile<<endl;
+					DEBUGCONDUIT << fname
+						<< "Invalid calendar file name "
+						<< fCalendarFile << endl;
 #endif
 					emit logError(i18n("You chose to sync with the file \"%1\", which "
 							"cannot be opened or created. Please make sure to supply a "
 							"valid file name in the conduit's configuration dialog. "
-							"Aborting the conduit.").arg(fCalendarFile));
+							"Aborting the conduit.").arg(config()->calendarFile()));
 					return false;
 				}
 				fl.close();
 				fFirstSync=true;
 			}
+			addSyncLogEntry(i18n("Syncing with file \"%1\"").arg(config()->calendarFile()));
 			break;
+		}
 
-		case eCalendarResource:
+		case VCalConduitSettings::eCalendarResource:
 #ifdef DEBUG
-			DEBUGCONDUIT<<"Using CalendarResource!"<<endl;
+			DEBUGCONDUIT << "Using CalendarResource!" << endl;
 #endif
-			fCalendar = new KCal::CalendarResources(tz);
+			rescal = new KCal::CalendarResources( tz );
+			fCalendar = rescal;
 			if ( !fCalendar)
 			{
 				kdWarning() << k_funcinfo << "Cannot initialize calendar "<<
 					"object for ResourceCalendar"<<endl;
 				return false;
 			}
+#if LIBKCAL_IS_VERSION(1,1,0)
+			rescal->readConfig();
+			rescal->load();
+#endif
+			addSyncLogEntry(i18n("Syncing with standard calendar resource."));
+			emit logMessage(i18n("Using %1 time zone: %2")
+				.arg(fCalendar->isLocalTime() ?
+					i18n("non-local") : i18n("local"))
+				.arg(tz));
 			break;
 		default:
 			break;
@@ -331,12 +341,17 @@ error:
 		return false;
 	}
 	fP = newVCalPrivate(fCalendar);
-	if (!fP) return false;
+	if (!fP)
+	{
+		return false;
+	}
 	fP->updateIncidences();
 	if (fP->count()<1)
+	{
 		fFirstSync=true;
+	}
 
-	return (fCalendar && fP);
+	return true;
 }
 
 
@@ -359,13 +374,15 @@ void VCalConduitBase::slotPalmRecToPC()
 	if (!r)
 	{
 		fP->updateIncidences();
-		if (fSyncDirection==SyncAction::eCopyHHToPC)
+		if (getSyncDirection()==SyncAction::eCopyHHToPC)
 		{
+			emit logMessage(i18n("Cleaning up ..."));
 			QTimer::singleShot(0, this, SLOT(cleanup()));
 			return;
 		}
 		else
 		{
+			emit logMessage(i18n("Copying records to Pilot ..."));
 			QTimer::singleShot(0 ,this,SLOT(slotPCRecToPalm()));
 			return;
 		}
@@ -378,20 +395,20 @@ void VCalConduitBase::slotPalmRecToPC()
 //	DEBUGCONDUIT<<fname<<": Time: "<<e->dtStart()<<" until "<<e->dtEnd()<<endl;
 	bool archiveRecord=(r->isArchived());
 
-	s = fLocalDatabase->readRecordById(r->getID());
+	s = fLocalDatabase->readRecordById(r->id());
 	if (!s || isFirstSync())
 	{
 #ifdef DEBUG
-		if (r->getID()>0 && !s)
+		if (r->id()>0 && !s)
 		{
 			DEBUGCONDUIT<<"---------------------------------------------------------------------------"<<endl;
-			DEBUGCONDUIT<< fname<<": Could not read palm record with ID "<<r->getID()<<endl;
+			DEBUGCONDUIT<< fname<<": Could not read palm record with ID "<<r->id()<<endl;
 		}
 #endif
-		if (!r->isDeleted() || (archive && archiveRecord))
+		if (!r->isDeleted() || (config()->syncArchived() && archiveRecord))
 		{
 			KCal::Incidence*e=addRecord(r);
-			if (archive && archiveRecord)  {
+			if (config()->syncArchived() && archiveRecord)  {
 				e->setSyncStatus(KCal::Incidence::SYNCDEL);
 			}
 		}
@@ -400,7 +417,7 @@ void VCalConduitBase::slotPalmRecToPC()
 	{
 		if (r->isDeleted())
 		{
-			if (archive && archiveRecord)
+			if (config()->syncArchived() && archiveRecord)
 			{
 				changeRecord(r,s);
 			}
@@ -432,7 +449,7 @@ void VCalConduitBase::slotPCRecToPalm()
 	if (!e)
 	{
 		pilotindex=0;
-		if ( (fSyncDirection==SyncAction::eCopyHHToPC) || (fSyncDirection==SyncAction::eCopyPCToHH) )
+		if ( (getSyncDirection()==SyncAction::eCopyHHToPC) || (getSyncDirection()==SyncAction::eCopyPCToHH) )
 		{
 			QTimer::singleShot(0, this, SLOT(cleanup()));
 			return;
@@ -488,24 +505,24 @@ void VCalConduitBase::slotDeletedIncidence()
 		return;
 	}
 
-	KCal::Incidence *e = fP->findIncidence(r->getID());
+	KCal::Incidence *e = fP->findIncidence(r->id());
 	if (!e)
 	{
 #ifdef DEBUG
-		DEBUGCONDUIT<<"didn't find incidence with id="<<r->getID()<<", deleting it"<<endl;
+		DEBUGCONDUIT<<"didn't find incidence with id="<<r->id()<<", deleting it"<<endl;
 #endif
 		// entry was deleted from Calendar, so delete it from the palm
-//		PilotRecord*s=fLocalDatabase->readRecordById(r->getID());
+//		PilotRecord*s=fLocalDatabase->readRecordById(r->id());
 //		if (s)
 //		{
 //			// delete the record from the palm
-//			s->makeDeleted();
+//			s->setDeleted();
 ////			s->setAttrib(s->getAttrib() & ~dlpRecAttrDeleted & ~dlpRecAttrDirty);
 //			fDatabase->writeRecord(s);
 //			KPILOT_DELETE(s);
 //		}
 		deletePalmRecord(NULL, r);
-//		r->makeDeleted();
+//		r->setDeleted();
 ////		r->setAttrib(r->getAttrib() & ~dlpRecAttrDeleted & ~dlpRecAttrDirty);
 //		fLocalDatabase->writeRecord(r);
 //		fDatabase->writeRecord(r);
@@ -535,12 +552,26 @@ void VCalConduitBase::cleanup()
 	KPILOT_DELETE(fLocalDatabase);
 	if (fCalendar)
 	{
-		switch(fCalendarType)
+		KURL kurl(config()->calendarFile());
+		switch(config()->calendarType())
 		{
-			case eCalendarLocal:
-					dynamic_cast<KCal::CalendarLocal*>(fCalendar)->save(fCalendarFile);
+			case VCalConduitSettings::eCalendarLocal:
+				dynamic_cast<KCal::CalendarLocal*>(fCalendar)->save(fCalendarFile);
+				if(!kurl.isLocalFile())
+				{
+					if(!KIO::NetAccess::upload(fCalendarFile, config()->calendarFile(), 0L)) {
+						emit logError(i18n("An error occurred while uploading \"%1\". You can try to upload "
+							"the temporary local file \"%2\" manually.")
+							.arg(config()->calendarFile()).arg(fCalendarFile));
+					}
+					else {
+						KIO::NetAccess::removeTempFile(fCalendarFile);
+					}
+					QFile backup(fCalendarFile + "~");
+					backup.remove();
+				}
 				break;
-			case eCalendarResource:
+			case VCalConduitSettings::eCalendarResource:
 				fCalendar->save();
 				break;
 			default:
@@ -559,8 +590,6 @@ void VCalConduitBase::cleanup()
 void VCalConduitBase::postSync()
 {
 	FUNCTIONSETUP;
-	fConfig->setGroup(configGroup());
-	fConfig->writeEntry(VCalConduitFactoryBase::nextSyncAction, 0);
 }
 
 
@@ -570,7 +599,9 @@ KCal::Incidence* VCalConduitBase::addRecord(PilotRecord *r)
 
 	recordid_t id=fLocalDatabase->writeRecord(r);
 #ifdef DEBUG
-	DEBUGCONDUIT<<fname<<": Pilot Record ID="<<r->getID()<<", backup ID="<<id<<endl;
+	DEBUGCONDUIT<<fname<<": Pilot Record ID="<<r->id()<<", backup ID="<<id<<endl;
+#else
+	Q_UNUSED(id);
 #endif
 
 	PilotAppCategory *de=newPilotEntry(r);
@@ -598,7 +629,7 @@ KCal::Incidence* VCalConduitBase::addRecord(PilotRecord *r)
 
 // return how to resolve conflicts. for now PalmOverrides=0=false, PCOverrides=1=true, Ask=2-> ask the user using a messagebox
 int VCalConduitBase::resolveConflict(KCal::Incidence*e, PilotAppCategory*de) {
-	if (fConflictResolution==SyncAction::eAskUser)
+	if (getConflictResolution()==SyncAction::eAskUser)
 	{
 		// TODO: This is messed up!!!
 		return KMessageBox::warningYesNo(NULL,
@@ -607,7 +638,7 @@ int VCalConduitBase::resolveConflict(KCal::Incidence*e, PilotAppCategory*de) {
 			i18n("Conflicting Entries")
 		)==KMessageBox::No;
 	}
-	return fConflictResolution;
+	return getConflictResolution();
 }
 
 KCal::Incidence*VCalConduitBase::changeRecord(PilotRecord *r,PilotRecord *)
@@ -615,7 +646,7 @@ KCal::Incidence*VCalConduitBase::changeRecord(PilotRecord *r,PilotRecord *)
 	FUNCTIONSETUP;
 
 	PilotAppCategory*de=newPilotEntry(r);
-	KCal::Incidence *e = fP->findIncidence(r->getID());
+	KCal::Incidence *e = fP->findIncidence(r->id());
 
 	if (e && de)
 	{
@@ -649,7 +680,7 @@ KCal::Incidence*VCalConduitBase::deleteRecord(PilotRecord *r, PilotRecord *)
 {
 	FUNCTIONSETUP;
 
-	KCal::Incidence *e = fP->findIncidence(r->getID());
+	KCal::Incidence *e = fP->findIncidence(r->id());
 	if (e)
 	{
 		// RemoveEvent also takes it out of the calendar.
@@ -684,10 +715,9 @@ void VCalConduitBase::deletePalmRecord(KCal::Incidence*e, PilotRecord*s)
 	if (s)
 	{
 #ifdef DEBUG
-		DEBUGCONDUIT << fname << ": deleting record " << s->getID() << endl;
+		DEBUGCONDUIT << fname << ": deleting record " << s->id() << endl;
 #endif
-		s->makeDeleted();
-//		s->setAttrib(s->getAttrib() & ~dlpRecAttrDeleted);
+		s->setDeleted();
 		fDatabase->writeRecord(s);
 		fLocalDatabase->writeRecord(s);
 	}
@@ -697,6 +727,8 @@ void VCalConduitBase::deletePalmRecord(KCal::Incidence*e, PilotRecord*s)
 		DEBUGCONDUIT << fname << ": could not find record to delete (" << e->pilotId() << ")" << endl;
 #endif
 	}
+
+	Q_UNUSED(e);
 }
 
 
