@@ -30,10 +30,8 @@
 ** Bug reports and questions can be sent to kde-pim@kde.org
 */
 
-#include <qstringlist.h>
-#include <qlistview.h>
-
 #include <klistview.h>
+class QStringList;
 
 /**
  * This Widget extends KListView for a particular purpose:
@@ -56,7 +54,7 @@
  * and their HTML equivalents (although why you would want
  * to categorize colors is beyond me).
  *
- * @version $Id: listCat.h,v 1.11 2003/03/20 21:08:56 adridg Exp $
+ * @version $Id: listCat.h,v 1.12 2004/04/14 00:18:18 kainhofe Exp $
  */
 
 class ListCategorizer : public KListView
